@@ -69,9 +69,9 @@ InlinedUnlockedMkFunctor(AtomEntry *ae, unsigned int arity)
   p->NameOfFE = AbsAtom(ae);
   p->ArityOfFE = arity;
   p->PropsOfFE = NIL;
-  p->NextOfPE = ae->PropsOfAE;
   INIT_RWLOCK(p->FRWLock);
-  ae->PropsOfAE = AbsProp((PropEntry *) p);
+  /* respect the first property, in case this is a wide atom */
+  AddPropToAtom(ae, (PropEntry *)p);
   return ((Functor) p);
 }
 
@@ -104,8 +104,7 @@ Yap_MkFunctorWithAddress(Atom ap, unsigned int arity, FunctorEntry *p)
   p->KindOfPE = FunctorProperty;
   p->NameOfFE = ap;
   p->ArityOfFE = arity;
-  p->NextOfPE = RepAtom(ap)->PropsOfAE;
-  ae->PropsOfAE = AbsProp((PropEntry *) p);
+  AddPropToAtom(ae, (PropEntry *)p);
   WRITE_UNLOCK(ae->ARWLock);
 }
 
@@ -116,7 +115,7 @@ SearchInInvisible(char *atom)
 
   READ_LOCK(INVISIBLECHAIN.AERWLock);
   chain = RepAtom(INVISIBLECHAIN.Entry);
-  while (!EndOfPAEntr(chain) && strcmp(chain->StrOfAE, atom) != 0) {
+  while (!EndOfPAEntr(chain) && strcmp(chain->StrOfAE, atom)) {
     chain = RepAtom(chain->NextOfAE);
   }
   READ_UNLOCK(INVISIBLECHAIN.AERWLock);
@@ -211,6 +210,7 @@ LookupAtom(char *atom)
   return na;
 }
 
+
 static Atom
 LookupWideAtom(wchar_t *atom)
 {				/* lookup atom in atom table            */
@@ -249,19 +249,23 @@ LookupWideAtom(wchar_t *atom)
 #endif  
   /* add new atom to start of chain */
   sz = wcslen(atom);
-  ae = (AtomEntry *) Yap_AllocAtomSpace(sizeof(AtomEntry) + sizeof(wchar_t)*(sz + 1)+sizeof(WideAtomEntry));
+  ae = (AtomEntry *) Yap_AllocAtomSpace((size_t)(((AtomEntry *)NULL)+1) + sizeof(wchar_t)*(sz + 1));
   if (ae == NULL) {
     WRITE_UNLOCK(WideHashChain[hash].AERWLock);
     return NIL;
   }
-  wae = (WideAtomEntry *)(ae->StrOfAE+sizeof(wchar_t)*(sz + 1));
+  wae = (WideAtomEntry *) Yap_AllocAtomSpace(sizeof(WideAtomEntry));
+  if (wae == NULL) {
+    WRITE_UNLOCK(WideHashChain[hash].AERWLock);
+    return NIL;
+  }
   na = AbsAtom(ae);
   ae->PropsOfAE = AbsWideAtomProp(wae);
   wae->NextOfPE = NIL;
   wae->KindOfPE = WideAtomProperty;
   wae->SizeOfAtom = sz;
-  if (ae->StrOfAE != (char *)atom)
-    wcscpy((wchar_t *)(ae->StrOfAE), atom);
+  if (ae->WStrOfAE != atom)
+    wcscpy(ae->WStrOfAE, atom);
   NOfAtoms++;
   ae->NextOfAE = a;
   WideHashChain[hash].Entry = na;
@@ -489,45 +493,98 @@ Yap_HasOp(Atom a)
 }
 
 OpEntry *
-Yap_GetOpProp(Atom a, op_type type)
+Yap_OpPropForModule(Atom a, Term mod)
 {				/* look property list of atom a for kind  */
   AtomEntry *ae = RepAtom(a);
   PropEntry *pp;
   OpEntry *info = NULL;
 
+  if (mod == TermProlog)
+    mod = PROLOG_MODULE;
+  WRITE_LOCK(ae->ARWLock);
+  pp = RepProp(ae->PropsOfAE);
+  while (!EndOfPAEntr(pp)) {
+    if ( pp->KindOfPE == OpProperty) {
+      info = (OpEntry *)pp;
+      if (info->OpModule == mod) {
+	WRITE_LOCK(info->OpRWLock);
+	WRITE_UNLOCK(ae->ARWLock);
+	return info;
+      }
+    }
+    pp = pp->NextOfPE;
+  }
+  info = (OpEntry *) Yap_AllocAtomSpace(sizeof(OpEntry));
+  info->KindOfPE = Ord(OpProperty);
+  info->OpModule = mod;
+  info->OpName = a;
+  LOCK(OpListLock);
+  info->OpNext = OpList;
+  OpList = info;
+  UNLOCK(OpListLock);
+  AddPropToAtom(ae, (PropEntry *)info);
+  INIT_RWLOCK(info->OpRWLock);
+  WRITE_LOCK(info->OpRWLock);
+  WRITE_UNLOCK(ae->ARWLock);
+  info->Prefix = info->Infix = info->Posfix = 0;
+  return info;
+}
+
+OpEntry *
+Yap_GetOpProp(Atom a, op_type type USES_REGS)
+{				/* look property list of atom a for kind  */
+  AtomEntry *ae = RepAtom(a);
+  PropEntry *pp;
+  OpEntry *oinfo = NULL;
+
   READ_LOCK(ae->ARWLock);
   pp = RepProp(ae->PropsOfAE);
-  while (!EndOfPAEntr(pp) &&
-	 ( pp->KindOfPE != OpProperty ||
-	    ((OpEntry *)pp)->OpModule != CurrentModule))
-    pp = RepProp(pp->NextOfPE);
-  if ((info = (OpEntry *)pp)) {
-    if ((type == INFIX_OP && !info->Infix) ||
-        (type == POSFIX_OP && !info->Posfix) ||
-	(type == PREFIX_OP && !info->Prefix))
-      pp =  RepProp(NIL);
-  }
-  if (EndOfPAEntr(pp)) {
-    pp = RepProp(ae->PropsOfAE);
-    while (!EndOfPAEntr(pp) &&
-	   ( pp->KindOfPE != OpProperty ||
-	     ((OpEntry *)pp)->OpModule != PROLOG_MODULE))
+  while (!EndOfPAEntr(pp)) {
+    OpEntry *info = NULL;
+    if ( pp->KindOfPE != OpProperty) {
       pp = RepProp(pp->NextOfPE);
-    if ((info = (OpEntry *)pp)) {
-      if ((type == INFIX_OP && !info->Infix) ||
-	  (type == POSFIX_OP && !info->Posfix) ||
-	  (type == PREFIX_OP && !info->Prefix))
-	pp =  RepProp(NIL);
+      continue;
+    }
+    info = (OpEntry *)pp;
+    if (info->OpModule != CurrentModule &&
+	info->OpModule != PROLOG_MODULE) {
+      pp = RepProp(pp->NextOfPE);
+      continue;
+    }
+    if (type == INFIX_OP) {
+      if (!info->Infix) {
+	pp = RepProp(pp->NextOfPE);
+	continue;
+      }
+    } else if (type == POSFIX_OP) {
+      if (!info->Posfix) {
+	pp = RepProp(pp->NextOfPE);
+	continue;
+      }
+    } else {
+      if (!info->Prefix) {
+	pp = RepProp(pp->NextOfPE);
+	continue;
+      }
+    }
+    /* if it is not the latest module */
+    if (info->OpModule == PROLOG_MODULE) {
+      /* cannot commit now */
+      oinfo = info;
+      pp = RepProp(pp->NextOfPE);
+    } else {
+      READ_LOCK(info->OpRWLock);
+      READ_UNLOCK(ae->ARWLock);
+      return info;
     }
   }
-  if (!info) {
+  if (oinfo) {
+    READ_LOCK(oinfo->OpRWLock);
     READ_UNLOCK(ae->ARWLock);
-    return NULL;
-  } else {
-    READ_LOCK(info->OpRWLock);
-    READ_UNLOCK(ae->ARWLock);
-    return info;
+    return oinfo;
   }
+  READ_UNLOCK(ae->ARWLock);
+  return NULL;
 }
 
 
@@ -546,7 +603,7 @@ GetPredPropByAtomHavingLock(AtomEntry* ae, Term cur_mod)
 #if THREADS
       /* Thread Local Predicates */
       if (pe->PredFlags & ThreadLocalPredFlag) {
-	return AbsPredProp(Yap_GetThreadPred(pe));
+	return AbsPredProp(Yap_GetThreadPred(pe INIT_REGS));
       }
 #endif
     }
@@ -582,7 +639,7 @@ GetPredPropByAtomHavingLockInThisModule(AtomEntry* ae, Term cur_mod)
 #if THREADS
       /* Thread Local Predicates */
       if (pe->PredFlags & ThreadLocalPredFlag) {
-	return AbsPredProp(Yap_GetThreadPred(pe));
+	return AbsPredProp(Yap_GetThreadPred(pe INIT_REGS));
       }
 #endif
       return(p0);
@@ -794,21 +851,19 @@ Yap_NewPredPropByFunctor(FunctorEntry *fe, Term cur_mod)
   }
   p->FunctorOfPred = fe;
   WRITE_UNLOCK(fe->FRWLock);
-#ifdef LOW_PROF
-  if (ProfilerOn &&
-      Yap_OffLineProfiler) {
-    Yap_inform_profiler_of_clause((yamop *)&(p->OpcodeOfPred), (yamop *)(&(p->OpcodeOfPred)+1), p, 1);
+  {
+    CACHE_REGS
+    Yap_inform_profiler_of_clause(&(p->OpcodeOfPred), &(p->OpcodeOfPred)+1, p, GPROF_NEW_PRED_FUNC);
     if (!(p->PredFlags & (CPredFlag|AsmPredFlag))) {
-      Yap_inform_profiler_of_clause((yamop *)&(p->cs.p_code.ExpandCode), (yamop *)(&(p->cs.p_code.ExpandCode)+1), p, 1);
+      Yap_inform_profiler_of_clause(&(p->cs.p_code.ExpandCode), &(p->cs.p_code.ExpandCode)+1, p, GPROF_NEW_PRED_FUNC);
     }
   }
-#endif /* LOW_PROF */
   return AbsPredProp(p);
 }
 
 #if THREADS
 Prop
-Yap_NewThreadPred(PredEntry *ap)
+Yap_NewThreadPred(PredEntry *ap USES_REGS)
 {
   PredEntry *p = (PredEntry *) Yap_AllocAtomSpace(sizeof(*p));
 
@@ -840,18 +895,13 @@ Yap_NewThreadPred(PredEntry *ap)
   p->beamTable = NULL;
 #endif 
   /* careful that they don't cross MkFunctor */
-  p->NextOfPE = AbsPredProp(MY_ThreadHandle.local_preds);
-  MY_ThreadHandle.local_preds = p;
+  p->NextOfPE = AbsPredProp(LOCAL_ThreadHandle.local_preds);
+  LOCAL_ThreadHandle.local_preds = p;
   p->FunctorOfPred = ap->FunctorOfPred;
-#ifdef LOW_PROF
-    if (ProfilerOn &&
-	Yap_OffLineProfiler) {
-      Yap_inform_profiler_of_clause((yamop *)&(p->OpcodeOfPred), (yamop *)(&(p->OpcodeOfPred)+1), p, 1);
-      if (!(p->PredFlags & (CPredFlag|AsmPredFlag))) {
-	Yap_inform_profiler_of_clause((yamop *)&(p->cs.p_code.ExpandCode), (yamop *)(&(p->cs.p_code.ExpandCode)+1), p, 1);
-      }
-    }
-#endif /* LOW_PROF */
+  Yap_inform_profiler_of_clause(&(p->OpcodeOfPred), &(p->OpcodeOfPred)+1, p, GPROF_NEW_PRED_THREAD);
+  if (!(p->PredFlags & (CPredFlag|AsmPredFlag))) {
+    Yap_inform_profiler_of_clause(&(p->cs.p_code.ExpandCode), &(p->cs.p_code.ExpandCode)+1, p, GPROF_NEW_PRED_THREAD);
+  }
   return AbsPredProp(p);
 }
 #endif
@@ -896,7 +946,6 @@ Yap_NewPredPropByAtom(AtomEntry *ae, Term cur_mod)
   p->beamTable = NULL;
 #endif 
   /* careful that they don't cross MkFunctor */
-  p->NextOfPE = ae->PropsOfAE;
   if (PRED_GOAL_EXPANSION_FUNC) {
     Prop p1 = ae->PropsOfAE;
 
@@ -912,18 +961,17 @@ Yap_NewPredPropByAtom(AtomEntry *ae, Term cur_mod)
       p1 = pe->NextOfPE;
     }
   }
-  ae->PropsOfAE = p0 = AbsPredProp(p);
+  AddPropToAtom(ae, (PropEntry *)p);
+  p0 = AbsPredProp(p);
   p->FunctorOfPred = (Functor)AbsAtom(ae);
   WRITE_UNLOCK(ae->ARWLock);
-#ifdef LOW_PROF
-    if (ProfilerOn &&
-	Yap_OffLineProfiler) {
-      Yap_inform_profiler_of_clause((yamop *)&(p->OpcodeOfPred), (yamop *)(&(p->OpcodeOfPred)+1), p, 1);
-      if (!(p->PredFlags & (CPredFlag|AsmPredFlag))) {
-	Yap_inform_profiler_of_clause((yamop *)&(p->cs.p_code.ExpandCode), (yamop *)(&(p->cs.p_code.ExpandCode)+1), p, 1);
-      }
+  {
+    CACHE_REGS
+    Yap_inform_profiler_of_clause(&(p->OpcodeOfPred), &(p->OpcodeOfPred)+1, p, GPROF_NEW_PRED_ATOM);
+    if (!(p->PredFlags & (CPredFlag|AsmPredFlag))) {
+      Yap_inform_profiler_of_clause(&(p->cs.p_code.ExpandCode), &(p->cs.p_code.ExpandCode)+1, p, GPROF_NEW_PRED_ATOM);
     }
-#endif /* LOW_PROF */
+  }
   return p0;
 }
 
@@ -938,8 +986,14 @@ Yap_PredPropByFunctorNonThreadLocal(Functor f, Term cur_mod)
     return Yap_NewPredPropByFunctor(f,cur_mod);
 
   if ((p->ModuleOfPred == cur_mod || !(p->ModuleOfPred))) {
-    WRITE_UNLOCK(f->FRWLock);
-    return AbsPredProp(p);
+    /* don't match multi-files */
+    if (!(p->PredFlags & MultiFileFlag) ||
+	p->ModuleOfPred ||
+	!cur_mod ||
+	cur_mod == TermProlog) {
+      WRITE_UNLOCK(f->FRWLock);
+      return AbsPredProp(p);
+    }
   }
   if (p->NextOfPE) {
     UInt hash = PRED_HASH(f,cur_mod,PredHashTableSize);
@@ -974,8 +1028,14 @@ Yap_PredPropByAtomNonThreadLocal(Atom at, Term cur_mod)
     PredEntry *pe = RepPredProp(p0);
     if ( pe->KindOfPE == PEProp && 
 	 (pe->ModuleOfPred == cur_mod || !pe->ModuleOfPred)) {
-      WRITE_UNLOCK(ae->ARWLock);
-      return(p0);
+      /* don't match multi-files */
+      if (!(pe->PredFlags & MultiFileFlag) ||
+	  pe->ModuleOfPred ||
+	  !cur_mod ||
+	  cur_mod == TermProlog) {
+	WRITE_UNLOCK(ae->ARWLock);
+	return(p0);
+      }
     }
     p0 = pe->NextOfPE;
   }
@@ -1031,10 +1091,9 @@ Yap_PutValue(Atom a, Term v)
       WRITE_UNLOCK(ae->ARWLock);
       return;
     }
-    p->NextOfPE = RepAtom(a)->PropsOfAE;
-    RepAtom(a)->PropsOfAE = AbsValProp(p);
     p->KindOfPE = ValProperty;
     p->ValueOfVE = TermNil;
+    AddPropToAtom(RepAtom(a), (PropEntry *)p);
     /* take care that the lock for the property will be inited even
        if someone else searches for the property */
     INIT_RWLOCK(p->VRWLock);
@@ -1121,6 +1180,7 @@ Yap_PutValue(Atom a, Term v)
 Term
 Yap_StringToList(char *s)
 {
+  CACHE_REGS
   register Term t;
   register unsigned char *cp = (unsigned char *)s + strlen(s);
 
@@ -1134,6 +1194,7 @@ Yap_StringToList(char *s)
 Term
 Yap_NStringToList(char *s, size_t len)
 {
+  CACHE_REGS
   Term t;
   unsigned char *cp = (unsigned char *)s + len;
 
@@ -1144,14 +1205,18 @@ Yap_NStringToList(char *s, size_t len)
   return t;
 }
 
+
 Term
 Yap_WideStringToList(wchar_t *s)
 {
+  CACHE_REGS
   Term t;
   wchar_t *cp = s + wcslen(s);
 
   t = MkAtomTerm(AtomNil);
   while (cp > s) {
+    if (ASP < H+1024)
+      return (CELL)0;    
     t = MkPairTerm(MkIntegerTerm(*--cp), t);
   }
   return t;
@@ -1160,23 +1225,28 @@ Yap_WideStringToList(wchar_t *s)
 Term
 Yap_NWideStringToList(wchar_t *s, size_t len)
 {
+  CACHE_REGS
   Term t;
   wchar_t *cp = s + len;
 
   t = MkAtomTerm(AtomNil);
   while (cp > s) {
+    if (ASP < H+1024)
+      return (CELL)0;    
     t = MkPairTerm(MkIntegerTerm(*--cp), t);
   }
   return t;
 }
 
 Term
-Yap_StringToDiffList(char *s, Term t)
+Yap_StringToDiffList(char *s, Term t USES_REGS)
 {
   register unsigned char *cp = (unsigned char *)s + strlen(s);
 
  t = Yap_Globalise(t);
   while (cp > (unsigned char *)s) {
+    if (ASP < H+1024)
+      return (CELL)0;
     t = MkPairTerm(MkIntTerm(*--cp), t);
   }
   return t;
@@ -1185,6 +1255,7 @@ Yap_StringToDiffList(char *s, Term t)
 Term
 Yap_NStringToDiffList(char *s, Term t, size_t len)
 {
+  CACHE_REGS
   register unsigned char *cp = (unsigned char *)s + len;
 
   t = Yap_Globalise(t);
@@ -1197,6 +1268,7 @@ Yap_NStringToDiffList(char *s, Term t, size_t len)
 Term
 Yap_WideStringToDiffList(wchar_t *s, Term t)
 {
+  CACHE_REGS
  wchar_t *cp = s + wcslen(s);
 
   t = Yap_Globalise(t);
@@ -1209,6 +1281,7 @@ Yap_WideStringToDiffList(wchar_t *s, Term t)
 Term
 Yap_NWideStringToDiffList(wchar_t *s, Term t, size_t len)
 {
+  CACHE_REGS
  wchar_t *cp = s + len;
 
  t = Yap_Globalise(t);
@@ -1221,6 +1294,7 @@ Yap_NWideStringToDiffList(wchar_t *s, Term t, size_t len)
 Term
 Yap_StringToListOfAtoms(char *s)
 {
+  CACHE_REGS
   register Term t;
   char so[2];
   register unsigned char *cp = (unsigned char *)s + strlen(s);
@@ -1237,6 +1311,7 @@ Yap_StringToListOfAtoms(char *s)
 Term
 Yap_NStringToListOfAtoms(char *s, size_t len)
 {
+  CACHE_REGS
   register Term t;
   char so[2];
   register unsigned char *cp = (unsigned char *)s + len;
@@ -1253,6 +1328,7 @@ Yap_NStringToListOfAtoms(char *s, size_t len)
 Term
 Yap_WideStringToListOfAtoms(wchar_t *s)
 {
+  CACHE_REGS
   register Term t;
   wchar_t so[2];
   wchar_t *cp = s + wcslen(s);
@@ -1261,6 +1337,8 @@ Yap_WideStringToListOfAtoms(wchar_t *s)
   t = MkAtomTerm(AtomNil);
   while (cp > s) {
     so[0] = *--cp;
+    if (ASP < H+1024)
+      return (CELL)0;    
     t = MkPairTerm(MkAtomTerm(LookupWideAtom(so)), t);
   }
   return t;
@@ -1269,6 +1347,7 @@ Yap_WideStringToListOfAtoms(wchar_t *s)
 Term
 Yap_NWideStringToListOfAtoms(wchar_t *s, size_t len)
 {
+  CACHE_REGS
   register Term t;
   wchar_t so[2];
   wchar_t *cp = s + len;
@@ -1276,6 +1355,8 @@ Yap_NWideStringToListOfAtoms(wchar_t *s, size_t len)
   so[1] = '\0';
   t = MkAtomTerm(AtomNil);
   while (cp > s) {
+    if (ASP < H+1024)
+      return (CELL)0;    
     so[0] = *--cp;
     t = MkPairTerm(MkAtomTerm(LookupWideAtom(so)), t);
   }
@@ -1285,6 +1366,7 @@ Yap_NWideStringToListOfAtoms(wchar_t *s, size_t len)
 Term
 Yap_NWideStringToDiffListOfAtoms(wchar_t *s, Term t0, size_t len)
 {
+  CACHE_REGS
   register Term t;
   wchar_t so[2];
   wchar_t *cp = s + len;
@@ -1301,13 +1383,14 @@ Yap_NWideStringToDiffListOfAtoms(wchar_t *s, Term t0, size_t len)
 Term
 Yap_ArrayToList(register Term *tp, int nof)
 {
+  CACHE_REGS
   register Term *pt = tp + nof;
   register Term t;
 
   t = MkAtomTerm(AtomNil);
   while (pt > tp) {
     Term tm = *--pt;
-#if SBA
+#if YAPOR_SBA
     if (tm == 0)
       t = MkPairTerm((CELL)pt, t);
     else
@@ -1377,7 +1460,7 @@ ArgsOfSFTerm(Term t)
 #endif
 
 Int
-Yap_NewSlots(int n)
+Yap_NewSlots(int n USES_REGS)
 {
   Int old_slots = IntOfTerm(ASP[0]), oldn = n;
   while (n > 0) {
@@ -1391,7 +1474,7 @@ Yap_NewSlots(int n)
 }
 
 Int
-Yap_InitSlot(Term t)
+Yap_InitSlot(Term t USES_REGS)
 {
   Int old_slots = IntOfTerm(ASP[0]);
   *ASP = t;
@@ -1402,7 +1485,7 @@ Yap_InitSlot(Term t)
 }
 
 int
-Yap_RecoverSlots(int n)
+Yap_RecoverSlots(int n USES_REGS)
 {
   Int old_slots = IntOfTerm(ASP[0]);
   if (old_slots - n < 0) {
@@ -1414,30 +1497,6 @@ Yap_RecoverSlots(int n)
   return TRUE;
 }
 
-Term
-Yap_GetFromSlot(Int slot)
-{
-  return(Deref(LCL0[slot]));
-}
-
-Term
-Yap_GetPtrFromSlot(Int slot)
-{
-  return(LCL0[slot]);
-}
-
-Term *
-Yap_AddressFromSlot(Int slot)
-{
-  return(LCL0+slot);
-}
-
-void
-Yap_PutInSlot(Int slot, Term t)
-{
-  LCL0[slot] = t;
-}
-
 static HoldEntry *
 InitAtomHold(void)
 {
