@@ -689,7 +689,7 @@ eval1(Int fi, Term t) {
       return Yap_ArithError(TYPE_ERROR_INTEGER, t, "random(%f)", FloatOfTerm(t));
     case big_int_e:
 #ifdef USE_GMP
-      return Yap_ArithError(TYPE_ERROR_INTEGER, t, "random(%f)", FloatOfTerm(t));
+      return Yap_gmp_mul_float_big(Yap_random(), t);
 #endif
     default:
       RERROR();
@@ -745,7 +745,7 @@ static InitUnEntry InitUnTab[] = {
 };
 
 static Int 
-p_unary_is(void)
+p_unary_is( USES_REGS1 )
 {				/* X is Y	 */
   Term t = Deref(ARG2);
   Term top;
@@ -790,7 +790,7 @@ p_unary_is(void)
 }
 
 static Int 
-p_unary_op_as_integer(void)
+p_unary_op_as_integer( USES_REGS1 )
 {				/* X is Y	 */
   Term t = Deref(ARG1);
 
@@ -835,8 +835,7 @@ Yap_InitUnaryExps(void)
     p->ArityOfEE = 1;
     p->ENoOfEE = 1;
     p->FOfEE = InitUnTab[i].f;
-    p->NextOfPE = ae->PropsOfAE;
-    ae->PropsOfAE = AbsExpProp(p);
+    AddPropToAtom(ae, (PropEntry *)p);
     WRITE_UNLOCK(ae->ARWLock);
   }
   Yap_InitCPred("is", 3, p_unary_is, TestPredFlag | SafePredFlag);
