@@ -48,7 +48,8 @@ p_mod(Term t1, Term t2) {
 	Int i2 = IntegerOfTerm(t2);
 	Int mod;
 
-	if (i2 == 0) goto zero_divisor;
+	if (i2 == 0)
+	  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is " Int_FORMAT " mod 0", i1);
 	if (i1 == Int_MIN && i2 == -1) {
 #ifdef USE_GMP
 	  return Yap_gmp_add_ints(Int_MAX, 1);	  
@@ -82,7 +83,8 @@ p_mod(Term t1, Term t2) {
       {
 	Int i2 = IntegerOfTerm(t2);
 
-	if (i2 == 0) goto zero_divisor;
+	if (i2 == 0)
+	  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is ... mod 0");
 	return Yap_gmp_mod_big_int(t1, i2);
       }
     case (CELL)big_int_e:
@@ -97,8 +99,6 @@ p_mod(Term t1, Term t2) {
   default:
     RERROR();
   }
-zero_divisor:
-  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is mod 0");
 }
 
 static Term
@@ -113,7 +113,8 @@ p_div2(Term t1, Term t2) {
 	Int i2 = IntegerOfTerm(t2);
 	Int res, mod;
 
-	if (i2 == 0) goto zero_divisor;
+	if (i2 == 0)
+	  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is " Int_FORMAT " div 0", i1);
 	if (i1 == Int_MIN && i2 == -1) {
 #ifdef USE_GMP
 	  return Yap_gmp_add_ints(Int_MAX, 1);	  
@@ -148,7 +149,8 @@ p_div2(Term t1, Term t2) {
       {
 	Int i2 = IntegerOfTerm(t2);
 
-	if (i2 == 0) goto zero_divisor;
+	if (i2 == 0)
+	  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is ... div 0");
 	return Yap_gmp_div2_big_int(t1, i2);
       }
     case (CELL)big_int_e:
@@ -163,8 +165,6 @@ p_div2(Term t1, Term t2) {
   default:
     RERROR();
   }
-zero_divisor:
-  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is div 0");
 }
 
 static Term
@@ -177,9 +177,9 @@ p_rem(Term t1, Term t2) {
       {
 	Int i1 = IntegerOfTerm(t1);
 	Int i2 = IntegerOfTerm(t2);
-	Int mod;
 
-	if (i2 == 0) goto zero_divisor;
+	if (i2 == 0)
+	  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is " Int_FORMAT " rem 0", i1);
 	if (i1 == Int_MIN && i2 == -1) {
 #ifdef USE_GMP
 	  return Yap_gmp_add_ints(Int_MAX, 1);	  
@@ -188,7 +188,6 @@ p_rem(Term t1, Term t2) {
 		    "rem/2 with %d and %d", i1, i2);
 #endif
 	}
-	mod = i1%i2;
 	RINT(i1%i2);
       }
     case (CELL)double_e:
@@ -207,6 +206,8 @@ p_rem(Term t1, Term t2) {
 #ifdef USE_GMP
     switch (ETypeOfTerm(t2)) {
     case long_int_e:
+      if (IntegerOfTerm(t2) == 0)
+	return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is ... rem 0");
       return Yap_gmp_rem_big_int(t1, IntegerOfTerm(t2));
     case (CELL)big_int_e:
       /* two bignums */
@@ -220,8 +221,6 @@ p_rem(Term t1, Term t2) {
   default:
     RERROR();
   }
- zero_divisor:
-  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is mod 0");
 }
 
 
@@ -239,7 +238,8 @@ p_rdiv(Term t1, Term t2) {
 	Int i1 = IntegerOfTerm(t1);
 	Int i2 = IntegerOfTerm(t2);
 
-	if (i2 == 0) goto zero_divisor;
+	if (i2 == 0)
+	  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is " Int_FORMAT " rdiv 0", i1);
 	return Yap_gmq_rdiv_int_int(i1, i2);
       }
     case (CELL)big_int_e:
@@ -252,6 +252,8 @@ p_rdiv(Term t1, Term t2) {
   case (CELL)big_int_e:
     switch (ETypeOfTerm(t2)) {
     case long_int_e:
+      if (IntegerOfTerm(t2) == 0)
+	return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is ... rdiv  0");
       /* I know the term is much larger, so: */
       return Yap_gmq_rdiv_big_int(t1, IntegerOfTerm(t2));
     case (CELL)big_int_e:
@@ -264,8 +266,6 @@ p_rdiv(Term t1, Term t2) {
   default:
     RERROR();
   }
- zero_divisor:
-  return Yap_ArithError(EVALUATION_ERROR_ZERO_DIVISOR, t2, "X is mod 0");
 #else
   RERROR();
 #endif
@@ -1050,7 +1050,7 @@ static InitBinEntry InitBinTab[] = {
 };
 
 static Int 
-p_binary_is(void)
+p_binary_is( USES_REGS1 )
 {				/* X is Y	 */
   Term t = Deref(ARG2);
   Term t1, t2;
@@ -1099,7 +1099,7 @@ p_binary_is(void)
 }
 
 static Int 
-do_arith23(arith2_op op)
+do_arith23(arith2_op op USES_REGS)
 {				/* X is Y	 */
   Term t = Deref(ARG1);
   Int out;
@@ -1121,55 +1121,55 @@ do_arith23(arith2_op op)
 }
 
 static Int 
-export_p_plus(void)
+export_p_plus( USES_REGS1 )
 {				/* X is Y	 */
-  return do_arith23(op_plus);
+  return do_arith23(op_plus PASS_REGS);
 }
 
 static Int 
-export_p_minus(void)
+export_p_minus( USES_REGS1 )
 {				/* X is Y	 */
-  return do_arith23(op_minus);
+  return do_arith23(op_minus PASS_REGS);
 }
 
 static Int 
-export_p_times(void)
+export_p_times( USES_REGS1 )
 {				/* X is Y	 */
-  return do_arith23(op_times);
+  return do_arith23(op_times PASS_REGS);
 }
 
 static Int 
-export_p_div(void)
+export_p_div( USES_REGS1 )
 {				/* X is Y	 */
-  return do_arith23(op_div);
+  return do_arith23(op_div PASS_REGS);
 }
 
 static Int 
-export_p_and(void)
+export_p_and( USES_REGS1 )
 {				/* X is Y	 */
-  return do_arith23(op_and);
+  return do_arith23(op_and PASS_REGS);
 }
 
 static Int 
-export_p_or(void)
+export_p_or( USES_REGS1 )
 {				/* X is Y	 */
-  return do_arith23(op_or);
+  return do_arith23(op_or PASS_REGS);
 }
 
 static Int 
-export_p_slr(void)
+export_p_slr( USES_REGS1 )
 {				/* X is Y	 */
-  return do_arith23(op_slr);
+  return do_arith23(op_slr PASS_REGS);
 }
 
 static Int 
-export_p_sll(void)
+export_p_sll( USES_REGS1 )
 {				/* X is Y	 */
-  return do_arith23(op_sll);
+  return do_arith23(op_sll PASS_REGS);
 }
 
 static Int 
-p_binary_op_as_integer(void)
+p_binary_op_as_integer( USES_REGS1 )
 {				/* X is Y	 */
   Term t = Deref(ARG1);
 
@@ -1215,8 +1215,7 @@ Yap_InitBinaryExps(void)
     p->ArityOfEE = 2;
     p->ENoOfEE = 2;
     p->FOfEE = InitBinTab[i].f;
-    p->NextOfPE = ae->PropsOfAE;
-    ae->PropsOfAE = AbsExpProp(p);
+    AddPropToAtom(ae, (PropEntry *)p);
     WRITE_UNLOCK(ae->ARWLock);
   }
   Yap_InitCPred("is", 4, p_binary_is, TestPredFlag | SafePredFlag);
