@@ -33,7 +33,7 @@ static char SccsId[]="%W% %G%";
 #define TermVoidAtt TermFoundVar
 
 static CELL *
-AddToQueue(attvar_record *attv)
+AddToQueue(attvar_record *attv USES_REGS)
 {
   Term t[2];
   Term WGs, ng;
@@ -41,10 +41,10 @@ AddToQueue(attvar_record *attv)
   t[0] = (CELL)&(attv->Done);
   t[1] = attv->Value;
   /* follow the chain */
-  WGs = Yap_ReadTimedVar(WokenGoals);
+  WGs = Yap_ReadTimedVar(LOCAL_WokenGoals);
   ng = Yap_MkApplTerm(FunctorAttGoal, 2, t);
 
-  Yap_UpdateTimedVar(WokenGoals, MkPairTerm(ng, WGs));
+  Yap_UpdateTimedVar(LOCAL_WokenGoals, MkPairTerm(ng, WGs));
   if ((Term)WGs == TermNil) {
     /* from now on, we have to start waking up goals */
     Yap_signal(YAP_WAKEUP_SIGNAL);
@@ -53,14 +53,14 @@ AddToQueue(attvar_record *attv)
 }
 
 static void
-AddFailToQueue(void)
+AddFailToQueue( USES_REGS1 )
 {
   Term WGs;
 
   /* follow the chain */
-  WGs = Yap_ReadTimedVar(WokenGoals);
+  WGs = Yap_ReadTimedVar(LOCAL_WokenGoals);
 
-  Yap_UpdateTimedVar(WokenGoals, MkPairTerm(MkAtomTerm(AtomFail),WGs));
+  Yap_UpdateTimedVar(LOCAL_WokenGoals, MkPairTerm(MkAtomTerm(AtomFail), WGs));
   if ((Term)WGs == TermNil) {
     /* from now on, we have to start waking up goals */
     Yap_signal(YAP_WAKEUP_SIGNAL);
@@ -68,13 +68,13 @@ AddFailToQueue(void)
 }
 
 static attvar_record *
-BuildNewAttVar(void)
+BuildNewAttVar( USES_REGS1 )
 {
   attvar_record *newv;
 
   /* add a new attributed variable */
-  if (!(newv = (attvar_record *)Yap_GetFromArena(&GlobalArena, sizeof(attvar_record)/sizeof(CELL),2)))
-    return NULL;
+  newv = (attvar_record *)H;
+  H = (CELL *)(newv+1);
   newv->AttFunc = FunctorAttVar;
   RESET_VARIABLE(&(newv->Value));
   RESET_VARIABLE(&(newv->Done));
@@ -83,20 +83,21 @@ BuildNewAttVar(void)
 }
 
 static int
-CopyAttVar(CELL *orig, struct cp_frame **to_visit_ptr, CELL *res)
+CopyAttVar(CELL *orig, struct cp_frame **to_visit_ptr, CELL *res USES_REGS)
 {
   register attvar_record *attv = RepAttVar(orig);
   register attvar_record *newv;
   struct cp_frame *to_visit = *to_visit_ptr;
   CELL *vt;
 
-  if (!(newv = BuildNewAttVar()))
+  
+  if (!(newv = BuildNewAttVar( PASS_REGS1 )))
     return FALSE;
   vt = &(attv->Atts);
   to_visit->start_cp = vt-1;
   to_visit->end_cp = vt;
   if (IsVarTerm(attv->Atts)) {
-    Bind(&newv->Atts, (CELL)H);
+    Bind_Global_NonAtt(&newv->Atts, (CELL)H);
     to_visit->to = H;
     H++;
   } else {
@@ -118,18 +119,18 @@ AttVarToTerm(CELL *orig)
 }
 
 static int
-TermToAttVar(Term attvar, Term to)
+TermToAttVar(Term attvar, Term to USES_REGS)
 {
-  attvar_record *attv = BuildNewAttVar();
+  attvar_record *attv = BuildNewAttVar( PASS_REGS1 );
   if (!attv)
     return FALSE;
-  Bind(&attv->Atts, attvar);
+  Bind_Global_NonAtt(&attv->Atts, attvar);
   *VarOfTerm(to) = AbsAttVar(attv);
   return TRUE;
 }
 
 static void
-WakeAttVar(CELL* pt1, CELL reg2)
+WakeAttVar(CELL* pt1, CELL reg2 USES_REGS)
 {
   
   /* if bound to someone else, follow until we find the last one */
@@ -152,26 +153,26 @@ WakeAttVar(CELL* pt1, CELL reg2)
 	if (!IsVarTerm(susp2->Value) || !IsUnboundVar(&susp2->Value)) {
 	  /* oops, our goal is on the queue to be woken */
 	  if (!Yap_unify(susp2->Value, (CELL)pt1)) {
-	    AddFailToQueue();
+	    AddFailToQueue( PASS_REGS1 );
 	  }
 	}
-	Bind_Global(&(susp2->Value), (CELL)pt1);
-	AddToQueue(susp2);
+	Bind_Global_NonAtt(&(susp2->Value), (CELL)pt1);
+	AddToQueue(susp2 PASS_REGS);
 	return;
       }
     } else {
-      Bind(VarOfTerm(reg2), (CELL)pt1);
+      Bind_NonAtt(VarOfTerm(reg2), (CELL)pt1);
       return;
     }
   }
   if (!IsVarTerm(attv->Value) || !IsUnboundVar(&attv->Value)) {
     /* oops, our goal is on the queue to be woken */
     if (!Yap_unify(attv->Value, reg2)) {
-      AddFailToQueue();
+      AddFailToQueue( PASS_REGS1 );
     }
     return;
   }
-  bind_ptr = AddToQueue(attv);
+  bind_ptr = AddToQueue(attv PASS_REGS);
   if (IsNonVarTerm(reg2)) {
     if (IsPairTerm(reg2) && RepPair(reg2) == myH)
       reg2 = AbsPair(H);
@@ -179,15 +180,15 @@ WakeAttVar(CELL* pt1, CELL reg2)
       reg2 = AbsAppl(H);
   }
   *bind_ptr = reg2;
-  Bind_Global(&(attv->Value), reg2);
+  Bind_Global_NonAtt(&(attv->Value), reg2);
 }
 
 void
 Yap_WakeUp(CELL *pt0) {
+  CACHE_REGS
   CELL d0 = *pt0;
   RESET_VARIABLE(pt0);
-  TR--;
-  WakeAttVar(pt0, d0);
+  WakeAttVar(pt0, d0 PASS_REGS);
 }
 
 
@@ -198,13 +199,13 @@ mark_attvar(CELL *orig)
 }
 
 static Term
-BuildAttTerm(Functor mfun, UInt ar)
+BuildAttTerm(Functor mfun, UInt ar USES_REGS)
 {
   CELL *h0 = H;
   UInt i;
 
   if (H+(1024+ar) > ASP) {
-    Yap_Error_Size=ar*sizeof(CELL);
+    LOCAL_Error_Size=ar*sizeof(CELL);
     return 0L;
   }
   H[0] = (CELL)mfun;
@@ -240,7 +241,7 @@ SearchAttsForModuleName(Term start, Atom mname)
 }
 
 static void 
-AddNewModule(attvar_record *attv, Term t, int new, int do_it)
+AddNewModule(attvar_record *attv, Term t, int new, int do_it USES_REGS)
 {
   CELL *newp = RepAppl(t)+2;
   UInt i, ar = ArityOfFunctor((Functor)newp[-2]);
@@ -257,14 +258,16 @@ AddNewModule(attvar_record *attv, Term t, int new, int do_it)
   }
   if (!do_it)
     return;
-  if (IsVarTerm(attv->Atts)) {
-    Bind(&(attv->Atts),t);
+  if (new) {
+    attv->Atts = t;
+  } else if (IsVarTerm(attv->Atts)) {
+    MaBind(&(attv->Atts),t);
   } else {
     Term *wherep = &attv->Atts;
 
     do {
       if (IsVarTerm(*wherep)) {
-	Bind_Global(wherep,t);      
+	Bind_Global_NonAtt(wherep,t);      
 	return;
       } else {
 	wherep = RepAppl(Deref(*wherep))+1;
@@ -274,7 +277,7 @@ AddNewModule(attvar_record *attv, Term t, int new, int do_it)
 }
 
 static void 
-ReplaceAtts(attvar_record *attv, Term oatt, Term att)
+ReplaceAtts(attvar_record *attv, Term oatt, Term att USES_REGS)
 {
   UInt ar = ArityOfFunctor(FunctorOfTerm(oatt)), i;
   CELL *oldp = RepAppl(oatt)+1;
@@ -326,18 +329,18 @@ ReplaceAtts(attvar_record *attv, Term oatt, Term att)
 }
 
 static void 
-DelAllAtts(attvar_record *attv)
+DelAllAtts(attvar_record *attv USES_REGS)
 {
   MaBind(&(attv->Done), attv->Value);
 }
 
 static void 
-DelAtts(attvar_record *attv, Term oatt)
+DelAtts(attvar_record *attv, Term oatt USES_REGS)
 {
   Term t = ArgOfTerm(1,oatt);
   if (attv->Atts == oatt) {
     if (IsVarTerm(t)) {
-      DelAllAtts(attv);
+      DelAllAtts(attv PASS_REGS);
       return;
     }
     if (RepAppl(attv->Atts) >= HB)
@@ -359,39 +362,39 @@ DelAtts(attvar_record *attv, Term oatt)
 }
 
 static void 
-PutAtt(Int pos, Term atts, Term att)
+PutAtt(Int pos, Term atts, Term att USES_REGS)
 {
   if (IsVarTerm(att) && (CELL *)att > H && (CELL *)att < LCL0) {
     /* globalise locals */
     Term tnew = MkVarTerm();
-    Bind((CELL *)att, tnew);
+    Bind_NonAtt((CELL *)att, tnew);
     att = tnew;
   }
   MaBind(RepAppl(atts)+pos, att);
 }
 
 static Int
-BindAttVar(attvar_record *attv) {
+BindAttVar(attvar_record *attv USES_REGS) {
   if (IsVarTerm(attv->Done) && IsUnboundVar(&attv->Done)) {
     /* make sure we are not trying to bind a variable against itself */
     if (!IsVarTerm(attv->Value)) {
-      Bind_Global(&(attv->Done), attv->Value);
+      Bind_Global_NonAtt(&(attv->Done), attv->Value);
     } else if (IsVarTerm(attv->Value)) {
       Term t = Deref(attv->Value);
       if (IsVarTerm(t)) {
 	if (IsAttachedTerm(t)) {
 	  attvar_record *attv2 = RepAttVar(VarOfTerm(t));
 	  if (attv2 < attv) {
-	    Bind_Global(&(attv->Done), t);
+	    Bind_Global_NonAtt(&(attv->Done), t);
 	  } else {
-	    Bind_Global(&(attv2->Done), AbsAttVar(attv));
+	    Bind_Global_NonAtt(&(attv2->Done), AbsAttVar(attv));
 	  }
 	} else {
 	  Yap_Error(SYSTEM_ERROR,(CELL)&(attv->Done),"attvar was bound when unset");
 	  return(FALSE);
 	}
       } else {
-	Bind_Global(&(attv->Done), t);
+	Bind_Global_NonAtt(&(attv->Done), t);
       }
     }
     return(TRUE);
@@ -414,7 +417,7 @@ GetAllAtts(attvar_record *attv) {
 }
 
 static Int
-p_put_att(void) {
+p_put_att( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   /* if this is unbound, ok */
@@ -429,10 +432,10 @@ p_put_att(void) {
     if (IsAttachedTerm(inp)) {
       attv = RepAttVar(VarOfTerm(inp));
     } else {
-      while (!(attv = BuildNewAttVar())) {
-	Yap_Error_Size = sizeof(attvar_record);
-	if (!Yap_gcl(Yap_Error_Size, 5, ENV, gc_P(P,CP))) {
-	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, Yap_ErrorMessage);
+      while (!(attv = BuildNewAttVar( PASS_REGS1 ))) {
+	LOCAL_Error_Size = sizeof(attvar_record);
+	if (!Yap_gcl(LOCAL_Error_Size, 5, ENV, gc_P(P,CP))) {
+	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, LOCAL_ErrorMessage);
 	  return FALSE;
 	}    
 	inp = Deref(ARG1);
@@ -441,16 +444,20 @@ p_put_att(void) {
     }
     mfun= Yap_MkFunctor(modname,ar);
     if (IsVarTerm(tatts = SearchAttsForModule(attv->Atts,mfun))) {
-      while (!(tatts = BuildAttTerm(mfun,ar))) {
-	if (!Yap_gcl(Yap_Error_Size, 5, ENV, gc_P(P,CP))) {
-	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, Yap_ErrorMessage);
+      while (!(tatts = BuildAttTerm(mfun,ar PASS_REGS))) {
+	if (!Yap_gcl(LOCAL_Error_Size, 5, ENV, gc_P(P,CP))) {
+	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, LOCAL_ErrorMessage);
 	  return FALSE;
 	}    
       }
-      Yap_unify(ARG1, AbsAttVar(attv));
-      AddNewModule(attv,tatts,new,TRUE);
+      {
+	CELL *ptr = VarOfTerm(Deref(ARG1));
+	CELL d0 = AbsAttVar(attv);
+	Bind_NonAtt(ptr, d0);
+      }
+      AddNewModule(attv, tatts, new, TRUE PASS_REGS);
     }
-    PutAtt(IntegerOfTerm(Deref(ARG4)), tatts, Deref(ARG5));
+    PutAtt(IntegerOfTerm(Deref(ARG4)), tatts, Deref(ARG5) PASS_REGS);
     return TRUE;
   } else {
     Yap_Error(REPRESENTATION_ERROR_VARIABLE,inp,"first argument of put_attributes/2");
@@ -459,32 +466,27 @@ p_put_att(void) {
 }
 
 static Int
-p_put_att_term(void) {
+p_put_att_term( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   /* if this is unbound, ok */
   if (IsVarTerm(inp)) {
     attvar_record *attv;
-    int new = FALSE;
 
     if (IsAttachedTerm(inp)) {
       attv = RepAttVar(VarOfTerm(inp));
+      MaBind(&(attv->Atts), Deref(ARG2));
     } else {
-      while (!(attv = BuildNewAttVar())) {
-	Yap_Error_Size = sizeof(attvar_record);
-	if (!Yap_gcl(Yap_Error_Size, 5, ENV, gc_P(P,CP))) {
-	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, Yap_ErrorMessage);
+      while (!(attv = BuildNewAttVar( PASS_REGS1 ))) {
+	LOCAL_Error_Size = sizeof(attvar_record);
+	if (!Yap_gcl(LOCAL_Error_Size, 5, ENV, gc_P(P,CP))) {
+	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, LOCAL_ErrorMessage);
 	  return FALSE;
 	}    
 	inp = Deref(ARG1);
       }
-      new = TRUE;
-    }
-    if (new) {
-      Bind(VarOfTerm(inp), AbsAttVar(attv));
-      Bind(&attv->Atts, Deref(ARG2));
-    } else {
-      MaBind(&(attv->Atts), Deref(ARG2));
+      Bind_NonAtt(VarOfTerm(inp), AbsAttVar(attv));
+      attv->Atts = Deref(ARG2);
     }
     return TRUE;
   } else {
@@ -494,7 +496,7 @@ p_put_att_term(void) {
 }
 
 static Int
-p_rm_att(void) {
+p_rm_att( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   /* if this is unbound, ok */
@@ -509,10 +511,10 @@ p_rm_att(void) {
     if (IsAttachedTerm(inp)) {
       attv = RepAttVar(VarOfTerm(inp));
     } else {
-      while (!(attv = BuildNewAttVar())) {
-	Yap_Error_Size = sizeof(attvar_record);
-	if (!Yap_gcl(Yap_Error_Size, 5, ENV, gc_P(P,CP))) {
-	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, Yap_ErrorMessage);
+      while (!(attv = BuildNewAttVar( PASS_REGS1 ))) {
+	LOCAL_Error_Size = sizeof(attvar_record);
+	if (!Yap_gcl(LOCAL_Error_Size, 5, ENV, gc_P(P,CP))) {
+	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, LOCAL_ErrorMessage);
 	  return FALSE;
 	}    
 	inp = Deref(ARG1);
@@ -522,15 +524,15 @@ p_rm_att(void) {
     }
     mfun= Yap_MkFunctor(modname,ar);
     if (IsVarTerm(tatts = SearchAttsForModule(attv->Atts,mfun))) {
-      while (!(tatts = BuildAttTerm(mfun,ar))) {
-	if (!Yap_gcl(Yap_Error_Size, 4, ENV, gc_P(P,CP))) {
-	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, Yap_ErrorMessage);
+      while (!(tatts = BuildAttTerm(mfun, ar PASS_REGS))) {
+	if (!Yap_gcl(LOCAL_Error_Size, 4, ENV, gc_P(P,CP))) {
+	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, LOCAL_ErrorMessage);
 	  return FALSE;
 	}    
       }
-      AddNewModule(attv,tatts,new, FALSE);
+      AddNewModule(attv,tatts,new, FALSE PASS_REGS);
     } else {
-      PutAtt(IntegerOfTerm(Deref(ARG4)), tatts, TermVoidAtt);
+      PutAtt(IntegerOfTerm(Deref(ARG4)), tatts, TermVoidAtt PASS_REGS);
     }
     return TRUE;
   } else {
@@ -540,7 +542,7 @@ p_rm_att(void) {
 }
 
 static Int
-p_put_atts(void) {
+p_put_atts( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
 
@@ -556,10 +558,10 @@ p_put_atts(void) {
     if (IsAttachedTerm(inp)) {
       attv = RepAttVar(VarOfTerm(inp));
     } else {
-      while (!(attv = BuildNewAttVar())) {
-	Yap_Error_Size = sizeof(attvar_record);
-	if (!Yap_gcl(Yap_Error_Size, 2, ENV, gc_P(P,CP))) {
-	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, Yap_ErrorMessage);
+      while (!(attv = BuildNewAttVar( PASS_REGS1 ))) {
+	LOCAL_Error_Size = sizeof(attvar_record);
+	if (!Yap_gcl(LOCAL_Error_Size, 2, ENV, gc_P(P,CP))) {
+	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, LOCAL_ErrorMessage);
 	  return FALSE;
 	}    
 	tatts = Deref(ARG2);
@@ -577,9 +579,9 @@ p_put_atts(void) {
       return FALSE;
     }
     if (IsVarTerm(otatts = SearchAttsForModule(attv->Atts,mfun))) {
-      AddNewModule(attv,tatts,new,FALSE);
+      AddNewModule(attv, tatts, new, FALSE PASS_REGS);
     } else {
-      ReplaceAtts(attv, otatts, tatts);
+      ReplaceAtts(attv, otatts, tatts PASS_REGS);
     }
     return TRUE;
   } else {
@@ -589,7 +591,7 @@ p_put_atts(void) {
 }
 
 static Int
-p_del_atts(void) {
+p_del_atts( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   Term otatts;
@@ -608,7 +610,7 @@ p_del_atts(void) {
     if (IsVarTerm(otatts = SearchAttsForModule(attv->Atts,mfun))) {
       return TRUE;
     } else {
-      DelAtts(attv, otatts);
+      DelAtts(attv, otatts PASS_REGS);
     }
     return TRUE;
   } else {
@@ -617,7 +619,7 @@ p_del_atts(void) {
 }
 
 static Int
-p_del_all_atts(void) {
+p_del_all_atts( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
 
@@ -626,13 +628,13 @@ p_del_all_atts(void) {
     attvar_record *attv;
       
     attv = RepAttVar(VarOfTerm(inp));
-    DelAllAtts(attv);
+    DelAllAtts(attv PASS_REGS);
   } 
   return TRUE;
 }
 
 static Int
-p_get_att(void) {
+p_get_att( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   /* if this is unbound, ok */
@@ -660,7 +662,7 @@ p_get_att(void) {
 }
 
 static Int
-p_free_att(void) {
+p_free_att( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   /* if this is unbound, ok */
@@ -687,7 +689,7 @@ p_free_att(void) {
 }
 
 static Int
-p_get_atts(void) {
+p_get_atts( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   /* if this is unbound, ok */
@@ -727,7 +729,7 @@ p_get_atts(void) {
 }
 
 static Int
-p_has_atts(void) {
+p_has_atts( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   /* if this is unbound, ok */
@@ -751,14 +753,14 @@ p_has_atts(void) {
 }
 
 static Int
-p_bind_attvar(void) {
+p_bind_attvar( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term  inp = Deref(ARG1);
   /* if this is unbound, ok */
   if (IsVarTerm(inp)) {
     if (IsAttachedTerm(inp)) {
       attvar_record *attv = RepAttVar(VarOfTerm(inp));
-      return(BindAttVar(attv));
+      return(BindAttVar(attv PASS_REGS));
     }
     return(TRUE);
   } else {
@@ -768,7 +770,7 @@ p_bind_attvar(void) {
 }
 
 static Int
-p_unbind_attvar(void) {
+p_unbind_attvar( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term  inp = Deref(ARG1);
   /* if this is unbound, ok */
@@ -785,7 +787,7 @@ p_unbind_attvar(void) {
 }
 
 static Int
-p_get_all_atts(void) {
+p_get_all_atts( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   /* if this is unbound, ok */
@@ -815,7 +817,7 @@ ActiveAtt(Term tatt, UInt ar)
 }
 
 static Int
-p_modules_with_atts(void) {
+p_modules_with_atts( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   /* if this is unbound, ok */
@@ -850,7 +852,7 @@ p_modules_with_atts(void) {
 }
 
 static Int
-p_swi_all_atts(void) {
+p_swi_all_atts( USES_REGS1 ) {
   /* receive a variable in ARG1 */
   Term inp = Deref(ARG1);
   Functor attf = FunctorAtt1;
@@ -895,16 +897,16 @@ p_swi_all_atts(void) {
 
 
 static Term
-AllAttVars(void) {
+AllAttVars( USES_REGS1 ) {
   CELL *pt = H0;
   CELL *myH = H;
   
-  while (pt < H) {
+  while (pt < myH) {
     switch(*pt) {
     case (CELL)FunctorAttVar:
       if (IsUnboundVar(pt+1)) {
 	if (ASP - myH < 1024) {
-	  Yap_Error_Size = (ASP-H)*sizeof(CELL);
+	  LOCAL_Error_Size = (ASP-H)*sizeof(CELL);
 	  return 0L;
 	}
 	if (myH != H) {
@@ -948,14 +950,14 @@ AllAttVars(void) {
 }
   
 static Int
-p_all_attvars(void)
+p_all_attvars( USES_REGS1 )
 {
   do {
     Term out;
 
-    if (!(out = AllAttVars())) {
-      if (!Yap_gcl(Yap_Error_Size, 1, ENV, gc_P(P,CP))) {
-	Yap_Error(OUT_OF_STACK_ERROR, TermNil, Yap_ErrorMessage);
+    if (!(out = AllAttVars( PASS_REGS1 ))) {
+      if (!Yap_gcl(LOCAL_Error_Size, 1, ENV, gc_P(P,CP))) {
+	Yap_Error(OUT_OF_STACK_ERROR, TermNil, LOCAL_ErrorMessage);
 	return FALSE;
       }    
     } else {
@@ -965,7 +967,7 @@ p_all_attvars(void)
 }
 
 static Int
-p_is_attvar(void)
+p_is_attvar( USES_REGS1 )
 {
   Term t = Deref(ARG1);
   return(IsVarTerm(t) &&
@@ -974,7 +976,7 @@ p_is_attvar(void)
 
 /* check if we are not redoing effort */
 static Int
-p_attvar_bound(void)
+p_attvar_bound( USES_REGS1 )
 {
   Term t = Deref(ARG1);
   return
@@ -984,19 +986,19 @@ p_attvar_bound(void)
 }
 
 static Int
-p_void_term(void)
+p_void_term( USES_REGS1 )
 {
   return Yap_unify(ARG1,TermVoidAtt);
 }
 
 static Int
-p_free_term(void)
+p_free_term( USES_REGS1 )
 {
   return Yap_unify(ARG1,TermFreeTerm);
 }
 
 static Int
-p_fast_unify(void)
+p_fast_unify( USES_REGS1 )
 {
   /*
     Case we want to unify two variables, but we do not
@@ -1011,9 +1013,9 @@ p_fast_unify(void)
   a = VarOfTerm(t1);
   b = VarOfTerm(t2);
   if(a > b) {						 
-    Bind_Global(a,t2);				 
+    Bind_Global_NonAtt(a,t2);				 
   } else if((a) < (b)){						 
-    Bind_Global(b,t1);				 
+    Bind_Global_NonAtt(b,t1);				 
   }
   return TRUE;
 }
@@ -1021,35 +1023,36 @@ p_fast_unify(void)
 #else
 
 static Int
-p_all_attvars(void)
+p_all_attvars( USES_REGS1 )
 {
   return FALSE;
 }
 
 static Int
-p_is_attvar(void)
+p_is_attvar( USES_REGS1 )
 {
   return FALSE;
 }
 
 static Int
-p_attvar_bound(void)
+p_attvar_bound( USES_REGS1 )
 {
-b  return FALSE;
+  return FALSE;
 }
 
 #endif /* COROUTINING */
 
 void Yap_InitAttVarPreds(void)
 {
+  CACHE_REGS
   Term OldCurrentModule = CurrentModule;
   CurrentModule = ATTRIBUTES_MODULE;
 #ifdef COROUTINING
-  attas[attvars_ext].bind_op = WakeAttVar;
-  attas[attvars_ext].copy_term_op = CopyAttVar;
-  attas[attvars_ext].to_term_op = AttVarToTerm;
-  attas[attvars_ext].term_to_op = TermToAttVar;
-  attas[attvars_ext].mark_op = mark_attvar;
+  GLOBAL_attas[attvars_ext].bind_op = WakeAttVar;
+  GLOBAL_attas[attvars_ext].copy_term_op = CopyAttVar;
+  GLOBAL_attas[attvars_ext].to_term_op = AttVarToTerm;
+  GLOBAL_attas[attvars_ext].term_to_op = TermToAttVar;
+  GLOBAL_attas[attvars_ext].mark_op = mark_attvar;
   Yap_InitCPred("get_att", 4, p_get_att, SafePredFlag);
   Yap_InitCPred("get_module_atts", 2, p_get_atts, SafePredFlag);
   Yap_InitCPred("has_module_atts", 2, p_has_atts, SafePredFlag);
@@ -1072,7 +1075,7 @@ void Yap_InitAttVarPreds(void)
   Yap_InitCPred("all_attvars", 1, p_all_attvars, 0);
   CurrentModule = OldCurrentModule;
   Yap_InitCPred("attvar", 1, p_is_attvar, SafePredFlag|TestPredFlag);
-  Yap_InitCPred("$att_bound", 1, p_attvar_bound, SafePredFlag|TestPredFlag|HiddenPredFlag);
+  Yap_InitCPred("$att_bound", 1, p_attvar_bound, SafePredFlag|TestPredFlag);
 }
 
 
