@@ -75,12 +75,7 @@ STATIC_PROTO (void ShowOp, (char *, struct PSEUDO *));
  * afterwards 
  */
 
-#ifdef DEBUG
 
-char            Yap_Option[20];
-
-YP_FILE *Yap_logfile;
-#endif
 
 typedef struct mem_blk {
   union {
@@ -111,26 +106,28 @@ AllocCMem (UInt size, struct intermediates *cip)
     else
       blksz = CMEM_BLK_SIZE;
     if (!cip->blks) {
-      if (Yap_CMemFirstBlock) {
-	p = Yap_CMemFirstBlock;
-	blksz = Yap_CMemFirstBlockSz;
+      CACHE_REGS
+      if (LOCAL_CMemFirstBlock) {
+	p = LOCAL_CMemFirstBlock;
+	blksz = LOCAL_CMemFirstBlockSz;
 	p->u.next = NULL;
       } else {
 	if (blksz < FIRST_CMEM_BLK_SIZE)
 	  blksz = FIRST_CMEM_BLK_SIZE;
 	p = (struct mem_blk *)Yap_AllocCodeSpace(blksz);
 	if (!p) {
-	  Yap_Error_Size = size;
+	  LOCAL_Error_Size = size;
 	  save_machine_regs();
 	  siglongjmp(cip->CompilerBotch, OUT_OF_HEAP_BOTCH);
 	}
-	Yap_CMemFirstBlock = p;
-	Yap_CMemFirstBlockSz = blksz;
+	LOCAL_CMemFirstBlock = p;
+	LOCAL_CMemFirstBlockSz = blksz;
       }
     } else {
       p = (struct mem_blk *)Yap_AllocCodeSpace(blksz);
       if (!p) {
-	Yap_Error_Size = size;
+	CACHE_REGS
+	LOCAL_Error_Size = size;
 	save_machine_regs();
 	siglongjmp(cip->CompilerBotch, OUT_OF_HEAP_BOTCH);
       }
@@ -150,7 +147,8 @@ AllocCMem (UInt size, struct intermediates *cip)
   p = cip->freep;
   cip->freep += size;
   if (ASP <= CellPtr (cip->freep) + 256) {
-    Yap_Error_Size = 256+((char *)cip->freep - (char *)H);
+    CACHE_REGS
+    LOCAL_Error_Size = 256+((char *)cip->freep - (char *)H);
     save_machine_regs();
     siglongjmp(cip->CompilerBotch, OUT_OF_STACK_BOTCH);
   }
@@ -162,16 +160,17 @@ void
 Yap_ReleaseCMem (struct intermediates *cip)
 {
 #if USE_SYSTEM_MALLOC
+  CACHE_REGS
   struct mem_blk *p = cip->blks;
   while (p) {
     struct mem_blk *nextp = p->u.next;
-    if (p != Yap_CMemFirstBlock)
+    if (p != LOCAL_CMemFirstBlock)
       Yap_FreeCodeSpace((ADDR)p);
     p = nextp;
   }
   cip->blks = NULL;
   if (cip->label_offset &&
-      cip->label_offset != Yap_LabelFirstArray) {
+      cip->label_offset != LOCAL_LabelFirstArray) {
     Yap_FreeCodeSpace((ADDR)cip->label_offset);
   }
 #endif
@@ -410,7 +409,7 @@ write_address(CELL address)
 #if HAVE_SNPRINTF
     snprintf(buf,32,"%p",(void *)address);
 #else
-    snprintf(buf,"%p",(void *)address);
+    sprintf(buf,"%p",(void *)address);
 #endif
     p[31] = '\0'; /* so that I don't have to worry */
     Yap_DebugErrorPutc('0');
@@ -429,6 +428,8 @@ write_functor(Functor f)
       Yap_DebugPlWrite(MkAtomTerm(AtomDBREF));
     } else if (f == FunctorLongInt) {
       Yap_DebugPlWrite(MkAtomTerm(AtomLONGINT));
+    } else if (f == FunctorBigInt) {
+      Yap_DebugPlWrite(MkAtomTerm(AtomLONGINT));
     } else if (f == FunctorDouble) {
       Yap_DebugPlWrite(MkAtomTerm(AtomDOUBLE));
     }
@@ -843,6 +844,7 @@ static char *opformat[] =
 void
 Yap_ShowCode (struct intermediates *cint)
 {
+  CACHE_REGS
   CELL *oldH = H;
   struct PSEUDO *cpc;
 
