@@ -29,13 +29,17 @@ static char     SccsId[] = "%W% %G%";
 #include "alloc.h"
 #include "clause.h"
 #include "Foreign.h"
+#include "SWI-Prolog.h"
 
 #ifdef LOW_LEVEL_TRACER
 #include "tracer.h"
 #endif
 #ifdef YAPOR
+#ifdef YAPOR_COW
+#include <signal.h>
+#endif /* YAPOR_COW */
 #include "or.macros.h"
-#endif	/* YAPOR */
+#endif /* YAPOR */
 #if defined(YAPOR) || defined(TABLING)
 #if HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -73,116 +77,15 @@ STATIC_PROTO(void  InitStdPreds,(void));
 STATIC_PROTO(void  InitFlags, (void));
 STATIC_PROTO(void  InitCodes, (void));
 STATIC_PROTO(void  InitVersion, (void));
-
-
 STD_PROTO(void  exit, (int));
+static void InitWorker(int wid);
+
 
 /**************	YAP PROLOG GLOBAL VARIABLES *************************/
 
 /************* variables related to memory allocation ***************/
-
-#if defined(THREADS)
-
 ADDR Yap_HeapBase;
 
-struct restore_info rinfo[MAX_THREADS];
-
-struct thread_globs Yap_thread_gl[MAX_THREADS];
-
-pthread_t Yap_master_thread;
-
-#else
-
-struct restore_info rinfo;
-
-ADDR Yap_HeapBase,
-  Yap_LocalBase,
-  Yap_GlobalBase,
-  Yap_TrailBase,
-  Yap_TrailTop;
-
-/************ variables	concerned with Error Handling *************/
-char           *Yap_ErrorMessage;	/* used to pass error messages */
-Term              Yap_Error_Term;	/* used to pass error terms */
-yap_error_number  Yap_Error_TYPE;	/* used to pass the error */
-UInt             Yap_Error_Size;	/* used to pass a size associated with an error */
-
-/******************* storing error messages ****************************/
-char      Yap_ErrorSay[MAX_ERROR_MSG_SIZE];
-
-/* if we botched in a LongIO operation */
-jmp_buf Yap_IOBotch;
-
-/* if we botched in the compiler */
-jmp_buf Yap_CompilerBotch;
-
-/************ variables	concerned with Error Handling *************/
-sigjmp_buf         Yap_RestartEnv;	/* used to restart after an abort execution */
-
-/********* IO support	*****/
-
-/********* parsing ********************************************/
-
-TokEntry *Yap_tokptr, *Yap_toktide;
-VarEntry *Yap_VarTable, *Yap_AnonVarTable;
-int Yap_eot_before_eof = FALSE;
-
-/******************* intermediate buffers **********************/
-
-char     Yap_FileNameBuf[YAP_FILENAME_MAX],
-         Yap_FileNameBuf2[YAP_FILENAME_MAX];
-
-#endif /* THREADS */
-
-/******** whether Yap is responsible for signal handling******************/
-int             Yap_PrologShouldHandleInterrupts;
-
-/********* readline support	*****/
-#if HAVE_LIBREADLINE
-
-char *_line = (char *) NULL;
-
-#endif
-
-#ifdef MPWSHELL
-/********** informing if we are in the MPW shell ********************/
-
-int             mpwshell = FALSE;
-
-#endif
-
-#ifdef EMACS
-
-int             emacs_mode = FALSE;
-char            emacs_tmp[256], emacs_tmp2[256];
-
-#endif
-
-/********* Prolog State ********************************************/
-
-Int      Yap_PrologMode = BootMode;
-
-int      Yap_CritLocks = 0;
-
-/********* streams ********************************************/
-
-YP_FILE *Yap_stdin;
-YP_FILE *Yap_stdout;
-YP_FILE *Yap_stderr;
-
-
-/************** Access to yap initial arguments ***************************/
-
-char          **Yap_argv;
-int             Yap_argc;
-
-/************** Extensions to Terms ***************************************/
-
-#ifdef COROUTINING
-/* array with the ops for your favourite extensions */
-ext_op attas[attvars_ext+1];
-#endif
-
 /**************	declarations local to init.c ************************/
 static char    *optypes[] =
 {"", "xfx", "xfy", "yfx", "xf", "yf", "fx", "fy"};
@@ -195,10 +98,6 @@ int Yap_page_size;
 void **Yap_ABSMI_OPCODES;
 #endif
 
-#if   USE_SOCKET
-int Yap_sockets_io=0;
-#endif
-
 #if DEBUG
 #if COROUTINING
 int  Yap_Portray_delays = FALSE;
@@ -253,14 +152,13 @@ OpDec(int p, char *type, Atom a, Term m)
   if (EndOfPAEntr(info)) {
     info = (OpEntry *) Yap_AllocAtomSpace(sizeof(OpEntry));
     info->KindOfPE = Ord(OpProperty);
-    info->NextOfPE = RepAtom(a)->PropsOfAE;
     info->OpModule = m;
     info->OpName = a;
     LOCK(OpListLock);
     info->OpNext = OpList;
     OpList = info;
     UNLOCK(OpListLock);
-    RepAtom(a)->PropsOfAE = AbsOpProp(info);
+    AddPropToAtom(ae, (PropEntry *)info);
     INIT_RWLOCK(info->OpRWLock);
     WRITE_LOCK(info->OpRWLock);
     WRITE_UNLOCK(ae->ARWLock);
@@ -304,7 +202,7 @@ static void
 SetOp(int p, int type, char *at, Term m)
 {
 #ifdef DEBUG
-  if (Yap_Option[5])
+  if (GLOBAL_Option[5])
     fprintf(stderr,"[setop %d %s %s]\n", p, optypes[type], at);
 #endif
   OpDec(p, optypes[type], Yap_LookupAtom(at), m);
@@ -409,6 +307,7 @@ static Opdef    Ops[] = {
   {"#", yfx, 500},
   {"rdiv", yfx, 400},
   {"div", yfx, 400},
+  {"xor", yfx, 400},
   {"*", yfx, 400},
   {"/", yfx, 400},
   {"//", yfx, 400},
@@ -446,7 +345,7 @@ InitDebug(void)
   int i;
 
   for (i = 1; i < 20; ++i)
-    Yap_Option[i] = 0;
+    GLOBAL_Option[i] = 0;
   if (Yap_output_msg) {
     char            ch;
 
@@ -462,10 +361,10 @@ InitDebug(void)
     fprintf(stderr,"m Machine\t p parser\n");
     while ((ch = YP_putchar(YP_getchar())) != '\n')
       if (ch >= 'a' && ch <= 'z')
-	Yap_Option[ch - 'a' + 1] = 1;
-    if (Yap_Option['l' - 96]) {
-      Yap_logfile = fopen(LOGFILE, "w");
-      if (Yap_logfile == NULL) {
+	GLOBAL_Option[ch - 'a' + 1] = 1;
+    if (GLOBAL_Option['l' - 96]) {
+      GLOBAL_logfile = fopen(LOGFILE, "w");
+      if (GLOBAL_logfile == NULL) {
 	fprintf(stderr,"can not open %s\n", LOGFILE);
 	getchar();
 	exit(0);
@@ -493,8 +392,6 @@ update_flags_from_prolog(UInt flags, PredEntry *pe)
     flags |= SourcePredFlag;
   if (pe->PredFlags & SequentialPredFlag)
     flags |= SequentialPredFlag;
-  if (pe->PredFlags & MyddasPredFlag)
-    flags |= MyddasPredFlag;
   if (pe->PredFlags & UDIPredFlag)
     flags |= UDIPredFlag;
   if (pe->PredFlags & ModuleTransparentPredFlag)
@@ -505,6 +402,7 @@ update_flags_from_prolog(UInt flags, PredEntry *pe)
 void 
 Yap_InitCPred(char *Name, unsigned long int Arity, CPredicate code, UInt flags)
 {
+  CACHE_REGS
   Atom              atom = NIL;
   PredEntry        *pe = NULL;
   yamop            *p_code;
@@ -512,7 +410,10 @@ Yap_InitCPred(char *Name, unsigned long int Arity, CPredicate code, UInt flags)
   Functor           f = NULL;
 
   while (atom == NIL) {
-    atom = Yap_FullLookupAtom(Name);
+    if (flags & UserCPredFlag)
+      atom = Yap_LookupAtom(Name);
+    else
+      atom = Yap_FullLookupAtom(Name);
     if (atom == NIL && !Yap_growheap(FALSE, 0L, NULL)) {
       Yap_Error(OUT_OF_HEAP_ERROR,TermNil,"while initialising %s", Name);
       return;
@@ -604,6 +505,7 @@ Yap_InitCPred(char *Name, unsigned long int Arity, CPredicate code, UInt flags)
 void 
 Yap_InitCmpPred(char *Name, unsigned long int Arity, CmpPredicate cmp_code, UInt flags)
 {
+  CACHE_REGS
   Atom              atom = NIL;
   PredEntry        *pe = NULL;
   yamop            *p_code = NULL;
@@ -681,6 +583,7 @@ Yap_InitCmpPred(char *Name, unsigned long int Arity, CmpPredicate cmp_code, UInt
 void 
 Yap_InitAsmPred(char *Name,  unsigned long int Arity, int code, CPredicate def, UInt flags)
 {
+  CACHE_REGS
   Atom            atom = NIL;
   PredEntry      *pe = NULL;
   Functor           f = NULL;
@@ -751,7 +654,7 @@ Yap_InitAsmPred(char *Name,  unsigned long int Arity, int code, CPredicate def,
       p_code->opc = Yap_opcode(_allocate);
       p_code = NEXTOP(p_code,e);
     }
-    p_code->opc = pe->OpcodeOfPred = Yap_opcode(_call_cpred);
+    p_code->opc = Yap_opcode(_call_cpred);
     p_code->u.Osbpp.bmap = NULL;
     p_code->u.Osbpp.s = -Signed(RealEnvSize);
     p_code->u.Osbpp.p = p_code->u.Osbpp.p0 = pe;
@@ -766,6 +669,7 @@ Yap_InitAsmPred(char *Name,  unsigned long int Arity, int code, CPredicate def,
     p_code = NEXTOP(p_code,p);
     p_code->opc = Yap_opcode(_Ystop);
     p_code->u.l.l = cl->ClCode;
+    pe->OpcodeOfPred = pe->CodeOfPred->opc;
   } else {
     pe->OpcodeOfPred = Yap_opcode(_undef_p);
     pe->CodeOfPred =  (yamop *)(&(pe->OpcodeOfPred)); 
@@ -789,6 +693,7 @@ CleanBack(PredEntry *pe, CPredicate Start, CPredicate Cont)
     return;
   }
   code = (yamop *)(pe->cs.p_code.FirstClause);
+  code->u.OtapFs.p = pe;
   if (pe->PredFlags & UserCPredFlag)
     code->opc = Yap_opcode(_try_userc);
   else
@@ -814,6 +719,7 @@ CleanBack(PredEntry *pe, CPredicate Start, CPredicate Cont)
     code->opc = Yap_opcode(_cut_c);
   else
     code->opc = Yap_opcode(_cut_userc);
+  code->u.OtapFs.p = pe;
   code->u.OtapFs.f = Cut;
 #endif
 }
@@ -852,6 +758,7 @@ Yap_InitCPredBack(char *Name, unsigned long int Arity,
 		  CPredicate Cont, UInt flags)
 #endif 
 {
+  CACHE_REGS
   PredEntry      *pe = NULL;
   Atom            atom = NIL;
   Functor           f = NULL;
@@ -895,9 +802,9 @@ Yap_InitCPredBack(char *Name, unsigned long int Arity,
     StaticClause *cl;
     yamop      *code = ((StaticClause *)NULL)->ClCode;
     if (flags &  UserCPredFlag) 
-      pe->PredFlags = UserCPredFlag | CompiledPredFlag | StandardPredFlag | flags;
+      pe->PredFlags = UserCPredFlag | BackCPredFlag| CompiledPredFlag | StandardPredFlag | flags;
     else
-      pe->PredFlags = CompiledPredFlag | StandardPredFlag;
+      pe->PredFlags = CompiledPredFlag | StandardPredFlag | BackCPredFlag;
 
 #ifdef YAPOR
     pe->PredFlags |= SequentialPredFlag;
@@ -1001,16 +908,14 @@ InitFlags(void)
   yap_flags[SOURCE_MODE_FLAG] = FALSE;
   yap_flags[CHARACTER_ESCAPE_FLAG] = SICSTUS_CHARACTER_ESCAPES;
   yap_flags[WRITE_QUOTED_STRING_FLAG] = FALSE;
-#if (defined(YAPOR) || defined(THREADS)) && PUREe_YAPOR
-  yap_flags[ALLOW_ASSERTING_STATIC_FLAG] = FALSE;
-#else
+  /* we do not garantee safe assert in parallel mode */
   yap_flags[ALLOW_ASSERTING_STATIC_FLAG] = TRUE;
-#endif
   yap_flags[GENERATE_DEBUG_INFO_FLAG] = TRUE;
   /* current default */
   yap_flags[INDEXING_MODE_FLAG] = INDEX_MODE_MULTI;
   yap_flags[TABLING_MODE_FLAG] = 0;
   yap_flags[QUIET_MODE_FLAG] = FALSE;
+  yap_flags[INDEXING_TERM_DEPTH_FLAG] = 0L;
 }
 
 static void
@@ -1100,9 +1005,12 @@ InitLogDBErasedMarker(void)
 static void 
 InitSWIAtoms(void)
 {
+  extern atom_t ATOM_;
+
   int i=0, j=0;
 #include "iswiatoms.h"
   Yap_InitSWIHash();
+  ATOM_ = PL_new_atom("");
 }
 
 static void 
@@ -1161,99 +1069,155 @@ InitInvisibleAtoms(void)
   INIT_RWLOCK(Yap_heap_regs->invisiblechain.AERWLock);
 }
 
+
+#ifdef YAPOR
+void Yap_init_yapor_workers(void) {
+  CACHE_REGS
+  int proc;
+#ifdef YAPOR_THREADS
+  return;
+#endif /* YAPOR_THREADS */
+#ifdef YAPOR_COW
+  GLOBAL_master_worker = getpid();
+  if (GLOBAL_number_workers > 1) {
+    int son;
+    son = fork();
+    if (son == -1)
+      Yap_Error(FATAL_ERROR, TermNil, "fork error (Yap_init_yapor_workers)");
+    if (son > 0) {
+      /* I am the father, I must stay here and wait for my children to all die */
+      struct sigaction sigact;
+      sigact.sa_handler = SIG_DFL;
+      sigemptyset(&sigact.sa_mask);
+      sigact.sa_flags = SA_RESTART;
+      sigaction(SIGINT, &sigact, NULL);
+      pause();
+      exit(0);
+    } else
+      GLOBAL_worker_pid(0) = getpid();
+  }
+#endif /* YAPOR_COW */
+  for (proc = 1; proc < GLOBAL_number_workers; proc++) {
+    int son;
+    son = fork();
+    if (son == -1)
+      Yap_Error(FATAL_ERROR, TermNil, "fork error (Yap_init_yapor_workers)");
+    if (son == 0) { 
+      /* new worker */
+      worker_id = proc;
+      Yap_remap_yapor_memory();
+      LOCAL = REMOTE(worker_id);
+      memcpy(REMOTE(worker_id), REMOTE(0), sizeof(struct worker_local));
+      InitWorker(worker_id);
+      break;
+    } else
+      GLOBAL_worker_pid(proc) = son;
+  }
+}
+#endif /* YAPOR */
+
+
 #ifdef  THREADS
 static void
 InitThreadHandle(int wid)
 {
-    FOREIGN_ThreadHandle(wid).in_use = FALSE;
-    FOREIGN_ThreadHandle(wid).zombie = FALSE;
-    FOREIGN_ThreadHandle(wid).local_preds = NULL;
+  REMOTE_ThreadHandle(wid).in_use = FALSE;
+  REMOTE_ThreadHandle(wid).zombie = FALSE;
+  REMOTE_ThreadHandle(wid).local_preds = NULL;
 #ifdef LOW_LEVEL_TRACER
-    FOREIGN_ThreadHandle(wid).thread_inst_count = 0LL;
+  REMOTE_ThreadHandle(wid).thread_inst_count = 0LL;
 #endif
-    pthread_mutex_init(&(FOREIGN_ThreadHandle(wid).tlock), NULL);  
-    pthread_mutex_init(&(FOREIGN_ThreadHandle(wid).tlock_status), NULL);  
-    FOREIGN_ThreadHandle(wid).tdetach = (CELL)0;
-    FOREIGN_ThreadHandle(wid).cmod = (CELL)0;
+  pthread_mutex_init(&(REMOTE_ThreadHandle(wid).tlock), NULL);  
+  pthread_mutex_init(&(REMOTE_ThreadHandle(wid).tlock_status), NULL);  
+  REMOTE_ThreadHandle(wid).tdetach = (CELL)0;
+  REMOTE_ThreadHandle(wid).cmod = (CELL)0;
 }
-#endif
 
-static void
-InitSWIBuffers(int wid)
+int
+Yap_InitThread(int new_id)
 {
-  int i;
-
-  FOREIGN_WL(wid)->SWI_buffers_[0] = malloc(SWI_BUF_SIZE);
-  FOREIGN_WL(wid)->SWI_buffers_sz_[0] = SWI_BUF_SIZE;
-  for (i=1; i <= SWI_BUF_RINGS; i++) {
-    FOREIGN_WL(wid)->SWI_buffers_[i] = NULL;
-    FOREIGN_WL(wid)->SWI_buffers_sz_[i] = 0;
+  struct worker_local *new_s;
+  if (new_id) {
+    if (!(new_s = (struct worker_local *)calloc(sizeof(struct worker_local), 1)))
+      return FALSE;
+    Yap_local[new_id] = new_s;
   }
+  InitWorker(new_id);
+  return TRUE;
 }
-
+#endif
 
 static void 
 InitFirstWorkerThreadHandle(void)
 {
 #ifdef  THREADS
-  ThreadHandle.id = 0;
-  ThreadHandle.in_use = TRUE;
-  ThreadHandle.default_yaam_regs = 
+  CACHE_REGS
+  LOCAL_ThreadHandle.id = 0;
+  LOCAL_ThreadHandle.in_use = TRUE;
+  LOCAL_ThreadHandle.default_yaam_regs = 
     &Yap_standard_regs;
-  ThreadHandle.pthread_handle = pthread_self();
-  pthread_mutex_init(&FOREIGN_ThreadHandle(0).tlock, NULL);
-  pthread_mutex_init(&FOREIGN_ThreadHandle(0).tlock_status, NULL);
-  ThreadHandle.tdetach = MkAtomTerm(AtomFalse);
+  LOCAL_ThreadHandle.pthread_handle = pthread_self();
+  pthread_mutex_init(&REMOTE_ThreadHandle(0).tlock, NULL);
+  pthread_mutex_init(&REMOTE_ThreadHandle(0).tlock_status, NULL);
+  LOCAL_ThreadHandle.tdetach = MkAtomTerm(AtomFalse);
+  LOCAL_ThreadHandle.ref_count = 1;
 #endif
 }
 
 static void
 InitScratchPad(int wid)
 {
-  FOREIGN_WL(wid)->scratchpad.ptr = NULL;
-  FOREIGN_WL(wid)->scratchpad.sz = SCRATCH_START_SIZE;
-  FOREIGN_WL(wid)->scratchpad.msz = SCRATCH_START_SIZE;
+  REMOTE_ScratchPad(wid).ptr = NULL;
+  REMOTE_ScratchPad(wid).sz = SCRATCH_START_SIZE;
+  REMOTE_ScratchPad(wid).msz = SCRATCH_START_SIZE;
 }
 
 void
 Yap_CloseScratchPad(void)
 {
-  Yap_FreeCodeSpace(ScratchPad.ptr);
-  ScratchPad.sz = SCRATCH_START_SIZE;
-  ScratchPad.msz = SCRATCH_START_SIZE;
+  CACHE_REGS
+  Yap_FreeCodeSpace(LOCAL_ScratchPad.ptr);
+  LOCAL_ScratchPad.sz = SCRATCH_START_SIZE;
+  LOCAL_ScratchPad.msz = SCRATCH_START_SIZE;
 }
 
 #include "iglobals.h"
+#include "ilocals.h"
 
-#if defined(YAPOR) || defined(THREADS)
-#define MAX_INITS MAX_AGENTS
+
+#if defined(YAPOR_COPY) || defined(YAPOR_COW) || defined(YAPOR_SBA)
+struct global_data *Yap_global;
+long Yap_worker_area_size;
 #else
-#define MAX_INITS 1
+struct global_data Yap_Global;
 #endif
 
-#if defined(YAPOR) &&  !defined(THREADS)
-struct worker_shared *Yap_global;
-#else
-struct worker_shared Yap_Global;
-#endif
-
-#if defined(YAPOR) &&  !defined(THREADS)
-struct worker_local	*Yap_WLocal;
-#elif defined(YAPOR) || defined(THREADS)
-struct worker_local	Yap_WLocal[MAX_AGENTS];
-#else
-struct worker_local	Yap_WLocal;
+#if defined(THREADS)
+struct worker_local	*Yap_local[MAX_THREADS];
+#elif defined(YAPOR)
+struct worker_local	*Yap_local;
+#else /* !THREADS && !YAPOR */
+struct worker_local	Yap_local;
 #endif
 
 static void 
 InitCodes(void)
 {
+  CACHE_REGS
+#if THREADS
   int wid;
-#include "ihstruct.h"
-  InitGlobal();
-  for (wid = 0; wid < MAX_INITS; wid++) {
-    InitWorker(wid);
+  for (wid = 1; wid < MAX_THREADS; wid++) {
+    Yap_local[wid] = NULL;
   }
+#endif
+#include "ihstruct.h"
+#if THREADS
+  Yap_InitThread(0);
+#endif /* THREADS */
+  InitGlobal();
+#if !THREADS
+  InitWorker(0);
+#endif /* THREADS */
   InitFirstWorkerThreadHandle();
   /* make sure no one else can use these two atoms */
   CurrentModule = 0;
@@ -1288,14 +1252,17 @@ void
 Yap_InitWorkspace(UInt Heap, UInt Stack, UInt Trail, UInt Atts, UInt max_table_size, 
                   int n_workers, int sch_loop, int delay_load)
 {
+  CACHE_REGS
   int             i;
 
   /* initialise system stuff */
 #if PUSH_REGS
 #ifdef THREADS
+  if (!(Yap_local[0] = (struct worker_local *)calloc(sizeof(struct worker_local), 1)))
+    return;
   pthread_key_create(&Yap_yaamregs_key, NULL);
   pthread_setspecific(Yap_yaamregs_key, (const void *)&Yap_standard_regs);
-  Yap_master_thread = pthread_self();
+  GLOBAL_master_thread = pthread_self();
 #else
   /* In this case we need to initialise the abstract registers */
   Yap_regp = &Yap_standard_regs;
@@ -1305,12 +1272,12 @@ Yap_InitWorkspace(UInt Heap, UInt Stack, UInt Trail, UInt Atts, UInt max_table_s
 #endif /* PUSH_REGS */
 
 #ifdef THREADS
-  Yap_regp->worker_id_ = 0;
-#endif
-  /* Init signal handling and time */
-  /* also init memory page size, required by later functions */
-  Yap_InitSysbits ();
-
+  Yap_regp = ((REGSTORE *)pthread_getspecific(Yap_yaamregs_key));
+  LOCAL = REMOTE(0);
+#endif /* THREADS */
+#if defined(YAPOR_COPY) || defined(YAPOR_COW) || defined(YAPOR_SBA)
+  LOCAL = REMOTE(0);
+#endif /* YAPOR_COPY || YAPOR_COW || YAPOR_SBA */
   if (Heap < MinHeapSpace)
     Heap = MinHeapSpace;
   Heap = AdjustPageSize(Heap * K);
@@ -1325,30 +1292,36 @@ Yap_InitWorkspace(UInt Heap, UInt Stack, UInt Trail, UInt Atts, UInt max_table_s
     Atts = 2048*sizeof(CELL);
   else
     Atts = AdjustPageSize(Atts * K);
-#if defined(YAPOR) && !defined(THREADS)
+#if defined(YAPOR) || defined(THREADS)
   worker_id = 0;
+#endif /* YAPOR || THREADS */
+#ifdef YAPOR
   if (n_workers > MAX_WORKERS)
-    Yap_Error(INTERNAL_ERROR, TermNil, "excessive number of workers (Yap_InitWorkspace)");
-#ifdef ENV_COPY
+    Yap_Error(INTERNAL_ERROR, TermNil, "excessive number of workers");
+#ifdef YAPOR_COPY
   INFORMATION_MESSAGE("YapOr: copy model with %d worker%s", n_workers, n_workers == 1 ? "":"s");
-#elif ACOW
+#elif YAPOR_COW
   INFORMATION_MESSAGE("YapOr: acow model with %d worker%s", n_workers, n_workers == 1 ? "":"s");
-#else /* SBA */
+#elif YAPOR_SBA
   INFORMATION_MESSAGE("YapOr: sba model with %d worker%s", n_workers, n_workers == 1 ? "":"s");
-#endif /* ENV_COPY - ACOW - SBA */
-  map_memory(Heap, Stack+Atts, Trail, n_workers);
+#elif YAPOR_THREADS
+  INFORMATION_MESSAGE("YapOr: threads model with %d worker%s", n_workers, n_workers == 1 ? "":"s");
+#endif /* YAPOR_COPY - YAPOR_COW - YAPOR_SBA - YAPOR_THREADS */
+#endif /* YAPOR */
+#if defined(YAPOR_COPY) || defined(YAPOR_COW) || defined(YAPOR_SBA)
+  Yap_init_yapor_stacks_memory(Trail, Heap, Stack+Atts, n_workers);
 #else
-  Yap_InitMemory (Trail, Heap, Stack+Atts);
-#endif /* YAPOR && !THREADS */
+  Yap_InitMemory(Trail, Heap, Stack+Atts);
+#endif
 #if defined(YAPOR) || defined(TABLING)
-  Yap_init_global(max_table_size, n_workers, sch_loop, delay_load);
+  Yap_init_global_optyap_data(max_table_size, n_workers, sch_loop, delay_load);
 #endif /* YAPOR || TABLING */
-  Yap_AttsSize = Atts;
 
-  Yap_InitTime ();
+  Yap_AttsSize = Atts;
+  Yap_InitTime( 0 );
   /* InitAbsmi must be done before InitCodes */
   /* This must be done before initialising predicates */
-  for (i = 0; i <= LAST_FLAG; i++) {
+  for (i = 0; i < NUMBER_OF_YAP_FLAGS; i++) {
     yap_flags[i] = 0;
   }
 #ifdef MPW
@@ -1377,17 +1350,17 @@ Yap_HaltRegisterHook (HaltHookFunc f, void * env)
     return FALSE;
   h->environment = env;
   h->hook = f;
-  LOCK(BGL);
-  h->next = Yap_HaltHooks;
-  Yap_HaltHooks = h;
-  UNLOCK(BGL);
+  LOCK(GLOBAL_BGL);
+  h->next = GLOBAL_HaltHooks;
+  GLOBAL_HaltHooks = h;
+  UNLOCK(GLOBAL_BGL);
   return TRUE;
 }
 
 static void
 run_halt_hooks(int code)
 {
-  struct halt_hook *hooke = Yap_HaltHooks;
+  struct halt_hook *hooke = GLOBAL_HaltHooks;
 
   while (hooke) {
     hooke->hook(code, hooke->environment);
@@ -1398,11 +1371,13 @@ run_halt_hooks(int code)
 void
 Yap_exit (int value)
 {
-#if defined(YAPOR) && !defined(THREADS)
-  unmap_memory();
-#endif /* YAPOR */
+  CACHE_REGS
+  void closeFiles(int all);
+#if defined(YAPOR_COPY) || defined(YAPOR_COW) || defined(YAPOR_SBA)
+  Yap_unmap_yapor_memory();
+#endif /* YAPOR_COPY || YAPOR_COW || YAPOR_SBA */
 
-  if (! (Yap_PrologMode & BootMode) ) {
+  if (! (LOCAL_PrologMode & BootMode) ) {
 #ifdef LOW_PROF
     remove("PROFPREDS");
     remove("PROFILING");
@@ -1413,6 +1388,7 @@ Yap_exit (int value)
     run_halt_hooks(value);
     Yap_ShutdownLoadForeign();
   }
+  closeFiles(TRUE);
   exit(value);
 }
 
