@@ -68,16 +68,16 @@ typedef struct jmp_buff_struct {
 	sigjmp_buf JmpBuff;
 } JMPBUFF;
 
-STATIC_PROTO(void GNextToken, (void));
-STATIC_PROTO(void checkfor, (Term, JMPBUFF *));
-STATIC_PROTO(Term ParseArgs, (Atom, JMPBUFF *));
-STATIC_PROTO(Term ParseList, (JMPBUFF *));
-STATIC_PROTO(Term ParseTerm, (int, JMPBUFF *));
+STATIC_PROTO(void GNextToken, ( CACHE_TYPE1 ));
+STATIC_PROTO(void checkfor, (Term, JMPBUFF * CACHE_TYPE));
+STATIC_PROTO(Term ParseArgs, (Atom, JMPBUFF * CACHE_TYPE));
+STATIC_PROTO(Term ParseList, (JMPBUFF * CACHE_TYPE));
+STATIC_PROTO(Term ParseTerm, (int, JMPBUFF * CACHE_TYPE));
 
 
 #define TRY(S,P)                               \
   {	Volatile JMPBUFF *saveenv, newenv;     \
-	Volatile TokEntry *saveT=Yap_tokptr;   \
+	Volatile TokEntry *saveT=LOCAL_tokptr;   \
         Volatile CELL *saveH=H;                \
         Volatile int savecurprio=curprio;      \
         saveenv=FailBuff;                      \
@@ -90,13 +90,13 @@ STATIC_PROTO(Term ParseTerm, (int, JMPBUFF *));
 	else { FailBuff=saveenv;               \
 		H=saveH;                       \
 		curprio = savecurprio;         \
-                Yap_tokptr=saveT;              \
+                LOCAL_tokptr=saveT;              \
 	}                                      \
    }
 
 #define TRY3(S,P,F)                            \
   {	Volatile JMPBUFF *saveenv, newenv;     \
-	Volatile TokEntry *saveT=Yap_tokptr;   \
+	Volatile TokEntry *saveT=LOCAL_tokptr;   \
         Volatile CELL *saveH=H;                \
         saveenv=FailBuff;                      \
         if(!sigsetjmp(newenv.JmpBuff, 0)) {      \
@@ -108,7 +108,7 @@ STATIC_PROTO(Term ParseTerm, (int, JMPBUFF *));
 	else {                                 \
                 FailBuff=saveenv;              \
                 H=saveH;                       \
-                Yap_tokptr=saveT;              \
+                LOCAL_tokptr=saveT;              \
                 F }                            \
    }
 
@@ -118,18 +118,19 @@ STATIC_PROTO(Term ParseTerm, (int, JMPBUFF *));
 VarEntry *
 Yap_LookupVar(char *var)	/* lookup variable in variables table   */
 {
+  CACHE_REGS
   VarEntry *p;
 
 #ifdef DEBUG
-  if (Yap_Option[4])
-    fprintf(Yap_stderr,"[LookupVar %s]", var);
+  if (GLOBAL_Option[4])
+    fprintf(GLOBAL_stderr,"[LookupVar %s]", var);
 #endif
   if (var[0] != '_' || var[1] != '\0') {
-    VarEntry **op = &Yap_VarTable;
+    VarEntry **op = &LOCAL_VarTable;
     unsigned char *vp = (unsigned char *)var;
     UInt hv;
 
-    p = Yap_VarTable;
+    p = LOCAL_VarTable;
     hv = HashFunction(vp) % AtomHashTableSize;
     while (p != NULL) {
       CELL hpv = p->hv;
@@ -160,8 +161,8 @@ Yap_LookupVar(char *var)	/* lookup variable in variables table   */
   } else {
     /* anon var */
     p = (VarEntry *) Yap_AllocScannerMemory(sizeof(VarEntry) + 2);
-    p->VarLeft = Yap_AnonVarTable;
-    Yap_AnonVarTable = p;
+    p->VarLeft = LOCAL_AnonVarTable;
+    LOCAL_AnonVarTable = p;
     p->VarRight = NULL;    
     p->hv = 0L;
     p->VarRep[0] = '_';
@@ -172,20 +173,25 @@ Yap_LookupVar(char *var)	/* lookup variable in variables table   */
 }
 
 static Term
-VarNames(VarEntry *p,Term l)
+VarNames(VarEntry *p,Term l USES_REGS)
 {
   if (p != NULL) {
     if (strcmp(p->VarRep, "_") != 0) {
-      Term o = MkPairTerm(MkPairTerm(Yap_StringToList(p->VarRep), p->VarAdr),
-			  VarNames(p->VarRight,
-				   VarNames(p->VarLeft,l)));
+      Term t[2];
+      Term o;
+      
+      t[0] = MkAtomTerm(Yap_LookupAtom(p->VarRep));
+      t[1] = p->VarAdr;
+      o = Yap_MkApplTerm(FunctorEq, 2, t);
+      o = MkPairTerm(o, VarNames(p->VarRight,
+				 VarNames(p->VarLeft,l PASS_REGS) PASS_REGS));
       if (H > ASP-4096) {
 	save_machine_regs();
-	siglongjmp(Yap_IOBotch,1);
+	siglongjmp(LOCAL_IOBotch,1);
       }  
       return(o);
     } else {
-      return(VarNames(p->VarRight,VarNames(p->VarLeft,l)));
+      return VarNames(p->VarRight,VarNames(p->VarLeft,l PASS_REGS) PASS_REGS);
     }
   } else {
     return (l);
@@ -195,20 +201,23 @@ VarNames(VarEntry *p,Term l)
 Term
 Yap_VarNames(VarEntry *p,Term l)
 {
-  return VarNames(p,l);
+  CACHE_REGS
+  return VarNames(p,l PASS_REGS);
 }
 
 static int
-IsPrefixOp(Atom op,int  *pptr, int *rpptr)
+IsPrefixOp(Atom op,int  *pptr, int *rpptr USES_REGS)
 {
   int p;
 
-  OpEntry *opp = Yap_GetOpProp(op, PREFIX_OP);
+  OpEntry *opp = Yap_GetOpProp(op, PREFIX_OP PASS_REGS);
   if (!opp)
     return FALSE;
   if (opp->OpModule &&
-      opp->OpModule != CurrentModule)
+      opp->OpModule != CurrentModule) {
+    READ_UNLOCK(opp->OpRWLock);
     return FALSE;
+  }
   if ((p = opp->Prefix) != 0) {
     READ_UNLOCK(opp->OpRWLock);
     *pptr = *rpptr = p & MaskPrio;
@@ -224,20 +233,23 @@ IsPrefixOp(Atom op,int  *pptr, int *rpptr)
 int
 Yap_IsPrefixOp(Atom op,int  *pptr, int *rpptr)
 {
-  return IsPrefixOp(op,pptr,rpptr);
+  CACHE_REGS
+  return IsPrefixOp(op,pptr,rpptr PASS_REGS);
 }
 
 static int
-IsInfixOp(Atom op, int *pptr, int *lpptr, int *rpptr)
+IsInfixOp(Atom op, int *pptr, int *lpptr, int *rpptr USES_REGS)
 {
   int p;
 
-  OpEntry *opp = Yap_GetOpProp(op, INFIX_OP);
+  OpEntry *opp = Yap_GetOpProp(op, INFIX_OP PASS_REGS);
   if (!opp)
     return FALSE;
   if (opp->OpModule &&
-      opp->OpModule != CurrentModule)
+      opp->OpModule != CurrentModule) {
+    READ_UNLOCK(opp->OpRWLock);
     return FALSE;
+  }
   if ((p = opp->Infix) != 0) {
     READ_UNLOCK(opp->OpRWLock);
     *pptr = *rpptr = *lpptr = p & MaskPrio;
@@ -255,20 +267,23 @@ IsInfixOp(Atom op, int *pptr, int *lpptr, int *rpptr)
 int
 Yap_IsInfixOp(Atom op, int *pptr, int *lpptr, int *rpptr)
 {
-  return IsInfixOp(op, pptr, lpptr, rpptr);
+  CACHE_REGS
+  return IsInfixOp(op, pptr, lpptr, rpptr PASS_REGS);
 }
 
 static int
-IsPosfixOp(Atom op, int *pptr, int *lpptr)
+IsPosfixOp(Atom op, int *pptr, int *lpptr USES_REGS)
 {
   int p;
 
-  OpEntry *opp = Yap_GetOpProp(op, POSFIX_OP);
+  OpEntry *opp = Yap_GetOpProp(op, POSFIX_OP PASS_REGS);
   if (!opp)
     return FALSE;
   if (opp->OpModule &&
-      opp->OpModule != CurrentModule)
+      opp->OpModule != CurrentModule) {
+    READ_UNLOCK(opp->OpRWLock);
     return FALSE;
+  }
   if ((p = opp->Posfix) != 0) {
     READ_UNLOCK(opp->OpRWLock);
     *pptr = *lpptr = p & MaskPrio;
@@ -284,36 +299,32 @@ IsPosfixOp(Atom op, int *pptr, int *lpptr)
 int
 Yap_IsPosfixOp(Atom op, int *pptr, int *lpptr)
 {
-  return IsPosfixOp(op, pptr, lpptr);
+  CACHE_REGS
+  return IsPosfixOp(op, pptr, lpptr PASS_REGS);
 }
 
 inline static void
-GNextToken(void)
+GNextToken( USES_REGS1 )
 {
-	if (Yap_tokptr->Tok == Ord(eot_tok))
+	if (LOCAL_tokptr->Tok == Ord(eot_tok))
 		return;
-#ifdef EMACS
-	if ((Yap_tokptr = Yap_tokptr->TokNext)->TokPos > Yap_toktide->TokPos)
-		Yap_toktide = Yap_tokptr;
-#else
-	if (Yap_tokptr == Yap_toktide)
-	  Yap_toktide = Yap_tokptr = Yap_tokptr->TokNext;
+	if (LOCAL_tokptr == LOCAL_toktide)
+	  LOCAL_toktide = LOCAL_tokptr = LOCAL_tokptr->TokNext;
 	else
-	  Yap_tokptr = Yap_tokptr->TokNext;
-#endif
+	  LOCAL_tokptr = LOCAL_tokptr->TokNext;
 }
 
 inline static void
-checkfor(Term c, JMPBUFF *FailBuff)
+checkfor(Term c, JMPBUFF *FailBuff USES_REGS)
 {
-  if (Yap_tokptr->Tok != Ord(Ponctuation_tok)
-      || Yap_tokptr->TokInfo != c)
+  if (LOCAL_tokptr->Tok != Ord(Ponctuation_tok)
+      || LOCAL_tokptr->TokInfo != c)
     FAIL;
   NextToken;
 }
 
 static Term
-ParseArgs(Atom a, JMPBUFF *FailBuff)
+ParseArgs(Atom a, JMPBUFF *FailBuff USES_REGS)
 {
   int nargs = 0;
   Term *p, t;
@@ -326,16 +337,16 @@ ParseArgs(Atom a, JMPBUFF *FailBuff)
   p = (Term *) ParserAuxSp;
   while (1) {
     Term *tp = (Term *)ParserAuxSp;
-    if (ParserAuxSp+1 > Yap_TrailTop) {
-      Yap_ErrorMessage = "Trail Overflow";
+    if (ParserAuxSp+1 > LOCAL_TrailTop) {
+      LOCAL_ErrorMessage = "Trail Overflow";
       FAIL;
     }
-    *tp++ = Unsigned(ParseTerm(999, FailBuff));
+    *tp++ = Unsigned(ParseTerm(999, FailBuff PASS_REGS));
     ParserAuxSp = (char *)tp;
     ++nargs;
-    if (Yap_tokptr->Tok != Ord(Ponctuation_tok))
+    if (LOCAL_tokptr->Tok != Ord(Ponctuation_tok))
       break;
-    if (((int) Yap_tokptr->TokInfo) != ',')
+    if (((int) LOCAL_tokptr->TokInfo) != ',')
       break;
     NextToken;
   }
@@ -345,12 +356,12 @@ ParseArgs(Atom a, JMPBUFF *FailBuff)
    * order 
    */
   if (H > ASP-(nargs+1)) {
-    Yap_ErrorMessage = "Stack Overflow";
+    LOCAL_ErrorMessage = "Stack Overflow";
     FAIL;
   }  
   func = Yap_MkFunctor(a, nargs);
   if (func == NULL) {
-    Yap_ErrorMessage = "Heap Overflow";
+    LOCAL_ErrorMessage = "Heap Overflow";
     FAIL;
   }
 #ifdef SFUNC
@@ -365,17 +376,17 @@ ParseArgs(Atom a, JMPBUFF *FailBuff)
     t = Yap_MkApplTerm(func, nargs, p);
 #endif
   if (H > ASP-4096) {
-    Yap_ErrorMessage = "Stack Overflow";
+    LOCAL_ErrorMessage = "Stack Overflow";
     return TermNil;
   }  
   /* check for possible overflow against local stack */
-  checkfor((Term) ')', FailBuff);
+  checkfor((Term) ')', FailBuff PASS_REGS);
   return t;
 }
 
 
 static Term
-ParseList(JMPBUFF *FailBuff)
+ParseList(JMPBUFF *FailBuff USES_REGS)
 {
   Term o;
   CELL *to_store;
@@ -383,28 +394,28 @@ ParseList(JMPBUFF *FailBuff)
  loop:
   to_store = H;
   H+=2;
-  to_store[0] = ParseTerm(999, FailBuff);
-  if (Yap_tokptr->Tok == Ord(Ponctuation_tok)) {
-    if (((int) Yap_tokptr->TokInfo) == ',') {
+  to_store[0] = ParseTerm(999, FailBuff PASS_REGS);
+  if (LOCAL_tokptr->Tok == Ord(Ponctuation_tok)) {
+    if (((int) LOCAL_tokptr->TokInfo) == ',') {
       NextToken;
-      if (Yap_tokptr->Tok == Ord(Name_tok)
-	  && strcmp(RepAtom((Atom)(Yap_tokptr->TokInfo))->StrOfAE, "..") == 0) {
+      if (LOCAL_tokptr->Tok == Ord(Name_tok)
+	  && strcmp(RepAtom((Atom)(LOCAL_tokptr->TokInfo))->StrOfAE, "..") == 0) {
 	NextToken;
-	to_store[1] = ParseTerm(999, FailBuff);
+	to_store[1] = ParseTerm(999, FailBuff PASS_REGS);
       } else {
 	/* check for possible overflow against local stack */
 	if (H > ASP-4096) {
 	  to_store[1] = TermNil;
-	  Yap_ErrorMessage = "Stack Overflow";
+	  LOCAL_ErrorMessage = "Stack Overflow";
 	  FAIL;
 	} else {
 	  to_store[1] = AbsPair(H);
 	  goto loop;
 	}
       }
-    } else if (((int) Yap_tokptr->TokInfo) == '|') {
+    } else if (((int) LOCAL_tokptr->TokInfo) == '|') {
       NextToken;
-      to_store[1] = ParseTerm(999, FailBuff);
+      to_store[1] = ParseTerm(999, FailBuff PASS_REGS);
     } else {
       to_store[1] = MkAtomTerm(AtomNil);
     }
@@ -422,7 +433,7 @@ ParseList(JMPBUFF *FailBuff)
 #endif
 
 static Term
-ParseTerm(int prio, JMPBUFF *FailBuff)
+ParseTerm(int prio, JMPBUFF *FailBuff USES_REGS)
 {
   /* parse term with priority prio */
   Volatile Term t;
@@ -431,72 +442,73 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
   Volatile int curprio = 0, opprio, oplprio, oprprio;
   Volatile Atom opinfo;
 
-  switch (Yap_tokptr->Tok) {
+  switch (LOCAL_tokptr->Tok) {
   case Name_tok:
-    t = Yap_tokptr->TokInfo;
+    t = LOCAL_tokptr->TokInfo;
     NextToken;
-    if ((Yap_tokptr->Tok != Ord(Ponctuation_tok)
-	 || Unsigned(Yap_tokptr->TokInfo) != 'l')
-	&& IsPrefixOp((Atom)t, &opprio, &oprprio)
-	) {
-      /* special rules apply for +1, -2.3, etc... */
-      if (Yap_tokptr->Tok == Number_tok) {
-	if ((Atom)t == AtomMinus) {
-	  t = Yap_tokptr->TokInfo;
-	  if (IsIntTerm(t))
-	    t = MkIntTerm(-IntOfTerm(t));
-	  else if (IsFloatTerm(t))
-	    t = MkFloatTerm(-FloatOfTerm(t));
+    /* special rules apply for +1, -2.3, etc... */
+    if (LOCAL_tokptr->Tok == Number_tok) {
+      if ((Atom)t == AtomMinus) {
+	t = LOCAL_tokptr->TokInfo;
+	if (IsIntTerm(t))
+	  t = MkIntTerm(-IntOfTerm(t));
+	else if (IsFloatTerm(t))
+	  t = MkFloatTerm(-FloatOfTerm(t));
 #ifdef USE_GMP
-	  else if (IsBigIntTerm(t)) {
-	    t = Yap_gmp_neg_big(t);
-	  }
-#endif
-	  else
-	    t = MkLongIntTerm(-LongIntOfTerm(t));
-	  NextToken;
-	  break;
-	}
-      } else if (Yap_tokptr->Tok == Name_tok) {
-	Atom at = (Atom)Yap_tokptr->TokInfo;
-#ifndef _MSC_VER
-	if ((Atom)t == AtomPlus) {
-	  if (at == AtomInf) {
-	    t = MkFloatTerm(INFINITY);
-	    NextToken;
-	    break;
-	  } else if (at == AtomNan) {
-	    t = MkFloatTerm(NAN);
-	    NextToken;
-	    break;
-	  }
-	} else if ((Atom)t == AtomMinus) {
-	  if (at == AtomInf) {
-	    t = MkFloatTerm(-INFINITY);
-	    NextToken;
-	    break;
-	  } else if (at == AtomNan) {
-	    t = MkFloatTerm(NAN);
-	    NextToken;
-	    break;
-	  }
+	else if (IsBigIntTerm(t)) {
+	  t = Yap_gmp_neg_big(t);
 	}
 #endif
+	else
+	  t = MkLongIntTerm(-LongIntOfTerm(t));
+	NextToken;
+	break;
       }
+    }
+    if ((LOCAL_tokptr->Tok != Ord(Ponctuation_tok)
+	 || Unsigned(LOCAL_tokptr->TokInfo) != 'l')
+	&& IsPrefixOp((Atom)t, &opprio, &oprprio PASS_REGS)
+	) {
+	if (LOCAL_tokptr->Tok == Name_tok) {
+	  Atom at = (Atom)LOCAL_tokptr->TokInfo;
+#ifndef _MSC_VER
+	  if ((Atom)t == AtomPlus) {
+	    if (at == AtomInf) {
+	      t = MkFloatTerm(INFINITY);
+	      NextToken;
+	    break;
+	    } else if (at == AtomNan) {
+	      t = MkFloatTerm(NAN);
+	      NextToken;
+	      break;
+	    }
+	  } else if ((Atom)t == AtomMinus) {
+	    if (at == AtomInf) {
+	      t = MkFloatTerm(-INFINITY);
+	      NextToken;
+	      break;
+	    } else if (at == AtomNan) {
+	      t = MkFloatTerm(NAN);
+	      NextToken;
+	      break;
+	    }
+	  }
+#endif
+	}
       if (opprio <= prio) {
       /* try to parse as a prefix operator */
 	TRY(
 	  /* build appl on the heap */
 	  func = Yap_MkFunctor((Atom) t, 1);
 	  if (func == NULL) {
-	    Yap_ErrorMessage = "Heap Overflow";
+	    LOCAL_ErrorMessage = "Heap Overflow";
 	    FAIL;
 	  }
-	  t = ParseTerm(oprprio, FailBuff);
+	  t = ParseTerm(oprprio, FailBuff PASS_REGS);
 	  t = Yap_MkApplTerm(func, 1, &t);
 	  /* check for possible overflow against local stack */
 	  if (H > ASP-4096) {
-	    Yap_ErrorMessage = "Stack Overflow";
+	    LOCAL_ErrorMessage = "Stack Overflow";
 	    FAIL;
 	  }  
 	  curprio = opprio;
@@ -505,21 +517,21 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
 	  )
 	}
     }
-    if (Yap_tokptr->Tok == Ord(Ponctuation_tok)
-	&& Unsigned(Yap_tokptr->TokInfo) == 'l')
-      t = ParseArgs((Atom) t, FailBuff);
+    if (LOCAL_tokptr->Tok == Ord(Ponctuation_tok)
+	&& Unsigned(LOCAL_tokptr->TokInfo) == 'l')
+      t = ParseArgs((Atom) t, FailBuff PASS_REGS);
     else
       t = MkAtomTerm((Atom)t);
     break;
 
   case Number_tok:
-    t = Yap_tokptr->TokInfo;
+    t = LOCAL_tokptr->TokInfo;
     NextToken;
     break;
 
   case String_tok:	/* build list on the heap */
     {
-      Volatile char *p = (char *) Yap_tokptr->TokInfo;
+      Volatile char *p = (char *) LOCAL_tokptr->TokInfo;
       if (*p == 0)
 	t = MkAtomTerm(AtomNil);
       else if (yap_flags[YAP_DOUBLE_QUOTES_FLAG] == STRING_AS_CHARS)
@@ -527,7 +539,7 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
       else if (yap_flags[YAP_DOUBLE_QUOTES_FLAG] == STRING_AS_ATOM) {
 	Atom at = Yap_LookupAtom(p);
 	if (at == NIL) {
-	  Yap_ErrorMessage = "Heap Overflow";
+	  LOCAL_ErrorMessage = "Heap Overflow";
 	  FAIL;	  
 	}
 	t = MkAtomTerm(at);
@@ -539,7 +551,7 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
 
   case WString_tok:	/* build list on the heap */
     {
-      Volatile wchar_t *p = (wchar_t *) Yap_tokptr->TokInfo;
+      Volatile wchar_t *p = (wchar_t *) LOCAL_tokptr->TokInfo;
       if (*p == 0)
 	t = MkAtomTerm(AtomNil);
       else if (yap_flags[YAP_DOUBLE_QUOTES_FLAG] == STRING_AS_CHARS)
@@ -548,12 +560,16 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
 	t = MkAtomTerm(Yap_LookupWideAtom(p));
       else
 	t = Yap_WideStringToList(p);
+      if (t == 0L) {
+	LOCAL_ErrorMessage = "Stack Overflow";
+	FAIL;
+      }
       NextToken;
     }
   break;
 
   case Var_tok:
-    varinfo = (VarEntry *) (Yap_tokptr->TokInfo);
+    varinfo = (VarEntry *) (LOCAL_tokptr->TokInfo);
     if ((t = varinfo->VarAdr) == TermNil) {
       t = varinfo->VarAdr = MkVarTerm();
     }
@@ -564,28 +580,40 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
     FAIL;
 
   case Ponctuation_tok:
-    switch ((int) Yap_tokptr->TokInfo) {
+    switch ((int) LOCAL_tokptr->TokInfo) {
     case '(':
     case 'l':	/* non solo ( */
       NextToken;
-      t = ParseTerm(1200, FailBuff);
-      checkfor((Term) ')', FailBuff);
+      t = ParseTerm(1200, FailBuff PASS_REGS);
+      checkfor((Term) ')', FailBuff PASS_REGS);
       break;
     case '[':
       NextToken;
-      t = ParseList(FailBuff);
-      checkfor((Term) ']', FailBuff);
+      if (LOCAL_tokptr->Tok == Ponctuation_tok &&
+	  (int) LOCAL_tokptr->TokInfo == ']') {
+	t = TermNil;
+	NextToken;
+	break;
+      }
+      t = ParseList(FailBuff PASS_REGS);
+      checkfor((Term) ']', FailBuff PASS_REGS);
       break;
     case '{':
       NextToken;
-      t = ParseTerm(1200, FailBuff);
+      if (LOCAL_tokptr->Tok == Ponctuation_tok &&
+	  (int) LOCAL_tokptr->TokInfo == '}') {
+	t = MkAtomTerm(AtomBraces);
+	NextToken;
+	break;
+      }
+      t = ParseTerm(1200, FailBuff PASS_REGS);
       t = Yap_MkApplTerm(FunctorBraces, 1, &t);
       /* check for possible overflow against local stack */
       if (H > ASP-4096) {
-	Yap_ErrorMessage = "Stack Overflow";
+	LOCAL_ErrorMessage = "Stack Overflow";
 	FAIL;
       }  
-      checkfor((Term) '}', FailBuff);
+      checkfor((Term) '}', FailBuff PASS_REGS);
       break;
     default:
       FAIL;
@@ -599,28 +627,28 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
 
   /* main loop to parse infix and posfix operators starts here */
   while (TRUE) {
-    if (Yap_tokptr->Tok == Ord(Name_tok)
-	&& Yap_HasOp((Atom)(Yap_tokptr->TokInfo))) {
-      Atom save_opinfo = opinfo = (Atom)(Yap_tokptr->TokInfo);
-      if (IsInfixOp(save_opinfo, &opprio, &oplprio, &oprprio) 
+    if (LOCAL_tokptr->Tok == Ord(Name_tok)
+	&& Yap_HasOp((Atom)(LOCAL_tokptr->TokInfo))) {
+      Atom save_opinfo = opinfo = (Atom)(LOCAL_tokptr->TokInfo);
+      if (IsInfixOp(save_opinfo, &opprio, &oplprio, &oprprio PASS_REGS) 
 	  && opprio <= prio && oplprio >= curprio) {
 	/* try parsing as infix operator */
 	Volatile int oldprio = curprio;
 	TRY3(
-	     func = Yap_MkFunctor((Atom) Yap_tokptr->TokInfo, 2);
+	     func = Yap_MkFunctor((Atom) LOCAL_tokptr->TokInfo, 2);
 	     if (func == NULL) {
-	       Yap_ErrorMessage = "Heap Overflow";
+	       LOCAL_ErrorMessage = "Heap Overflow";
 	       FAIL;
 	     }
 	     NextToken;
 	     {
 	       Term args[2];
 	       args[0] = t;
-	       args[1] = ParseTerm(oprprio, FailBuff);
+	       args[1] = ParseTerm(oprprio, FailBuff PASS_REGS);
 	       t = Yap_MkApplTerm(func, 2, args);
 	       /* check for possible overflow against local stack */
 	       if (H > ASP-4096) {
-		 Yap_ErrorMessage = "Stack Overflow";
+		 LOCAL_ErrorMessage = "Stack Overflow";
 		 FAIL;
 	       }  
 	     },
@@ -632,18 +660,18 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
 	       curprio = oldprio;
 	     )
       }
-      if (IsPosfixOp(opinfo, &opprio, &oplprio)
+      if (IsPosfixOp(opinfo, &opprio, &oplprio PASS_REGS)
 	  && opprio <= prio && oplprio >= curprio) {
 	/* parse as posfix operator */
-	Functor func = Yap_MkFunctor((Atom) Yap_tokptr->TokInfo, 1);
+	Functor func = Yap_MkFunctor((Atom) LOCAL_tokptr->TokInfo, 1);
 	if (func == NULL) {
-	  Yap_ErrorMessage = "Heap Overflow";
+	  LOCAL_ErrorMessage = "Heap Overflow";
 	  FAIL;
 	}
 	t = Yap_MkApplTerm(func, 1, &t);
 	/* check for possible overflow against local stack */
 	if (H > ASP-4096) {
-	  Yap_ErrorMessage = "Stack Overflow";
+	  LOCAL_ErrorMessage = "Stack Overflow";
 	  FAIL;
 	}  
 	curprio = opprio;
@@ -652,48 +680,48 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
       }
       break;
     }
-    if (Yap_tokptr->Tok == Ord(Ponctuation_tok)) {
-      if (Unsigned(Yap_tokptr->TokInfo) == ',' &&
+    if (LOCAL_tokptr->Tok == Ord(Ponctuation_tok)) {
+      if (Unsigned(LOCAL_tokptr->TokInfo) == ',' &&
 	  prio >= 1000 && curprio <= 999) {
 	Volatile Term args[2];
 	NextToken;
 	args[0] = t;
-	args[1] = ParseTerm(1000, FailBuff);
+	args[1] = ParseTerm(1000, FailBuff PASS_REGS);
 	t = Yap_MkApplTerm(FunctorComma, 2, args);
 	/* check for possible overflow against local stack */
 	if (H > ASP-4096) {
-	  Yap_ErrorMessage = "Stack Overflow";
+	  LOCAL_ErrorMessage = "Stack Overflow";
 	  FAIL;
 	}  
 	curprio = 1000;
 	continue;
-      } else if (Unsigned(Yap_tokptr->TokInfo) == '|' &&
-                IsInfixOp(AtomVBar, &opprio, &oplprio, &oprprio)
+      } else if (Unsigned(LOCAL_tokptr->TokInfo) == '|' &&
+                IsInfixOp(AtomVBar, &opprio, &oplprio, &oprprio PASS_REGS)
                 && opprio <= prio && oplprio >= curprio) {
 	Volatile Term args[2];
 	NextToken;
 	args[0] = t;
-	args[1] = ParseTerm(oprprio, FailBuff);
+	args[1] = ParseTerm(oprprio, FailBuff PASS_REGS);
 	t = Yap_MkApplTerm(FunctorVBar, 2, args);
 	/* check for possible overflow against local stack */
 	if (H > ASP-4096) {
-	  Yap_ErrorMessage = "Stack Overflow";
+	  LOCAL_ErrorMessage = "Stack Overflow";
 	  FAIL;
 	}  
 	curprio = opprio;
 	continue;
       }
     }
-    if (Yap_tokptr->Tok <= Ord(WString_tok))
+    if (LOCAL_tokptr->Tok <= Ord(WString_tok))
       FAIL;
     break;
   }
 #ifdef DEBUG
-  if (Yap_Option['p' - 'a' + 1]) {
-    Yap_DebugPutc(Yap_c_error_stream,'[');
+  if (GLOBAL_Option['p' - 'a' + 1]) {
+    Yap_DebugPutc(LOCAL_c_error_stream,'[');
     Yap_DebugPlWrite(t);
-    Yap_DebugPutc(Yap_c_error_stream,']');
-    Yap_DebugPutc(Yap_c_error_stream,'\n');
+    Yap_DebugPutc(LOCAL_c_error_stream,']');
+    Yap_DebugPutc(LOCAL_c_error_stream,'\n');
   }
 #endif
   return t;
@@ -703,12 +731,13 @@ ParseTerm(int prio, JMPBUFF *FailBuff)
 Term
 Yap_Parse(void)
 {
+  CACHE_REGS
   Volatile Term t;
   JMPBUFF FailBuff;
 
   if (!sigsetjmp(FailBuff.JmpBuff, 0)) {
-    t = ParseTerm(1200, &FailBuff);
-    if (Yap_tokptr->Tok != Ord(eot_tok))
+    t = ParseTerm(1200, &FailBuff PASS_REGS);
+    if (LOCAL_tokptr->Tok != Ord(eot_tok))
       return (0L);
     return (t);
   } else
