@@ -8,7 +8,7 @@
 *									 *
 **************************************************************************
 *									 *
-* File:		%W% %G%						 *
+* File:		%W% %G%						         *
 * Last rev:	22-1-03							 *
 * mods:									 *
 * comments:	Prolog's scanner					 *
@@ -35,6 +35,7 @@
 #include "Yap.h"
 #include "Yatom.h"
 #include "YapHeap.h"
+#include "SWI-Stream.h"
 #include "yapio.h"
 #include "alloc.h"
 #include "eval.h"
@@ -58,9 +59,8 @@
 #define my_isupper(C)	( C >= 'A' && C <= 'Z' )
 #define my_islower(C)	( C >= 'a' && C <= 'z' )
 
-STATIC_PROTO(int my_getch, (int (*) (int)));
 STATIC_PROTO(Term float_send, (char *, int));
-STATIC_PROTO(Term get_num, (int *, int *, int, int (*) (int), int (*) (int),char *,UInt,int));
+STATIC_PROTO(Term get_num, (int *, int *, IOSTREAM *,char *,UInt,int));
 
 /* token table with some help from Richard O'Keefe's PD scanner */
 static char chtype0[NUMBER_OF_CHARS+1] =
@@ -139,6 +139,27 @@ Yap_wide_chtype(Int ch) {
 }
 
 
+static inline int
+getchr__(IOSTREAM *inp)
+{ int c = Sgetcode(inp);
+
+  if ( !CharConversionTable || c < 0 || c >= 256 )
+    return c;
+
+  return CharConversionTable[c];
+}
+
+
+#define getchr(inp)  getchr__(inp)
+#define getchrq(inp) Sgetcode(inp)
+
+EXTERN inline int
+GetCurInpPos (IOSTREAM *inp_stream)
+{
+  return inp_stream->posbuf.lineno;
+}
+
+
 
 /* in case there is an overflow */
 typedef struct scanner_extra_alloc {
@@ -149,20 +170,21 @@ typedef struct scanner_extra_alloc {
 static char *
 AllocScannerMemory(unsigned int size)
 {
+  CACHE_REGS
   char *AuxSpScan;
 
-  AuxSpScan = ScannerStack;
+  AuxSpScan = LOCAL_ScannerStack;
   size = AdjustSize(size);
-  if (ScannerExtraBlocks) {
+  if (LOCAL_ScannerExtraBlocks) {
     struct scanner_extra_alloc *ptr;
 
     if (!(ptr = (struct scanner_extra_alloc *)malloc(size+sizeof(ScannerExtraBlock)))) {
       return NULL;
     }
-    ptr->next = ScannerExtraBlocks;
-    ScannerExtraBlocks = ptr;
+    ptr->next = LOCAL_ScannerExtraBlocks;
+    LOCAL_ScannerExtraBlocks = ptr;
     return (char *)(ptr+1);
-  } else if (Yap_TrailTop <= AuxSpScan+size) {
+  } else if (LOCAL_TrailTop <= AuxSpScan+size) {
     UInt alloc_size = sizeof(CELL) * K16;
  
     if (size > alloc_size)
@@ -173,24 +195,25 @@ AllocScannerMemory(unsigned int size)
       if (!(ptr = (struct scanner_extra_alloc *)malloc(size+sizeof(ScannerExtraBlock)))) {
 	return NULL;
       }
-      ptr->next = ScannerExtraBlocks;
-      ScannerExtraBlocks = ptr;
+      ptr->next = LOCAL_ScannerExtraBlocks;
+      LOCAL_ScannerExtraBlocks = ptr;
       return (char *)(ptr+1);
     }
   }
-  ScannerStack = AuxSpScan+size;
+  LOCAL_ScannerStack = AuxSpScan+size;
   return AuxSpScan;
 }
 
 static void
 PopScannerMemory(char *block, unsigned int size)
 {
-  if (block == ScannerStack-size) {
-    ScannerStack -= size;
-  } else if (block == (char *)(ScannerExtraBlocks+1)) {
-    struct scanner_extra_alloc *ptr = ScannerExtraBlocks;
+  CACHE_REGS
+  if (block == LOCAL_ScannerStack-size) {
+    LOCAL_ScannerStack -= size;
+  } else if (block == (char *)(LOCAL_ScannerExtraBlocks+1)) {
+    struct scanner_extra_alloc *ptr = LOCAL_ScannerExtraBlocks;
 
-    ScannerExtraBlocks = ptr->next;
+    LOCAL_ScannerExtraBlocks = ptr->next;
     free(ptr);
   }
 }
@@ -202,17 +225,6 @@ Yap_AllocScannerMemory(unsigned int size)
   return AllocScannerMemory(size);
 }
 
-inline static int
-my_getch(int (*Nextch) (int))
-{
-  int ch = (*Nextch) (Yap_c_input_stream); 
-#ifdef DEBUG
-  if (Yap_Option[1])
-    fprintf(Yap_stderr, "[getch %c]", ch);
-#endif
-  return(ch);
-}
-
 extern double atof(const char *);
 
 static Term
@@ -222,7 +234,8 @@ float_send(char *s, int sign)
 #if HAVE_FINITE
   if (yap_flags[LANGUAGE_MODE_FLAG] == 1) { /* iso */
     if (!finite(f)) {
-      Yap_ErrorMessage = "Float overflow while scanning";
+      CACHE_REGS
+      LOCAL_ErrorMessage = "Float overflow while scanning";
       return(MkEvalFl(0.0));
     }
   }
@@ -251,18 +264,26 @@ read_int_overflow(const char *s, Int base, Int val, int sign)
 #endif    
 }
 
+static int
+send_error_message(char s[])
+{
+  CACHE_REGS
+  LOCAL_ErrorMessage = s;
+  return 0;
+}
+
 static wchar_t
-read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
+read_quoted_char(int *scan_nextp, IOSTREAM *inp_stream)
 {
   int ch;
 
   /* escape sequence */
  restart:
-  ch = QuotedNxtch(inp_stream);
+  ch = getchrq(inp_stream);
   switch (ch) {
   case 10:
     do {
-      ch = QuotedNxtch(inp_stream);
+      ch = getchrq(inp_stream);
       if (ch == '\\') goto restart;
       if (chtype(ch) != BS || ch == 10) {
 	return ch;
@@ -274,11 +295,10 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
     return '\b';
   case 'c':
     if (yap_flags[CHARACTER_ESCAPE_FLAG] == ISO_CHARACTER_ESCAPES) {
-      Yap_ErrorMessage = "invalid escape sequence \\c";
-      return 0;
+      return send_error_message("invalid escape sequence \\c");
     } else {
       /* sicstus */
-      ch = QuotedNxtch(inp_stream);
+      ch = getchrq(inp_stream);
       if (chtype(ch) == SL) {
 	goto restart;
       } else {
@@ -295,6 +315,11 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
     return '\n';
   case 'r':
     return '\r';
+  case 's':         /* space */
+    if (yap_flags[CHARACTER_ESCAPE_FLAG] == ISO_CHARACTER_ESCAPES) {
+      return send_error_message("invalid escape sequence \\s");
+    } else
+      return ' ';
   case 't':
     return '\t';
   case 'u':
@@ -303,7 +328,7 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
       wchar_t wc='\0';
 
       for (i=0; i< 4; i++) {
-	ch = QuotedNxtch(inp_stream);
+	ch = getchrq(inp_stream);
 	if (ch>='0' && ch <= '9') {
 	  wc += (ch-'0')<<((3-i)*4);
 	} else if (ch>='a' && ch <= 'f') {
@@ -311,8 +336,7 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
 	} else if (ch>='A' && ch <= 'F') {
 	  wc += ((ch-'A')+10)<<((3-i)*4);
 	} else {
-	  Yap_ErrorMessage = "invalid escape sequence";
-	  return 0;
+	  return send_error_message("invalid escape sequence");
 	}
       }
       return wc;
@@ -323,7 +347,7 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
       wchar_t wc='\0';
 
       for (i=0; i< 8; i++) {
-	ch = QuotedNxtch(inp_stream);
+	ch = getchrq(inp_stream);
 	if (ch>='0' && ch <= '9') {
 	  wc += (ch-'0')<<((7-i)*4);
 	} else if (ch>='a' && ch <= 'f') {
@@ -331,14 +355,18 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
 	} else if (ch>='A' && ch <= 'F') {
 	  wc += ((ch-'A')+10)<<((7-i)*4);
 	} else {
-	  Yap_ErrorMessage = "invalid escape sequence";
-	  return 0;
+	  return send_error_message("invalid escape sequence");
 	}
       }
       return wc;
     }
   case 'v':
     return '\v';
+  case 'z':         /* Prolog end-of-file */
+    if (yap_flags[CHARACTER_ESCAPE_FLAG] == ISO_CHARACTER_ESCAPES) {
+      return send_error_message("invalid escape sequence \\z");
+    } else
+      return -1;
   case '\\':
     return '\\';
   case '\'':
@@ -349,15 +377,14 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
     return '`';
   case '^':
     if (yap_flags[CHARACTER_ESCAPE_FLAG] == ISO_CHARACTER_ESCAPES) {
-      Yap_ErrorMessage = "invalid escape sequence";
-      return 0;
+      return send_error_message("invalid escape sequence");
     } else {
-      ch = QuotedNxtch(inp_stream);
+      ch = getchrq(inp_stream);
       if (ch ==  '?') {/* delete character */
 	return 127;
-      } else if (ch >= 'a' && ch < 'z') {/* octal */
+      } else if (ch >= 'a' && ch < 'z') {/* hexa */
 	return ch - 'a';
-      } else if (ch >= 'A' && ch < 'Z') {/* octal */
+      } else if (ch >= 'A' && ch < 'Z') {/* hexa */
 	return ch - 'A';
       } else {
 	 return '^';
@@ -372,38 +399,37 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
   case '6':
   case '7':
     /* character in octal: maximum of 3 digits, terminates with \ */
-    if (yap_flags[CHARACTER_ESCAPE_FLAG] == ISO_CHARACTER_ESCAPES) {
+    /* follow ISO */
+    if (TRUE || yap_flags[CHARACTER_ESCAPE_FLAG] == ISO_CHARACTER_ESCAPES) {
       unsigned char so_far = ch-'0';
-      ch = QuotedNxtch(inp_stream);
+      ch = getchrq(inp_stream);
       if (ch >= '0' && ch < '8') {/* octal */
 	so_far = so_far*8+(ch-'0');
-	ch = QuotedNxtch(inp_stream);
+	ch = getchrq(inp_stream);
 	if (ch >= '0' && ch < '8') { /* octal */
 	  so_far = so_far*8+(ch-'0');
-	  ch = QuotedNxtch(inp_stream);
+	  ch = getchrq(inp_stream);
 	  if (ch != '\\') {
-	    Yap_ErrorMessage = "invalid octal escape sequence";
-	    return 0;
+	    return send_error_message("invalid octal escape sequence");
 	  }
+	  return so_far;
 	} else if (ch == '\\') {
 	  return so_far;
 	} else {
-	  Yap_ErrorMessage = "invalid octal escape sequence";
-	  return 0;
+	  return send_error_message("invalid octal escape sequence");
 	}
       } else if (ch == '\\') {
 	return so_far;
       } else {
-	Yap_ErrorMessage = "invalid octal escape sequence";
-	return 0;
+	return send_error_message("invalid octal escape sequence");
       }
     } else {
       /* sicstus */
       unsigned char so_far = ch-'0';
-      ch = QuotedNxtch(inp_stream);
+      ch = getchrq(inp_stream);
       if (ch >= '0' && ch < '8') {/* octal */
 	so_far = so_far*8+(ch-'0');
-	ch = QuotedNxtch(inp_stream);
+	ch = getchrq(inp_stream);
 	if (ch >= '0' && ch < '8') { /* octal */
 	  return so_far*8+(ch-'0');
 	} else {
@@ -419,41 +445,38 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
     /* hexadecimal character (YAP allows empty hexadecimal  */
     if (yap_flags[CHARACTER_ESCAPE_FLAG] == ISO_CHARACTER_ESCAPES) {
       unsigned char so_far = 0; 
-      ch = QuotedNxtch(inp_stream);
+      ch = getchrq(inp_stream);
       if (my_isxdigit(ch,'f','F')) {/* hexa */
 	so_far = so_far * 16 + (chtype(ch) == NU ? ch - '0' :
 				(my_isupper(ch) ? ch - 'A' : ch - 'a') + 10);
-	ch = QuotedNxtch(inp_stream);
+	ch = getchrq(inp_stream);
 	if (my_isxdigit(ch,'f','F')) { /* hexa */
 	  so_far = so_far * 16 + (chtype(ch) == NU ? ch - '0' :
 				  (my_isupper(ch) ? ch - 'A' : ch - 'a') + 10);
-	  ch = QuotedNxtch(inp_stream);
+	  ch = getchrq(inp_stream);
 	  if (ch == '\\') {
 	    return so_far;
 	  } else {
-	    Yap_ErrorMessage = "invalid hexadecimal escape sequence";
-	    return 0;
+	    return send_error_message("invalid hexadecimal escape sequence");
 	  }
 	} else if (ch == '\\') {
 	  return so_far;
 	} else {
-	  Yap_ErrorMessage = "invalid hexadecimal escape sequence";
-	  return 0;
+	  return send_error_message("invalid hexadecimal escape sequence");
 	} 
       } else if (ch == '\\') {
 	return so_far;
       } else {
-	Yap_ErrorMessage = "invalid hexadecimal escape sequence";
-	return 0;
+	return send_error_message("invalid hexadecimal escape sequence");
       }
     } else {
       /* sicstus mode */
       unsigned char so_far = 0;
-      ch = QuotedNxtch(inp_stream);
+      ch = getchrq(inp_stream);
       so_far = (chtype(ch) == NU ? ch - '0' :
 		my_isupper(ch) ? ch - 'A' + 10 : 
 		my_islower(ch) ? ch - 'a' +10 : 0);
-      ch = QuotedNxtch(inp_stream);
+      ch = getchrq(inp_stream);
       return so_far*16 + (chtype(ch) == NU ? ch - '0' :
 		       my_isupper(ch) ? ch - 'A' +10 :
 		       my_islower(ch) ? ch - 'a' + 10 : 0);
@@ -463,8 +486,7 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
        consider this sequence legal, whereas SICStus would
        eat up the escape sequence. */
     if (yap_flags[CHARACTER_ESCAPE_FLAG] == ISO_CHARACTER_ESCAPES) {
-      Yap_ErrorMessage = "invalid escape sequence";
-      return 0;
+      return send_error_message("invalid escape sequence");
     } else {
       /* sicstus */
       if (chtype(ch) == SL) {
@@ -476,10 +498,19 @@ read_quoted_char(int *scan_nextp, int inp_stream, int (*QuotedNxtch)(int))
   }
 }
 	    
+static int
+num_send_error_message(char s[])
+{
+  CACHE_REGS
+  LOCAL_ErrorMessage = s;
+  return TermNil;
+}
+
 /* reads a number, either integer or float */
 
+
 static Term
-get_num(int *chp, int *chbuffp, int inp_stream, int (*Nxtch) (int), int (*QuotedNxtch) (int), char *s, UInt max_size, int sign)
+get_num(int *chp, int *chbuffp, IOSTREAM *inp_stream, char *s, UInt max_size, int sign)
 {
   char *sp = s;
   int ch = *chp;
@@ -487,7 +518,7 @@ get_num(int *chp, int *chbuffp, int inp_stream, int (*Nxtch) (int), int (*Quoted
   int might_be_float = TRUE, has_overflow = FALSE;
 
   *sp++ = ch;
-  ch = Nxtch(inp_stream);
+  ch = getchr(inp_stream);
   /*
    * because of things like 00'2, 03'2 and even better 12'2, I need to
    * do this (have mercy) 
@@ -495,35 +526,32 @@ get_num(int *chp, int *chbuffp, int inp_stream, int (*Nxtch) (int), int (*Quoted
   if (chtype(ch) == NU) {
     *sp++ = ch;
     if (--max_size == 0) {
-      Yap_ErrorMessage = "Number Too Long";
-      return TermNil;
+      return num_send_error_message("Number Too Long");
     }
     base = 10 * base + ch - '0';
-    ch = Nxtch(inp_stream);
+    ch = getchr(inp_stream);
   }
   if (ch == '\'') {
     if (base > 36) {
-      Yap_ErrorMessage = "Admissible bases are 0..36";
-      return TermNil;
+      return num_send_error_message("Admissible bases are 0..36");
     }
     might_be_float = FALSE;
     if (--max_size == 0) {
-      Yap_ErrorMessage = "Number Too Long";
-      return TermNil;
+      return num_send_error_message("Number Too Long");
     }
     *sp++ = ch;
-    ch = Nxtch(inp_stream);
+    ch = getchr(inp_stream);
     if (base == 0) {
       wchar_t ascii = ch;
       int scan_extra = TRUE;
 
       if (ch == '\\' &&
 	  yap_flags[CHARACTER_ESCAPE_FLAG] != CPROLOG_CHARACTER_ESCAPES) {
-	ascii = read_quoted_char(&scan_extra, inp_stream, QuotedNxtch);
+	ascii = read_quoted_char(&scan_extra, inp_stream);
       }
       /* a quick way to represent ASCII */
       if (scan_extra)
-	*chp = Nxtch(inp_stream);
+	*chp = getchr(inp_stream);
       if (sign == -1) {
 	return MkIntegerTerm(-ascii);
       }
@@ -537,48 +565,45 @@ get_num(int *chp, int *chbuffp, int inp_stream, int (*Nxtch) (int), int (*Quoted
 	int chval = (chtype(ch) == NU ? ch - '0' :
 		     (my_isupper(ch) ? ch - 'A' : ch - 'a') + 10);
 	if (--max_size == 0) {
-	  Yap_ErrorMessage = "Number Too Long";
-	  return TermNil;
+	  return num_send_error_message("Number Too Long");
 	}
 	*sp++ = ch;
 	val = oval * base + chval;
 	if (oval != (val-chval)/base) /* overflow */
 	  has_overflow = (has_overflow || TRUE);
-	ch = Nxtch(inp_stream);
+	ch = getchr(inp_stream);
       }
     }
   } else if (ch == 'x' && base == 0) {
     might_be_float = FALSE;
     if (--max_size == 0) {
-      Yap_ErrorMessage = "Number Too Long";
-      return TermNil;
+      return num_send_error_message("Number Too Long");
     }
     *sp++ = ch;
-    ch = Nxtch(inp_stream);
+    ch = getchr(inp_stream);
     while (my_isxdigit(ch, 'F', 'f')) {
       Int oval = val;
       int chval = (chtype(ch) == NU ? ch - '0' :
 		   (my_isupper(ch) ? ch - 'A' : ch - 'a') + 10);
       if (--max_size == 0) {
-	Yap_ErrorMessage = "Number Too Long";
-	return TermNil;
+	return num_send_error_message("Number Too Long");
       }
       *sp++ = ch;
       val = val * 16 + chval;
       if (oval != (val-chval)/16) /* overflow */
 	has_overflow = TRUE;
-      ch = Nxtch(inp_stream);
+      ch = getchr(inp_stream);
     }
     *chp = ch;
   }
   else if (ch == 'o' && base == 0) {
     might_be_float = FALSE;
     base = 8;
-    ch = Nxtch(inp_stream);
+    ch = getchr(inp_stream);
   } else if (ch == 'b' && base == 0) {
     might_be_float = FALSE;
     base = 2;
-    ch = Nxtch(inp_stream);
+    ch = getchr(inp_stream);
   } else {
     val = base;
     base = 10;
@@ -587,8 +612,7 @@ get_num(int *chp, int *chbuffp, int inp_stream, int (*Nxtch) (int), int (*Quoted
     Int oval = val;
     if (!(val == 0 && ch == '0') || has_overflow) {
       if (--max_size == 0) {
-	Yap_ErrorMessage = "Number Too Long";
-	return (TermNil);
+	return num_send_error_message("Number Too Long");
       }
       *sp++ = ch;
     }
@@ -600,20 +624,18 @@ get_num(int *chp, int *chbuffp, int inp_stream, int (*Nxtch) (int), int (*Quoted
     val = val * base + ch - '0';
     if (val/base != oval || val -oval*base != ch-'0') /* overflow */
       has_overflow = TRUE;
-    ch = Nxtch(inp_stream);
+    ch = getchr(inp_stream);
   }
   if (might_be_float && ( ch == '.'  || ch == 'e' || ch == 'E')) {
     if (yap_flags[STRICT_ISO_FLAG] && (ch == 'e' || ch == 'E')) {
-	Yap_ErrorMessage = "Float format not allowed in ISO mode";
-	return TermNil;
+      return num_send_error_message("Float format not allowed in ISO mode");
     }
     if (ch == '.') {
       if (--max_size == 0) {
-	Yap_ErrorMessage = "Number Too Long";
-	return TermNil;
+	return num_send_error_message("Number Too Long");
       }
       *sp++ = '.';
-      if (chtype(ch = Nxtch(inp_stream)) != NU) {
+      if (chtype(ch = getchr(inp_stream)) != NU) {
 	*chbuffp = '.';
 	*chp = ch;
 	*--sp = '\0';
@@ -625,34 +647,31 @@ get_num(int *chp, int *chbuffp, int inp_stream, int (*Nxtch) (int), int (*Quoted
       }
       do {
 	if (--max_size == 0) {
-	  Yap_ErrorMessage = "Number Too Long";
-	  return TermNil;
+	  return num_send_error_message("Number Too Long");
 	}
 	*sp++ = ch;
       }
-      while (chtype(ch = Nxtch(inp_stream)) == NU);
+      while (chtype(ch = getchr(inp_stream)) == NU);
     }
     if (ch == 'e' || ch == 'E') {
       char *sp0 = sp;
       char cbuff = ch;
 
       if (--max_size == 0) {
-	Yap_ErrorMessage = "Number Too Long";
-	return TermNil;
+	return num_send_error_message("Number Too Long");
       }
       *sp++ = ch;
-      ch = Nxtch(inp_stream);
+      ch = getchr(inp_stream);
       if (ch == '-') {
 	cbuff = '-';
 	if (--max_size == 0) {
-	  Yap_ErrorMessage = "Number Too Long";
-	  return TermNil;
+	  return num_send_error_message("Number Too Long");
 	}
 	*sp++ = '-';
-	ch = Nxtch(inp_stream);
+	ch = getchr(inp_stream);
       } else if (ch == '+') {
 	cbuff = '+';
-	ch = Nxtch(inp_stream);
+	ch = getchr(inp_stream);
       }
       if (chtype(ch) != NU) {
 	/* error */
@@ -668,11 +687,10 @@ get_num(int *chp, int *chbuffp, int inp_stream, int (*Nxtch) (int), int (*Quoted
       }
       do {
 	if (--max_size == 0) {
-	  Yap_ErrorMessage = "Number Too Long";
-	  return TermNil;
+	  return num_send_error_message("Number Too Long");
 	}
 	*sp++ = ch;
-      } while (chtype(ch = Nxtch(inp_stream)) == NU);
+      } while (chtype(ch = getchr(inp_stream)) == NU);
     }
     *sp = '\0';
     *chp = ch;
@@ -698,59 +716,117 @@ get_num(int *chp, int *chbuffp, int inp_stream, int (*Nxtch) (int), int (*Quoted
   }
 }
 
-/* given a function Nxtch scan until we  either find the number
+/* given a function getchr scan until we  either find the number
    or end of file */
 Term
-Yap_scan_num(int (*Nxtch) (int))
+Yap_scan_num(IOSTREAM *inp)
 {
+  CACHE_REGS
   Term out;
   int sign = 1;
   int ch, cherr;
   char *ptr;
 
-  Yap_ErrorMessage = NULL;
-  ScannerStack = (char *)TR;
-  ScannerExtraBlocks = NULL;
+  LOCAL_ErrorMessage = NULL;
+  LOCAL_ScannerStack = (char *)TR;
+  LOCAL_ScannerExtraBlocks = NULL;
   if (!(ptr = AllocScannerMemory(4096))) {
-    Yap_ErrorMessage = "Trail Overflow";
-    Yap_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
+    LOCAL_ErrorMessage = "Trail Overflow";
+    LOCAL_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
     return TermNil;
   }
-  ch = Nxtch(-1);
+  ch = getchr(inp);
   while (chtype(ch) == BS) {
-    ch = Nxtch(-1);
+    ch = getchr(inp);
   }
   if (ch == '-') {
     sign = -1;
-    ch = Nxtch(-1);
+    ch = getchr(inp);
   } else if (ch == '+') {
-    ch = Nxtch(-1);
+    ch = getchr(inp);
   }
   if (chtype(ch) != NU) {
-    Yap_clean_tokenizer(NULL, NULL, NULL);
+    Yap_clean_tokenizer(NULL, NULL, NULL, 0L);
     return TermNil;
   }
   cherr = '\0';
   if (ASP-H < 1024)
     return TermNil;
-  out = get_num(&ch, &cherr, -1, Nxtch, Nxtch, ptr, 4096, sign); /*  */
+  out = get_num(&ch, &cherr, inp, ptr, 4096, sign); /*  */
   PopScannerMemory(ptr, 4096);
-  Yap_clean_tokenizer(NULL, NULL, NULL);
-  if (Yap_ErrorMessage != NULL || ch != -1 || cherr)
+  Yap_clean_tokenizer(NULL, NULL, NULL, 0L);
+  if (LOCAL_ErrorMessage != NULL || ch != -1 || cherr)
     return TermNil;
   return out;
 }
 
 
+#define CHECK_SPACE() \
+	  if (ASP-H < 1024) { \
+	    LOCAL_ErrorMessage = "Stack Overflow";     \
+	    LOCAL_Error_TYPE = OUT_OF_STACK_ERROR;	\
+	    LOCAL_Error_Size = 0L;	               \
+	    if (p) \
+	      p->Tok = Ord(kind = eot_tok);           \
+	    /* serious error now */                    \
+	    return l;                                  \
+	  } 
+
+
+static void
+open_comment(int ch, IOSTREAM *inp_stream USES_REGS) {
+  CELL *h0 = H;
+  H += 5;
+  h0[0] = AbsAppl(h0+2);
+  h0[1] = TermNil;
+  if (!LOCAL_CommentsTail) {
+    /* first comment */
+    LOCAL_Comments = AbsPair(h0);
+  } else {
+    /* extra comment */
+    *LOCAL_CommentsTail = AbsPair(h0);
+  }  
+  LOCAL_CommentsTail = h0+1;
+  h0 += 2;
+  h0[0] = (CELL)FunctorMinus;
+  h0[1] = Yap_StreamPosition(inp_stream);
+  h0[2] = TermNil;
+  LOCAL_CommentsNextChar = h0+2;
+  LOCAL_CommentsBuff = (wchar_t *)malloc(1024*sizeof(wchar_t));
+  LOCAL_CommentsBuffLim = 1024;
+  LOCAL_CommentsBuff[0] = ch;
+  LOCAL_CommentsBuffPos = 1;
+}
+
+static void
+extend_comment(int ch USES_REGS) {
+  LOCAL_CommentsBuff[LOCAL_CommentsBuffPos] = ch;
+  LOCAL_CommentsBuffPos++;
+  if (LOCAL_CommentsBuffPos == LOCAL_CommentsBuffLim-1) {
+    LOCAL_CommentsBuff = (wchar_t *)realloc(LOCAL_CommentsBuff,sizeof(wchar_t)*(LOCAL_CommentsBuffLim+4096));
+    LOCAL_CommentsBuffLim += 4096;
+  }
+}
+
+static void
+close_comment( USES_REGS1 ) {
+  LOCAL_CommentsBuff[LOCAL_CommentsBuffPos] = '\0';
+  *LOCAL_CommentsNextChar = Yap_MkBlobWideStringTerm(LOCAL_CommentsBuff, LOCAL_CommentsBuffPos);
+  free(LOCAL_CommentsBuff);
+  LOCAL_CommentsBuff = NULL;
+  LOCAL_CommentsBuffLim = 0;
+}
+
 static wchar_t *
 ch_to_wide(char *base, char *charp)
 {
+  CACHE_REGS
   int n = charp-base, i;
   wchar_t *nb = (wchar_t *)base;
 
   if ((nb+n) + 1024 > (wchar_t *)AuxSp) {
-    Yap_Error_TYPE = OUT_OF_AUXSPACE_ERROR;	  
-    Yap_ErrorMessage = "Heap Overflow While Scanning: please increase code space (-h)";
+    LOCAL_Error_TYPE = OUT_OF_AUXSPACE_ERROR;	  
+    LOCAL_ErrorMessage = "Heap Overflow While Scanning: please increase code space (-h)";
     return NULL;
   }
   for (i=n; i > 0; i--) {
@@ -760,41 +836,40 @@ ch_to_wide(char *base, char *charp)
 }
 
 #define  add_ch_to_buff(ch) \
-  if (wcharp) { *wcharp++ = (ch); charp = (char *)wcharp; }	\
+  if (wcharp) { *wcharp++ = (ch); if (wcharp >= (wchar_t *)AuxSp-1024) goto huge_var_error; charp = (char *)wcharp; } \
   else { \
     if (ch > MAX_ISO_LATIN1 && !wcharp) { \
       /* does not fit in ISO-LATIN */		\
       wcharp = ch_to_wide(TokImage, charp);	\
       if (!wcharp) goto huge_var_error;		\
       *wcharp++ = (ch); charp = (char *)wcharp; \
-    } else *charp++ = ch;			\
+    } else { if (charp >= (char *)AuxSp-1024) goto huge_var_error; *charp++ = ch; }	\
   }
 
 TokEntry *
-Yap_tokenizer(int inp_stream, Term *tposp)
+Yap_tokenizer(IOSTREAM *inp_stream, int store_comments, Term *tposp)
 {
+  CACHE_REGS
   TokEntry *t, *l, *p;
   enum TokenKinds kind;
   int solo_flag = TRUE;
   int ch;
   wchar_t *wcharp;
-  int (*Nxtch) (int) = Stream[inp_stream].stream_wgetc_for_read;
-  int (*QuotedNxtch) (int) = Stream[inp_stream].stream_wgetc;
 
-  Yap_ErrorMessage = NULL;
-  Yap_Error_Size = 0;
-  Yap_VarTable = NULL;
-  Yap_AnonVarTable = NULL;
-  Yap_eot_before_eof = FALSE;
-  ScannerStack = (char *)TR;
-  ScannerExtraBlocks = NULL;
+  LOCAL_ErrorMessage = NULL;
+  LOCAL_Error_Size = 0;
+  LOCAL_VarTable = NULL;
+  LOCAL_AnonVarTable = NULL;
+  LOCAL_ScannerStack = (char *)TR;
+  LOCAL_ScannerExtraBlocks = NULL;
   l = NULL;
   p = NULL;			/* Just to make lint happy */
-  ch = Nxtch(inp_stream);
+  ch = getchr(inp_stream);
   while (chtype(ch) == BS) {
-    ch = Nxtch(inp_stream);
+    ch = getchr(inp_stream);
   }
   *tposp = Yap_StreamPosition(inp_stream);
+  LOCAL_StartLine = inp_stream->posbuf.lineno;
   do {
     wchar_t och;
     int quote, isvar;
@@ -806,8 +881,8 @@ Yap_tokenizer(int inp_stream, Term *tposp)
     t = (TokEntry *) AllocScannerMemory(sizeof(TokEntry));
     t->TokNext = NULL;
     if (t == NULL) {
-      Yap_ErrorMessage = "Trail Overflow";
-      Yap_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
+      LOCAL_ErrorMessage = "Trail Overflow";
+      LOCAL_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
       if (p)
 	p->Tok = Ord(kind = eot_tok);
       /* serious error now */
@@ -820,21 +895,42 @@ Yap_tokenizer(int inp_stream, Term *tposp)
     p = t;
   restart:
     while (chtype(ch) == BS) {
-      ch = Nxtch(inp_stream);
+      ch = getchr(inp_stream);
     }
     t->TokPos = GetCurInpPos(inp_stream);
 
     switch (chtype(ch)) {
 
     case CC:
-      while ((ch = Nxtch(inp_stream)) != 10 && chtype(ch) != EF);
+      if (store_comments) {
+	CHECK_SPACE();
+	open_comment(ch, inp_stream PASS_REGS);
+      continue_comment:
+	while ((ch = getchr(inp_stream)) != 10 && chtype(ch) != EF) {
+	  CHECK_SPACE();
+	  extend_comment(ch PASS_REGS);
+	}
+	CHECK_SPACE();
+	extend_comment(ch PASS_REGS);
+	if (chtype(ch) != EF) {
+	  ch = getchr(inp_stream);
+	  if (chtype(ch) == CC) {
+	    extend_comment(ch PASS_REGS);
+	    goto continue_comment;
+	  }
+	}
+	close_comment( PASS_REGS1 );
+      } else {
+	while ((ch = getchr(inp_stream)) != 10 && chtype(ch) != EF);
+      }
       if (chtype(ch) != EF) {
 	/* blank space */
 	if (t == l) {
 	  /* we found a comment before reading characters */
 	  while (chtype(ch) == BS) {
-	    ch = Nxtch(inp_stream);
+	    ch = getchr(inp_stream);
 	  }
+	  CHECK_SPACE();
 	  *tposp = Yap_StreamPosition(inp_stream);
 	}
 	goto restart;
@@ -847,19 +943,19 @@ Yap_tokenizer(int inp_stream, Term *tposp)
     case UL:
     case LC:
       och = ch;
-      ch = Nxtch(inp_stream);
+      ch = getchr(inp_stream);
     scan_name:
       TokImage = ((AtomEntry *) ( Yap_PreAllocCodeSpace()))->StrOfAE;
       charp = TokImage;
       wcharp = NULL;
       isvar = (chtype(och) != LC);
       add_ch_to_buff(och);
-      for (; chtype(ch) <= NU; ch = Nxtch(inp_stream)) {
+      for (; chtype(ch) <= NU; ch = getchr(inp_stream)) {
 	if (charp == (char *)AuxSp-1024) {
 	huge_var_error:
 	  /* huge atom or variable, we are in trouble */
-	  Yap_ErrorMessage = "Code Space Overflow due to huge atom";
-	  Yap_Error_TYPE = OUT_OF_AUXSPACE_ERROR;	  
+	  LOCAL_ErrorMessage = "Code Space Overflow due to huge atom";
+	  LOCAL_Error_TYPE = OUT_OF_AUXSPACE_ERROR;	  
 	  Yap_ReleasePreAllocCodeSpace((CODEADDR)TokImage);
 	  if (p)
 	    p->Tok = Ord(kind = eot_tok);
@@ -873,7 +969,7 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	  goto huge_var_error;
 	}
 	add_ch_to_buff(ch);
-	ch = Nxtch(inp_stream);
+	ch = getchr(inp_stream);
       }
       add_ch_to_buff('\0');
       if (!isvar) {
@@ -885,8 +981,8 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	  ae = Yap_LookupAtom(TokImage);
 	}
 	if (ae == NIL) {
-	  Yap_Error_TYPE = OUT_OF_HEAP_ERROR;	  
-	  Yap_ErrorMessage = "Code Space Overflow";
+	  LOCAL_Error_TYPE = OUT_OF_HEAP_ERROR;	  
+	  LOCAL_ErrorMessage = "Code Space Overflow";
 	  if (p)
 	    t->Tok = Ord(kind = eot_tok);
 	  /* serious error now */
@@ -912,23 +1008,15 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 
 	cherr = 0;
 	if (!(ptr = AllocScannerMemory(4096))) {
-	  Yap_ErrorMessage = "Trail Overflow";
-	  Yap_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
+	  LOCAL_ErrorMessage = "Trail Overflow";
+	  LOCAL_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
 	  if (p)
 	    t->Tok = Ord(kind = eot_tok);
 	  /* serious error now */
 	  return l;
 	}
-	if (ASP-H < 1024) {
-	  Yap_ErrorMessage = "Stack Overflow";
-	  Yap_Error_TYPE = OUT_OF_STACK_ERROR;	            
-	  Yap_Error_Size = 0L;	            
-	  if (p)
-	    p->Tok = Ord(kind = eot_tok);
-	  /* serious error now */
-	  return l;
-	}
-	if ((t->TokInfo = get_num(&cha,&cherr,inp_stream,Nxtch,QuotedNxtch,ptr,4096,1)) == 0L) {
+	CHECK_SPACE();
+	if ((t->TokInfo = get_num(&cha,&cherr,inp_stream,ptr,4096,1)) == 0L) {
 	  if (p)
 	    p->Tok = Ord(kind = eot_tok);
 	  /* serious error now */
@@ -942,8 +1030,8 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	  t->TokPos = GetCurInpPos(inp_stream);
 	  e = (TokEntry *) AllocScannerMemory(sizeof(TokEntry));
 	  if (e == NULL) {
-	    Yap_ErrorMessage = "Trail Overflow";
-	    Yap_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
+	    LOCAL_ErrorMessage = "Trail Overflow";
+	    LOCAL_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
 	    if (p)
 	      p->Tok = Ord(kind = eot_tok);
 	    /* serious error now */
@@ -971,8 +1059,8 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	      t->TokPos = GetCurInpPos(inp_stream);
 	      e2 = (TokEntry *) AllocScannerMemory(sizeof(TokEntry));
 	      if (e2 == NULL) {
-		Yap_ErrorMessage = "Trail Overflow";
-		Yap_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
+		LOCAL_ErrorMessage = "Trail Overflow";
+		LOCAL_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
 		if (p)
 		  p->Tok = Ord(kind = eot_tok);
 		/* serious error now */
@@ -1002,8 +1090,8 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	      t->TokPos = GetCurInpPos(inp_stream);
 	      e2 = (TokEntry *) AllocScannerMemory(sizeof(TokEntry));
 	      if (e2 == NULL) {
-		Yap_ErrorMessage = "Trail Overflow";
-		Yap_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
+		LOCAL_ErrorMessage = "Trail Overflow";
+		LOCAL_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
 		t->Tok = Ord(kind = eot_tok);
 		/* serious error now */
 		return l;
@@ -1030,32 +1118,32 @@ Yap_tokenizer(int inp_stream, Term *tposp)
       charp = TokImage;
       quote = ch;
       len = 0;
-      ch = QuotedNxtch(inp_stream);
+      ch = getchrq(inp_stream);
       wcharp = NULL;
 
       while (TRUE) {
 	if (charp + 1024 > (char *)AuxSp) {
-	  Yap_Error_TYPE = OUT_OF_AUXSPACE_ERROR;	  
-	  Yap_ErrorMessage = "Heap Overflow While Scanning: please increase code space (-h)";
+	  LOCAL_Error_TYPE = OUT_OF_AUXSPACE_ERROR;	  
+	  LOCAL_ErrorMessage = "Heap Overflow While Scanning: please increase code space (-h)";
 	  break;
 	}
 	if (ch == 10  &&  yap_flags[CHARACTER_ESCAPE_FLAG] == ISO_CHARACTER_ESCAPES) {
 	  /* in ISO a new line terminates a string */
-	  Yap_ErrorMessage = "layout character \n inside quotes";
+	  LOCAL_ErrorMessage = "layout character \n inside quotes";
 	  break;
 	}
 	if (ch == quote) {
-	  ch = QuotedNxtch(inp_stream);
+	  ch = getchrq(inp_stream);
 	  if (ch != quote)
 	    break;
 	  add_ch_to_buff(ch);
-	  ch = QuotedNxtch(inp_stream);
+	  ch = getchrq(inp_stream);
 	} else if (ch == '\\' && yap_flags[CHARACTER_ESCAPE_FLAG] != CPROLOG_CHARACTER_ESCAPES) {
 	  int scan_next = TRUE;
-	  ch = read_quoted_char(&scan_next, inp_stream, QuotedNxtch);
+	  ch = read_quoted_char(&scan_next, inp_stream);
 	  add_ch_to_buff(ch);
 	  if (scan_next) {
-	    ch = QuotedNxtch(inp_stream);
+	    ch = getchrq(inp_stream);
 	  }
 	} else if (chtype(ch) == EF && ch <= MAX_ISO_LATIN1) {
 	  Yap_ReleasePreAllocCodeSpace((CODEADDR)TokImage);
@@ -1063,13 +1151,13 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	  break;
 	} else {
 	  add_ch_to_buff(ch);
-	  ch = QuotedNxtch(inp_stream);
+	  ch = getchrq(inp_stream);
 	}
 	++len;
 	if (charp > (char *)AuxSp - 1024) {
 	  /* Not enough space to read in the string. */
-	  Yap_Error_TYPE = OUT_OF_AUXSPACE_ERROR;	  
-	  Yap_ErrorMessage = "not enough space to read in string or quoted atom";
+	  LOCAL_Error_TYPE = OUT_OF_AUXSPACE_ERROR;	  
+	  LOCAL_ErrorMessage = "not enough space to read in string or quoted atom";
 	  /* serious error now */
 	  Yap_ReleasePreAllocCodeSpace((CODEADDR)TokImage);
 	  t->Tok = Ord(kind = eot_tok);
@@ -1088,7 +1176,7 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	  mp = AllocScannerMemory(len + 1);
 	}
 	if (mp == NULL) {
-	  Yap_ErrorMessage = "not enough heap space to read in string or quoted atom";
+	  LOCAL_ErrorMessage = "not enough heap space to read in string or quoted atom";
 	  Yap_ReleasePreAllocCodeSpace((CODEADDR)TokImage);
 	  t->Tok = Ord(kind = eot_tok);
 	  return l;
@@ -1111,8 +1199,8 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	  t->TokInfo = Unsigned(Yap_LookupAtom(TokImage));
 	}
 	if (!(t->TokInfo)) {
-	  Yap_Error_TYPE = OUT_OF_HEAP_ERROR;	  
-	  Yap_ErrorMessage = "Code Space Overflow";
+	  LOCAL_Error_TYPE = OUT_OF_HEAP_ERROR;	  
+	  LOCAL_ErrorMessage = "Code Space Overflow";
 	  if (p)
 	    t->Tok = Ord(kind = eot_tok);
 	  /* serious error now */
@@ -1127,41 +1215,57 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 
     case SY:
       och = ch;
-      ch = Nxtch(inp_stream);
+      ch = getchr(inp_stream);
       if (och == '/' && ch == '*') {
-	while ((och != '*' || ch != '/') && chtype(ch) != EF) {
-	  och = ch;
-	  ch = Nxtch(inp_stream);
+	if (store_comments) {
+	  CHECK_SPACE();
+	  open_comment('/', inp_stream PASS_REGS);
+	  while ((och != '*' || ch != '/') && chtype(ch) != EF) {
+	    och = ch;
+	    CHECK_SPACE();
+	    extend_comment(ch PASS_REGS);
+	    ch = getchr(inp_stream);
+	  }
+	  if (chtype(ch) != EF) {
+	    CHECK_SPACE();
+	    extend_comment(ch PASS_REGS);
+	  }
+	  close_comment( PASS_REGS1 );
+	} else {
+	  while ((och != '*' || ch != '/') && chtype(ch) != EF) {
+	    och = ch;
+	    ch = getchr(inp_stream);
+	  }
 	}
 	if (chtype(ch) == EF) {
 	  t->Tok = Ord(kind = eot_tok);
-	}
-	ch = Nxtch(inp_stream);
-	if (t == l) {
-	  /* we found a comment before reading characters */
-	  while (chtype(ch) == BS) {
-	    ch = Nxtch(inp_stream);
+	} else {
+	  /* leave comments */
+	  ch = getchr(inp_stream);
+	  if (t == l) {
+	    /* we found a comment before reading characters */
+	    while (chtype(ch) == BS) {
+	      ch = getchr(inp_stream);
+	    }
+	    CHECK_SPACE();
+	    *tposp = Yap_StreamPosition(inp_stream);
 	  }
-	  *tposp = Yap_StreamPosition(inp_stream);
 	}
 	goto restart;
       }
     enter_symbol:
       if (och == '.' && (chtype(ch) == BS || chtype(ch) == EF
 			 || chtype(ch) == CC)) {
-	Yap_eot_before_eof = TRUE;
 	if (chtype(ch) == CC)
-	  while ((ch = Nxtch(inp_stream)) != 10 && chtype(ch) != EF);
+	  while ((ch = getchr(inp_stream)) != 10 && chtype(ch) != EF);
 	t->Tok = Ord(kind = eot_tok);
-      }
-      else {
+      } else {
+	Atom ae;
 	TokImage = ((AtomEntry *) ( Yap_PreAllocCodeSpace()))->StrOfAE;
-	AtomEntry *ae;
-
 	charp = TokImage;
 	wcharp = NULL;
 	add_ch_to_buff(och);
-	for (; chtype(ch) == SY; ch = Nxtch(inp_stream)) {
+	for (; chtype(ch) == SY; ch = getchr(inp_stream)) {
 	  if (charp == (char *)AuxSp-1024) {
 	    goto huge_var_error;
 	  }
@@ -1174,8 +1278,8 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	  ae = Yap_LookupAtom(TokImage);
 	}
 	if (ae == NIL) {
-	  Yap_Error_TYPE = OUT_OF_HEAP_ERROR;	  
-	  Yap_ErrorMessage = "Code Space Overflow";
+	  LOCAL_Error_TYPE = OUT_OF_HEAP_ERROR;	  
+	  LOCAL_ErrorMessage = "Code Space Overflow";
 	  if (p)
 	    t->Tok = Ord(kind = eot_tok);
 	  /* serious error now */
@@ -1183,8 +1287,8 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	}
 	t->TokInfo = Unsigned(ae);
 	if (t->TokInfo == (CELL)NIL) {
-	  Yap_Error_TYPE = OUT_OF_HEAP_ERROR;	  
-	  Yap_ErrorMessage = "Code Space Overflow";
+	  LOCAL_Error_TYPE = OUT_OF_HEAP_ERROR;	  
+	  LOCAL_ErrorMessage = "Code Space Overflow";
 	  if (p)
 	    t->Tok = Ord(kind = eot_tok);
 	  /* serious error now */
@@ -1204,7 +1308,7 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 	char chs[2];
 	chs[0] = ch;
 	chs[1] = '\0';
-	ch = Nxtch(inp_stream);
+	ch = getchr(inp_stream);
 	t->TokInfo = Unsigned(Yap_LookupAtom(chs));
 	t->Tok = Ord(kind = Name_tok);
 	if (ch == '(')
@@ -1214,26 +1318,26 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 
     case BK:
       och = ch;
-      ch = Nxtch(inp_stream);
+      ch = getchr(inp_stream);
       t->TokInfo = och;
       if (t->TokInfo == '(' && !solo_flag) {
 	t->TokInfo = 'l';
 	solo_flag = TRUE;
       } else if (och == '[')  {
-	while (chtype(ch) == BS) {  ch = Nxtch(inp_stream); };
+	while (chtype(ch) == BS) {  ch = getchr(inp_stream); };
 	if (ch == ']') {
 	  t->TokInfo = Unsigned(AtomNil);
 	  t->Tok = Ord(kind = Name_tok);
-	  ch = Nxtch(inp_stream);
+	  ch = getchr(inp_stream);
 	  solo_flag = FALSE;
 	  break;
 	}
       } else if (och == '{')  {
-	while (chtype(ch) == BS) {  ch = Nxtch(inp_stream); };
+	while (chtype(ch) == BS) {  ch = getchr(inp_stream); };
 	if (ch == '}') {
 	  t->TokInfo = Unsigned(AtomBraces);
 	  t->Tok = Ord(kind = Name_tok);
-	  ch = Nxtch(inp_stream);
+	  ch = getchr(inp_stream);
 	  solo_flag = FALSE;
 	  break;
 	}
@@ -1247,29 +1351,29 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 
     default:
 #ifdef DEBUG
-      fprintf(Yap_stderr, "\n++++ token: wrong char type %c %d\n", ch, chtype(ch));
+      fprintf(GLOBAL_stderr, "\n++++ token: wrong char type %c %d\n", ch, chtype(ch));
 #endif
       t->Tok = Ord(kind = eot_tok);
     }
 #ifdef DEBUG
-    if(Yap_Option[2]) fprintf(Yap_stderr,"[Token %d %ld]",Ord(kind),(unsigned long int)t->TokInfo);
+    if(GLOBAL_Option[2]) fprintf(GLOBAL_stderr,"[Token %d %ld]",Ord(kind),(unsigned long int)t->TokInfo);
 #endif
-    if (Yap_ErrorMessage) {
+    if (LOCAL_ErrorMessage) {
       /* insert an error token to inform the system of what happened */
       TokEntry *e = (TokEntry *) AllocScannerMemory(sizeof(TokEntry));
       if (e == NULL) {
-	Yap_ErrorMessage = "Trail Overflow";
-	Yap_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
+	LOCAL_ErrorMessage = "Trail Overflow";
+	LOCAL_Error_TYPE = OUT_OF_TRAIL_ERROR;	            
 	p->Tok = Ord(kind = eot_tok);
 	/* serious error now */
 	return l;
       }
       p->TokNext = e;
       e->Tok = Error_tok;
-      e->TokInfo = MkAtomTerm(Yap_LookupAtom(Yap_ErrorMessage));
+      e->TokInfo = MkAtomTerm(Yap_LookupAtom(LOCAL_ErrorMessage));
       e->TokPos = GetCurInpPos(inp_stream);
       e->TokNext = NULL;
-      Yap_ErrorMessage = NULL;
+      LOCAL_ErrorMessage = NULL;
       p = e;
     }
   } while (kind != eot_tok);
@@ -1277,13 +1381,21 @@ Yap_tokenizer(int inp_stream, Term *tposp)
 }
 
 void
-Yap_clean_tokenizer(TokEntry *tokstart, VarEntry *vartable, VarEntry *anonvartable)
+Yap_clean_tokenizer(TokEntry *tokstart, VarEntry *vartable, VarEntry *anonvartable, Term commentable)
 {
-  struct scanner_extra_alloc *ptr = ScannerExtraBlocks;
+  CACHE_REGS
+  struct scanner_extra_alloc *ptr = LOCAL_ScannerExtraBlocks;
   while (ptr) {
     struct scanner_extra_alloc *next = ptr->next;
     free(ptr);
     ptr = next;
   }
+  LOCAL_Comments = TermNil;
+  LOCAL_CommentsNextChar = LOCAL_CommentsTail = NULL;
+  if (LOCAL_CommentsBuff) {
+    free(LOCAL_CommentsBuff);
+    LOCAL_CommentsBuff = NULL;
+  }
+  LOCAL_CommentsBuffLim = 0;
 }
 
