@@ -1,4 +1,3 @@
-
 /*************************************************************************
 *									 *
 *	 YAP Prolog 							 *
@@ -28,6 +27,9 @@ static char     SccsId[] = "%W% %G%";
 #if HAVE_STRING_H
 #include <string.h>
 #endif
+#ifdef TABLING
+#include "tab.macros.h"
+#endif /* TABLING */
 
 #if THREADS
 
@@ -51,25 +53,36 @@ static int
 allocate_new_tid(void)
 {
   int new_worker_id = 0;
-  LOCK(ThreadHandlesLock);
+  LOCK(GLOBAL_ThreadHandlesLock);
   while(new_worker_id < MAX_THREADS &&
-	(FOREIGN_ThreadHandle(new_worker_id).in_use == TRUE ||
-	 FOREIGN_ThreadHandle(new_worker_id).zombie == TRUE) )
+	Yap_local[new_worker_id] &&
+	(REMOTE_ThreadHandle(new_worker_id).in_use == TRUE ||
+	 REMOTE_ThreadHandle(new_worker_id).zombie == TRUE) )
     new_worker_id++;
-  if (new_worker_id < MAX_THREADS) {
+  if (new_worker_id >= MAX_THREADS) {
+    new_worker_id = -1;
+  } else if (!Yap_local[new_worker_id]) {
+    if (!Yap_InitThread(new_worker_id)) {
+      return -1;
+    }
+    pthread_mutex_lock(&(REMOTE_ThreadHandle(new_worker_id).tlock));
     DEBUG_TLOCK_ACCESS(new_worker_id, 0);
-    pthread_mutex_lock(&(FOREIGN_ThreadHandle(new_worker_id).tlock));
-    FOREIGN_ThreadHandle(new_worker_id).in_use = TRUE;
+    REMOTE_ThreadHandle(new_worker_id).in_use = TRUE;
+  } else if (new_worker_id < MAX_THREADS) {
+    pthread_mutex_lock(&(REMOTE_ThreadHandle(new_worker_id).tlock));
+    DEBUG_TLOCK_ACCESS(new_worker_id, 0);
+    REMOTE_ThreadHandle(new_worker_id).in_use = TRUE;
   } else {
     new_worker_id = -1;
   }
-  UNLOCK(ThreadHandlesLock);
+  UNLOCK(GLOBAL_ThreadHandlesLock);
   return new_worker_id;  
 }
 
 static int
 store_specs(int new_worker_id, UInt ssize, UInt tsize, UInt sysize, Term *tpgoal, Term *tpdetach, Term *tpexit)
 {
+  CACHE_REGS
   UInt pm;	/* memory to be requested         */
   Term tmod;
   Term tdetach, tgoal;
@@ -78,32 +91,40 @@ store_specs(int new_worker_id, UInt ssize, UInt tsize, UInt sysize, Term *tpgoal
     tsize = MinTrailSpace;
   if (ssize < MinStackSpace)
     ssize = MinStackSpace;
-  FOREIGN_ThreadHandle(new_worker_id).ssize = ssize;
-  FOREIGN_ThreadHandle(new_worker_id).tsize = tsize;
-  FOREIGN_ThreadHandle(new_worker_id).sysize = sysize;
-  FOREIGN_WL(new_worker_id)->c_input_stream = Yap_c_input_stream;
-  FOREIGN_WL(new_worker_id)->c_output_stream = Yap_c_output_stream;
-  FOREIGN_WL(new_worker_id)->c_error_stream = Yap_c_error_stream;
+  REMOTE_ThreadHandle(new_worker_id).ssize = ssize;
+  REMOTE_ThreadHandle(new_worker_id).tsize = tsize;
+  REMOTE_ThreadHandle(new_worker_id).sysize = sysize;
+  REMOTE_c_input_stream(new_worker_id) = LOCAL_c_input_stream;
+  REMOTE_c_output_stream(new_worker_id) = LOCAL_c_output_stream;
+  REMOTE_c_error_stream(new_worker_id) = LOCAL_c_error_stream;
   pm = (ssize + tsize)*1024;
-  if (!(FOREIGN_ThreadHandle(new_worker_id).stack_address = malloc(pm))) {
+  if (!(REMOTE_ThreadHandle(new_worker_id).stack_address = malloc(pm))) {
     return FALSE;
   }
-  FOREIGN_ThreadHandle(new_worker_id).tgoal =
+  REMOTE_ThreadHandle(new_worker_id).tgoal =
     Yap_StoreTermInDB(Deref(*tpgoal),7);
-  FOREIGN_ThreadHandle(new_worker_id).cmod =
+  REMOTE_ThreadHandle(new_worker_id).cmod =
     CurrentModule;
   tdetach = Deref(*tpdetach);
   if (IsVarTerm(tdetach)){
-    FOREIGN_ThreadHandle(new_worker_id).tdetach =  
+    REMOTE_ThreadHandle(new_worker_id).tdetach =  
       MkAtomTerm(AtomFalse);
   } else {
-    FOREIGN_ThreadHandle(new_worker_id).tdetach = 
+    REMOTE_ThreadHandle(new_worker_id).tdetach = 
       tdetach;
   }
   tgoal = Yap_StripModule(Deref(*tpexit), &tmod);
-  FOREIGN_ThreadHandle(new_worker_id).texit_mod = tmod;
-  FOREIGN_ThreadHandle(new_worker_id).texit =
+  REMOTE_ThreadHandle(new_worker_id).texit_mod = tmod;
+  REMOTE_ThreadHandle(new_worker_id).texit =
     Yap_StoreTermInDB(tgoal,7);
+  REMOTE_ThreadHandle(new_worker_id).local_preds =
+    NULL;
+  REMOTE_ThreadHandle(new_worker_id).start_of_timesp =
+    NULL;
+  REMOTE_ThreadHandle(new_worker_id).last_timep =
+    NULL;
+  REMOTE_ScratchPad(new_worker_id).ptr =
+    NULL;
   return TRUE;
 }
 
@@ -111,11 +132,11 @@ store_specs(int new_worker_id, UInt ssize, UInt tsize, UInt sysize, Term *tpgoal
 static void
 kill_thread_engine (int wid, int always_die)
 {
-  Prop p0 = AbsPredProp(FOREIGN_ThreadHandle(wid).local_preds);
-  GlobalEntry *gl = GlobalVariables;
+  Prop p0 = AbsPredProp(REMOTE_ThreadHandle(wid).local_preds);
+  GlobalEntry *gl = REMOTE_GlobalVariables(wid);
 
-  FOREIGN_ThreadHandle(wid).local_preds = NIL;
-  GlobalVariables = NULL;
+  REMOTE_ThreadHandle(wid).local_preds = NIL;
+  REMOTE_GlobalVariables(wid) = NULL;
   /* kill all thread local preds */
   while(p0) {
     PredEntry *ap = RepPredProp(p0);
@@ -128,96 +149,154 @@ kill_thread_engine (int wid, int always_die)
     gl = gl->NextGE;
   }
   Yap_KillStacks(wid);
-  FOREIGN_WL(wid)->active_signals = 0L;
-  free(FOREIGN_WL(wid)->scratchpad.ptr);
-  free(FOREIGN_ThreadHandle(wid).default_yaam_regs);
-  FOREIGN_ThreadHandle(wid).current_yaam_regs = NULL;
-  free(FOREIGN_ThreadHandle(wid).start_of_timesp);
-  free(FOREIGN_ThreadHandle(wid).last_timep);
-  Yap_FreeCodeSpace((ADDR)FOREIGN_ThreadHandle(wid).texit);
-  LOCK(ThreadHandlesLock);
-  if (FOREIGN_ThreadHandle(wid).tdetach == MkAtomTerm(AtomTrue) ||
-      always_die) {
-    FOREIGN_ThreadHandle(wid).zombie = FALSE;
-    FOREIGN_ThreadHandle(wid).in_use = FALSE;
-    DEBUG_TLOCK_ACCESS(1, wid);
-    pthread_mutex_unlock(&(FOREIGN_ThreadHandle(wid).tlock));
+  REMOTE_ActiveSignals(wid) = 0L;
+  if (REMOTE_ScratchPad(wid).ptr)
+    free(REMOTE_ScratchPad(wid).ptr);
+  REMOTE_ThreadHandle(wid).current_yaam_regs = NULL;
+  if (REMOTE_ThreadHandle(wid).start_of_timesp)
+    free(REMOTE_ThreadHandle(wid).start_of_timesp);
+  if (REMOTE_ThreadHandle(wid).last_timep)
+    free(REMOTE_ThreadHandle(wid).last_timep);
+  if (REMOTE_ThreadHandle(wid).texit) {
+    Yap_FreeCodeSpace((ADDR)REMOTE_ThreadHandle(wid).texit);
   }
-  UNLOCK(ThreadHandlesLock);
+  /* FreeCodeSpace requires LOCAL requires yaam_regs */
+  free(REMOTE_ThreadHandle(wid).default_yaam_regs);
+  REMOTE_ThreadHandle(wid).default_yaam_regs = NULL;
+  LOCK(GLOBAL_ThreadHandlesLock);
+  if (REMOTE_ThreadHandle(wid).tdetach == MkAtomTerm(AtomTrue) ||
+      always_die) {
+    REMOTE_ThreadHandle(wid).zombie = FALSE;
+    REMOTE_ThreadHandle(wid).in_use = FALSE;
+    DEBUG_TLOCK_ACCESS(1, wid);
+    pthread_mutex_unlock(&(REMOTE_ThreadHandle(wid).tlock));
+  }
+  UNLOCK(GLOBAL_ThreadHandlesLock);
 }
 
 static void
 thread_die(int wid, int always_die)
 {
+#ifdef TABLING
+  CACHE_REGS
+  tab_ent_ptr tab_ent;
+
+  tab_ent = GLOBAL_root_tab_ent;
+  while (tab_ent) {
+    abolish_table(tab_ent);
+    tab_ent = TabEnt_next(tab_ent);
+  }
+  FREE_DEPENDENCY_FRAME(LOCAL_top_dep_fr);
+  LOCAL_top_dep_fr = NULL;
+#ifdef USE_PAGES_MALLOC
+  DETACH_PAGES(_pages_void);
+#endif /* USE_PAGES_MALLOC */
+  DETACH_PAGES(_pages_tab_ent);
+#if defined(THREADS_FULL_SHARING) || defined(THREADS_CONSUMER_SHARING)
+  DETACH_PAGES(_pages_sg_ent);
+#endif /* THREADS_FULL_SHARING || THREADS_CONSUMER_SHARING */
+  DETACH_PAGES(_pages_sg_fr);
+  DETACH_PAGES(_pages_dep_fr);
+  DETACH_PAGES(_pages_sg_node);
+  DETACH_PAGES(_pages_sg_hash);
+  DETACH_PAGES(_pages_ans_node);
+  DETACH_PAGES(_pages_ans_hash);
+#if defined(THREADS_FULL_SHARING)
+  DETACH_PAGES(_pages_ans_ref_node);
+#endif /* THREADS_FULL_SHARING */
+  DETACH_PAGES(_pages_gt_node);
+  DETACH_PAGES(_pages_gt_hash);
+#ifdef OUTPUT_THREADS_TABLING 
+  fclose(LOCAL_thread_output);
+#endif /* OUTPUT_THREADS_TABLING */
+#endif /* TABLING */
+  GLOBAL_NOfThreads--;
   if (!always_die) {
     /* called by thread itself */
-    ThreadsTotalTime += Yap_cputime();
+    GLOBAL_ThreadsTotalTime += Yap_cputime();
   }
   kill_thread_engine(wid, always_die);
 }
 
-static void
+static int
 setup_engine(int myworker_id, int init_thread)
 {
+  CACHE_REGS
   REGSTORE *standard_regs;
   
   standard_regs = (REGSTORE *)calloc(1,sizeof(REGSTORE));
+  if (!standard_regs)
+    return FALSE;
+  regcache = standard_regs;
   /* create the YAAM descriptor */
-  FOREIGN_ThreadHandle(myworker_id).default_yaam_regs = standard_regs;
-  if (init_thread) {
-    pthread_setspecific(Yap_yaamregs_key, (void *)FOREIGN_ThreadHandle(myworker_id).default_yaam_regs);
-  }
-  worker_id = myworker_id;
-  Yap_InitExStacks(FOREIGN_ThreadHandle(myworker_id).tsize, FOREIGN_ThreadHandle(myworker_id).ssize);
-  CurrentModule = FOREIGN_ThreadHandle(myworker_id).cmod;
-  Yap_InitTime();
-  Yap_InitYaamRegs();
-#ifdef YAPOR
-  Yap_init_local();
-#endif
+  REMOTE_ThreadHandle(myworker_id).default_yaam_regs = standard_regs;
+  Yap_InitExStacks(myworker_id, REMOTE_ThreadHandle(myworker_id).tsize, REMOTE_ThreadHandle(myworker_id).ssize);
+  CurrentModule = REMOTE_ThreadHandle(myworker_id).cmod;
+  Yap_InitTime( myworker_id );
+  Yap_InitYaamRegs( myworker_id );
+  REFRESH_CACHE_REGS
   Yap_ReleasePreAllocCodeSpace(Yap_PreAllocCodeSpace());
   /* I exist */
-  NOfThreadsCreated++;
+  GLOBAL_NOfThreadsCreated++;
+  GLOBAL_NOfThreads++;
   DEBUG_TLOCK_ACCESS(2, myworker_id);
-  pthread_mutex_unlock(&(FOREIGN_ThreadHandle(myworker_id).tlock));  
+  pthread_mutex_unlock(&(REMOTE_ThreadHandle(myworker_id).tlock));  
+#ifdef TABLING
+  new_dependency_frame(REMOTE_top_dep_fr(myworker_id), FALSE, NULL, NULL, B, NULL, FALSE, NULL);  /* same as in Yap_init_root_frames() */
+#endif /* TABLING */
+  return TRUE;
 }
 
 static void
 start_thread(int myworker_id)
 {
-  setup_engine(myworker_id, TRUE);
+  CACHE_REGS
+  pthread_setspecific(Yap_yaamregs_key, (void *)REMOTE_ThreadHandle(myworker_id).default_yaam_regs);
+  REFRESH_CACHE_REGS;
+  worker_id = myworker_id;
+  LOCAL = REMOTE(myworker_id);
 }
 
 static void *
 thread_run(void *widp)
 {
+  CACHE_REGS
   Term tgoal, t;
   Term tgs[2];
   int myworker_id = *((int *)widp); 
+#ifdef OUTPUT_THREADS_TABLING
+  char thread_name[25];
+  char filename[YAP_FILENAME_MAX]; 
 
+  sprintf(thread_name, "/thread_output_%d", myworker_id);
+  strcpy(filename, YAP_BINDIR);
+  strncat(filename, thread_name, 25);
+  REMOTE_thread_output(myworker_id) = fopen(filename, "w");
+#endif /* OUTPUT_THREADS_TABLING */
   start_thread(myworker_id);
+  REFRESH_CACHE_REGS;
   do {
-    t = tgs[0] = Yap_PopTermFromDB(MY_ThreadHandle.tgoal);
+    t = tgs[0] = Yap_PopTermFromDB(LOCAL_ThreadHandle.tgoal);
     if (t == 0) {
-      if (Yap_Error_TYPE == OUT_OF_ATTVARS_ERROR) {
-	Yap_Error_TYPE = YAP_NO_ERROR;
+      if (LOCAL_Error_TYPE == OUT_OF_ATTVARS_ERROR) {
+	LOCAL_Error_TYPE = YAP_NO_ERROR;
 	if (!Yap_growglobal(NULL)) {
-	  Yap_Error(OUT_OF_ATTVARS_ERROR, TermNil, Yap_ErrorMessage);
+	  Yap_Error(OUT_OF_ATTVARS_ERROR, TermNil, LOCAL_ErrorMessage);
 	  thread_die(worker_id, FALSE);
 	  return NULL;
 	}
       } else {
-	Yap_Error_TYPE = YAP_NO_ERROR;
-	if (!Yap_growstack(MY_ThreadHandle.tgoal->NOfCells*CellSize)) {
-	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, Yap_ErrorMessage);
+	LOCAL_Error_TYPE = YAP_NO_ERROR;
+	if (!Yap_growstack(LOCAL_ThreadHandle.tgoal->NOfCells*CellSize)) {
+	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, LOCAL_ErrorMessage);
 	  thread_die(worker_id, FALSE);
 	  return NULL;
 	}
       }
     }
   } while (t == 0);
-  FOREIGN_ThreadHandle(myworker_id).tgoal = NULL;
-  tgs[1] = MY_ThreadHandle.tdetach;
+  REMOTE_ThreadHandle(myworker_id).tgoal = NULL;
+  tgs[1] = LOCAL_ThreadHandle.tdetach;
   tgoal = Yap_MkApplTerm(FunctorThreadRun, 2, tgs);
   Yap_RunTopGoal(tgoal);
   thread_die(worker_id, FALSE);
@@ -225,7 +304,7 @@ thread_run(void *widp)
 }
 
 static Int
-p_thread_new_tid(void)
+p_thread_new_tid( USES_REGS1 )
 {
   int new_worker = allocate_new_tid();
   if (new_worker == -1) {
@@ -242,7 +321,7 @@ init_thread_engine(int new_worker_id, UInt ssize, UInt tsize, UInt sysize, Term
 }
 
 static Int
-p_create_thread(void)
+p_create_thread( USES_REGS1 )
 {
   UInt ssize;
   UInt tsize;
@@ -250,7 +329,8 @@ p_create_thread(void)
   Term x2 = Deref(ARG2);
   Term x3 = Deref(ARG3);
   Term x4 = Deref(ARG4);
-  int new_worker_id = IntegerOfTerm(Deref(ARG7));
+  int new_worker_id = IntegerOfTerm(Deref(ARG7)),
+    owid = worker_id;
   
   //  fprintf(stderr," %d --> %d\n", worker_id, new_worker_id); 
   if (IsBigIntTerm(x2))
@@ -268,18 +348,21 @@ p_create_thread(void)
   /* make sure we can proceed */
   if (!init_thread_engine(new_worker_id, ssize, tsize, sysize, &ARG1, &ARG5, &ARG6))
     return FALSE;
-  FOREIGN_ThreadHandle(new_worker_id).pthread_handle = 0L;
-  FOREIGN_ThreadHandle(new_worker_id).id = new_worker_id;
-  FOREIGN_ThreadHandle(new_worker_id).ref_count = 1;
-  if ((FOREIGN_ThreadHandle(new_worker_id).ret = pthread_create(&FOREIGN_ThreadHandle(new_worker_id).pthread_handle, NULL, thread_run, (void *)(&(FOREIGN_ThreadHandle(new_worker_id).id)))) == 0) {
+  //REMOTE_ThreadHandle(new_worker_id).pthread_handle = 0L;
+  REMOTE_ThreadHandle(new_worker_id).id = new_worker_id;
+  REMOTE_ThreadHandle(new_worker_id).ref_count = 1;
+  setup_engine(new_worker_id, FALSE);
+  if ((REMOTE_ThreadHandle(new_worker_id).ret = pthread_create(&REMOTE_ThreadHandle(new_worker_id).pthread_handle, NULL, thread_run, (void *)(&(REMOTE_ThreadHandle(new_worker_id).id)))) == 0) {
+    pthread_setspecific(Yap_yaamregs_key, (const void *)REMOTE_ThreadHandle(owid).current_yaam_regs);
     /* wait until the client is initialised */
     return TRUE;
   }
+  pthread_setspecific(Yap_yaamregs_key, (const void *)REMOTE_ThreadHandle(owid).current_yaam_regs);
   return FALSE;
 }
 
 static Int
-p_thread_sleep(void)
+p_thread_sleep( USES_REGS1 )
 {
   UInt time = IntegerOfTerm(Deref(ARG1));
 #if HAVE_NANOSLEEP
@@ -314,7 +397,7 @@ p_thread_sleep(void)
 }
 
 static Int
-p_thread_self(void)
+p_thread_self( USES_REGS1 )
 {
   if (pthread_getspecific(Yap_yaamregs_key) == NULL)
     return Yap_unify(MkIntegerTerm(-1), ARG1);
@@ -323,47 +406,48 @@ p_thread_self(void)
 
 
 static Int
-p_thread_zombie_self(void)
+p_thread_zombie_self( USES_REGS1 )
 {
   /* make sure the lock is available */
   if (pthread_getspecific(Yap_yaamregs_key) == NULL)
     return Yap_unify(MkIntegerTerm(-1), ARG1);
+  pthread_mutex_lock(&(LOCAL_ThreadHandle.tlock));
   DEBUG_TLOCK_ACCESS(4, worker_id);
-  pthread_mutex_lock(&(MY_ThreadHandle.tlock));
-  if (ActiveSignals &= YAP_ITI_SIGNAL) {
+  if (LOCAL_ActiveSignals &= YAP_ITI_SIGNAL) {
     DEBUG_TLOCK_ACCESS(5, worker_id);
-    pthread_mutex_unlock(&(MY_ThreadHandle.tlock));
+    pthread_mutex_unlock(&(LOCAL_ThreadHandle.tlock));
     return FALSE;
   }
   //  fprintf(stderr," -- %d\n", worker_id); 
-  MY_ThreadHandle.in_use = FALSE;
-  MY_ThreadHandle.zombie = TRUE;
+  LOCAL_ThreadHandle.in_use = FALSE;
+  LOCAL_ThreadHandle.zombie = TRUE;
   return Yap_unify(MkIntegerTerm(worker_id), ARG1);
 }
 
 static Int
-p_thread_status_lock(void)
+p_thread_status_lock( USES_REGS1 )
 {
   /* make sure the lock is available */
   if (pthread_getspecific(Yap_yaamregs_key) == NULL)
     return FALSE;
-  pthread_mutex_lock(&(MY_ThreadHandle.tlock_status));
+  pthread_mutex_lock(&(LOCAL_ThreadHandle.tlock_status));
   return Yap_unify(MkIntegerTerm(worker_id), ARG1);
 }
 
 static Int
-p_thread_status_unlock(void)
+p_thread_status_unlock( USES_REGS1 )
 {
   /* make sure the lock is available */
   if (pthread_getspecific(Yap_yaamregs_key) == NULL)
     return FALSE;
-  pthread_mutex_unlock(&(MY_ThreadHandle.tlock_status));
+  pthread_mutex_unlock(&(LOCAL_ThreadHandle.tlock_status));
   return Yap_unify(MkIntegerTerm(worker_id), ARG1);
 }
 
 Int
 Yap_thread_self(void)
 {
+  CACHE_REGS
   if (pthread_getspecific(Yap_yaamregs_key) == NULL)
     return -1;
   return worker_id;
@@ -377,8 +461,9 @@ Yap_thread_create_engine(thread_attr *ops)
   Term t = TermNil;
 
   /* 
-     ok, this creates a problem, because we are initializing an engine from some "empty" thread. 
-     We need first to foool the thread into believing it is the main thread
+     ok, this creates a problem, because we are initializing an engine from
+     some "empty" thread. 
+     We need first to fool the thread into believing it is the main thread
   */
   if (new_id == -1) {
     /* YAP ERROR */
@@ -391,20 +476,21 @@ Yap_thread_create_engine(thread_attr *ops)
     ops->sysize = 0;
     ops->egoal = t;
   }
-  if (pthread_self() != Yap_master_thread) {
+  if (!pthread_equal(pthread_self() , GLOBAL_master_thread) ) {
     /* we are worker_id 0 for now, lock master thread so that no one messes with us */ 
     pthread_setspecific(Yap_yaamregs_key, (const void *)&Yap_standard_regs);
-    pthread_mutex_lock(&(FOREIGN_ThreadHandle(0).tlock));
+    pthread_mutex_lock(&(REMOTE_ThreadHandle(0).tlock));
   }
   if (!init_thread_engine(new_id, ops->ssize, ops->tsize, ops->sysize, &t, &t, &(ops->egoal)))
     return -1;
-  FOREIGN_ThreadHandle(new_id).pthread_handle = 0L;
-  FOREIGN_ThreadHandle(new_id).id = new_id;
-  FOREIGN_ThreadHandle(new_id).ref_count = 0;
-  setup_engine(new_id, FALSE);
-  if (pthread_self() != Yap_master_thread) {
+  //REMOTE_ThreadHandle(new_id).pthread_handle = 0L;
+  REMOTE_ThreadHandle(new_id).id = new_id;
+  REMOTE_ThreadHandle(new_id).ref_count = 0;
+  if (!setup_engine(new_id, FALSE))
+    return -1;
+  if (!pthread_equal(pthread_self(), GLOBAL_master_thread)) {
     pthread_setspecific(Yap_yaamregs_key, NULL);
-    pthread_mutex_unlock(&(FOREIGN_ThreadHandle(0).tlock));
+    pthread_mutex_unlock(&(REMOTE_ThreadHandle(0).tlock));
   }
   return new_id;
 }
@@ -412,73 +498,74 @@ Yap_thread_create_engine(thread_attr *ops)
 Int
 Yap_thread_attach_engine(int wid)
 {
+  CACHE_REGS
   /* 
      already locked
-     pthread_mutex_lock(&(FOREIGN_ThreadHandle(wid).tlock));
+     pthread_mutex_lock(&(REMOTE_ThreadHandle(wid).tlock));
   */
-  if (FOREIGN_ThreadHandle(wid).ref_count ) {
+  if (REMOTE_ThreadHandle(wid).ref_count ) {
+    REMOTE_ThreadHandle(wid).ref_count++;
+    REMOTE_ThreadHandle(wid).pthread_handle = pthread_self();
     DEBUG_TLOCK_ACCESS(8, wid);
-    FOREIGN_ThreadHandle(wid).ref_count++;
-    FOREIGN_ThreadHandle(wid).pthread_handle = pthread_self();
-    pthread_mutex_unlock(&(FOREIGN_ThreadHandle(wid).tlock));
+    pthread_mutex_unlock(&(REMOTE_ThreadHandle(wid).tlock));
     return TRUE;
   }
-  FOREIGN_ThreadHandle(wid).pthread_handle = pthread_self();
-  FOREIGN_ThreadHandle(wid).ref_count++;
-  pthread_setspecific(Yap_yaamregs_key, (const void *)FOREIGN_ThreadHandle(wid).default_yaam_regs);
-  worker_id = wid;
+  REMOTE_ThreadHandle(wid).pthread_handle = pthread_self();
+  REMOTE_ThreadHandle(wid).ref_count++;
+  pthread_setspecific(Yap_yaamregs_key, (const void *)REMOTE_ThreadHandle(wid).current_yaam_regs);
+  REFRESH_CACHE_REGS;
   DEBUG_TLOCK_ACCESS(9, wid);
-  pthread_mutex_unlock(&(FOREIGN_ThreadHandle(wid).tlock));
+  pthread_mutex_unlock(&(REMOTE_ThreadHandle(wid).tlock));
   return TRUE;
 }
 
 Int
 Yap_thread_detach_engine(int wid)
 {
+  pthread_mutex_lock(&(REMOTE_ThreadHandle(wid).tlock));
   DEBUG_TLOCK_ACCESS(10, wid);
-  pthread_mutex_lock(&(FOREIGN_ThreadHandle(wid).tlock));
-  FOREIGN_ThreadHandle(wid).pthread_handle = 0;
-  FOREIGN_ThreadHandle(wid).ref_count--;
+  //REMOTE_ThreadHandle(wid).pthread_handle = 0;
+  REMOTE_ThreadHandle(wid).ref_count--;
   pthread_setspecific(Yap_yaamregs_key, NULL);
   DEBUG_TLOCK_ACCESS(11, wid);
-  pthread_mutex_unlock(&(FOREIGN_ThreadHandle(wid).tlock));
+  pthread_mutex_unlock(&(REMOTE_ThreadHandle(wid).tlock));
   return TRUE;
 }
 
 Int
 Yap_thread_destroy_engine(int wid)
 {
+  pthread_mutex_lock(&(REMOTE_ThreadHandle(wid).tlock));
   DEBUG_TLOCK_ACCESS(10, wid);
-  pthread_mutex_lock(&(FOREIGN_ThreadHandle(wid).tlock));
-  if (FOREIGN_ThreadHandle(wid).ref_count == 0) {
+  if (REMOTE_ThreadHandle(wid).ref_count == 0) {
     kill_thread_engine(wid, TRUE);
     return TRUE;
   } else {
     DEBUG_TLOCK_ACCESS(12, wid);
-    pthread_mutex_unlock(&(FOREIGN_ThreadHandle(wid).tlock));
+    pthread_mutex_unlock(&(REMOTE_ThreadHandle(wid).tlock));
     return FALSE;
   }
 }
 
 
 static Int
-p_thread_join(void)
+p_thread_join( USES_REGS1 )
 {
   Int tid = IntegerOfTerm(Deref(ARG1));
 
-  LOCK(ThreadHandlesLock);
-  if (!FOREIGN_ThreadHandle(tid).in_use &&
-      !FOREIGN_ThreadHandle(tid).zombie) {
-    UNLOCK(ThreadHandlesLock);
+  LOCK(GLOBAL_ThreadHandlesLock);
+  if (!REMOTE_ThreadHandle(tid).in_use &&
+      !REMOTE_ThreadHandle(tid).zombie) {
+    UNLOCK(GLOBAL_ThreadHandlesLock);
     return FALSE;
   }
-  if (!FOREIGN_ThreadHandle(tid).tdetach == MkAtomTerm(AtomTrue)) {
-    UNLOCK(ThreadHandlesLock);
+  if (!REMOTE_ThreadHandle(tid).tdetach == MkAtomTerm(AtomTrue)) {
+    UNLOCK(GLOBAL_ThreadHandlesLock);
     return FALSE;
   }
-  UNLOCK(ThreadHandlesLock);
+  UNLOCK(GLOBAL_ThreadHandlesLock);
   /* make sure this lock is accessible */
-  if (pthread_join(FOREIGN_ThreadHandle(tid).pthread_handle, NULL) < 0) {
+  if (pthread_join(REMOTE_ThreadHandle(tid).pthread_handle, NULL) < 0) {
     /* ERROR */
     return FALSE;
   }
@@ -487,59 +574,59 @@ p_thread_join(void)
 }
 
 static Int
-p_thread_destroy(void)
+p_thread_destroy( USES_REGS1 )
 {
   Int tid = IntegerOfTerm(Deref(ARG1));
 
-  LOCK(ThreadHandlesLock);
-  FOREIGN_ThreadHandle(tid).zombie = FALSE;
-  FOREIGN_ThreadHandle(tid).in_use = FALSE;
+  LOCK(GLOBAL_ThreadHandlesLock);
+  REMOTE_ThreadHandle(tid).zombie = FALSE;
+  REMOTE_ThreadHandle(tid).in_use = FALSE;
   DEBUG_TLOCK_ACCESS(32, tid);
-  pthread_mutex_unlock(&(FOREIGN_ThreadHandle(tid).tlock));
-  UNLOCK(ThreadHandlesLock);
+  pthread_mutex_unlock(&(REMOTE_ThreadHandle(tid).tlock));
+  UNLOCK(GLOBAL_ThreadHandlesLock);
   return TRUE;
 }
 
 static Int
-p_thread_detach(void)
+p_thread_detach( USES_REGS1 )
 {
   Int tid = IntegerOfTerm(Deref(ARG1));
-  pthread_mutex_lock(&(FOREIGN_ThreadHandle(tid).tlock));
+  pthread_mutex_lock(&(REMOTE_ThreadHandle(tid).tlock));
   DEBUG_TLOCK_ACCESS(14, tid);
-  if (pthread_detach(FOREIGN_ThreadHandle(tid).pthread_handle) < 0) {
+  if (pthread_detach(REMOTE_ThreadHandle(tid).pthread_handle) < 0) {
     /* ERROR */
     DEBUG_TLOCK_ACCESS(15, tid);
-    pthread_mutex_unlock(&(FOREIGN_ThreadHandle(tid).tlock));
+    pthread_mutex_unlock(&(REMOTE_ThreadHandle(tid).tlock));
     return FALSE;
   }
-  FOREIGN_ThreadHandle(tid).tdetach = 
+  REMOTE_ThreadHandle(tid).tdetach = 
     MkAtomTerm(AtomTrue);
   DEBUG_TLOCK_ACCESS(30, tid);
-  pthread_mutex_unlock(&(FOREIGN_ThreadHandle(tid).tlock));
+  pthread_mutex_unlock(&(REMOTE_ThreadHandle(tid).tlock));
   return TRUE;
 }
 
 static Int
-p_thread_detached(void)
+p_thread_detached( USES_REGS1 )
 {
-  if (MY_ThreadHandle.tdetach)
-    return Yap_unify(ARG1,MY_ThreadHandle.tdetach);
+  if (LOCAL_ThreadHandle.tdetach)
+    return Yap_unify(ARG1,LOCAL_ThreadHandle.tdetach);
   else
     return FALSE;
 }
 
 static Int
-p_thread_detached2(void)
+p_thread_detached2( USES_REGS1 )
 {
   Int tid = IntegerOfTerm(Deref(ARG1));
-  if (FOREIGN_ThreadHandle(tid).tdetach)
-    return Yap_unify(ARG2,FOREIGN_ThreadHandle(tid).tdetach);
+  if (REMOTE_ThreadHandle(tid).tdetach)
+    return Yap_unify(ARG2,REMOTE_ThreadHandle(tid).tdetach);
   else
     return FALSE;
 }
 
 static Int
-p_thread_exit(void)
+p_thread_exit( USES_REGS1 )
 {
   thread_die(worker_id, FALSE); 
   pthread_exit(NULL);
@@ -548,7 +635,7 @@ p_thread_exit(void)
 }
 
 static Int
-p_thread_set_concurrency(void)
+p_thread_set_concurrency( USES_REGS1 )
 {
   Term tnew = Deref(ARG2);
   int newc, cur;
@@ -569,7 +656,7 @@ p_thread_set_concurrency(void)
 }
 
 static Int
-p_thread_yield(void)
+p_thread_yield( USES_REGS1 )
 {
   if (sched_yield() != 0) {
     return FALSE;
@@ -578,10 +665,10 @@ p_thread_yield(void)
 }
 
 static Int
-p_valid_thread(void)
+p_valid_thread( USES_REGS1 )
 {
   Int i = IntegerOfTerm(Deref(ARG1)); 
-  return FOREIGN_ThreadHandle(i).in_use || FOREIGN_ThreadHandle(i).zombie;
+  return REMOTE_ThreadHandle(i).in_use || REMOTE_ThreadHandle(i).zombie;
 }
 
 /* Mutex Support */
@@ -593,10 +680,13 @@ typedef struct swi_mutex {
 } SWIMutex;
 
 static Int
-p_new_mutex(void)
+p_new_mutex( USES_REGS1 )
 {
   SWIMutex* mutp;
   pthread_mutexattr_t mat;
+#if defined(HAVE_PTHREAD_MUTEXATTR_SETKIND_NP) && !defined(__MINGW32__)
+  extern int pthread_mutexattr_setkind_np(pthread_mutexattr_t *attr, int kind);
+#endif
 
   mutp = (SWIMutex *)Yap_AllocCodeSpace(sizeof(SWIMutex));
   if (mutp == NULL) {
@@ -617,7 +707,7 @@ p_new_mutex(void)
 }
 
 static Int
-p_destroy_mutex(void)
+p_destroy_mutex( USES_REGS1 )
 {
   SWIMutex *mut = (SWIMutex*)IntegerOfTerm(Deref(ARG1));
 
@@ -628,7 +718,7 @@ p_destroy_mutex(void)
 }
 
 static Int
-p_lock_mutex(void)
+p_lock_mutex( USES_REGS1 )
 {
   SWIMutex *mut = (SWIMutex*)IntegerOfTerm(Deref(ARG1));
 
@@ -640,7 +730,7 @@ p_lock_mutex(void)
 }
 
 static Int
-p_trylock_mutex(void)
+p_trylock_mutex( USES_REGS1 )
 {
   SWIMutex *mut = (SWIMutex*)IntegerOfTerm(Deref(ARG1));
 
@@ -652,7 +742,7 @@ p_trylock_mutex(void)
 }
 
 static Int
-p_unlock_mutex(void)
+p_unlock_mutex( USES_REGS1 )
 {
   SWIMutex *mut = (SWIMutex*)IntegerOfTerm(Deref(ARG1));
 
@@ -663,7 +753,7 @@ p_unlock_mutex(void)
 }
 
 static Int
-p_mutex_info(void)
+p_mutex_info( USES_REGS1 )
 {
   SWIMutex *mut = (SWIMutex*)IntegerOfTerm(Deref(ARG1));
 
@@ -673,7 +763,7 @@ p_mutex_info(void)
 }
 
 static Int
-p_cond_create(void)
+p_cond_create( USES_REGS1 )
 {
   pthread_cond_t* condp;
 
@@ -686,7 +776,7 @@ p_cond_create(void)
 }
 
 static Int
-p_cond_destroy(void)
+p_cond_destroy( USES_REGS1 )
 {
   pthread_cond_t *condp = (pthread_cond_t *)IntegerOfTerm(Deref(ARG1));
 
@@ -697,7 +787,7 @@ p_cond_destroy(void)
 }
 
 static Int
-p_cond_signal(void)
+p_cond_signal( USES_REGS1 )
 {
   pthread_cond_t *condp = (pthread_cond_t *)IntegerOfTerm(Deref(ARG1));
 
@@ -707,7 +797,7 @@ p_cond_signal(void)
 }
 
 static Int
-p_cond_broadcast(void)
+p_cond_broadcast( USES_REGS1 )
 {
   pthread_cond_t *condp = (pthread_cond_t *)IntegerOfTerm(Deref(ARG1));
 
@@ -717,7 +807,7 @@ p_cond_broadcast(void)
 }
 
 static Int
-p_cond_wait(void)
+p_cond_wait( USES_REGS1 )
 {
   pthread_cond_t *condp = (pthread_cond_t *)IntegerOfTerm(Deref(ARG1));
   SWIMutex *mut = (SWIMutex*)IntegerOfTerm(Deref(ARG2));
@@ -727,178 +817,179 @@ p_cond_wait(void)
 }
 
 static Int 
-p_thread_stacks(void)
+p_thread_stacks( USES_REGS1 )
 {				/* '$thread_signal'(+P)	 */
   Int tid = IntegerOfTerm(Deref(ARG1));
   Int status= TRUE;
 
-  LOCK(ThreadHandlesLock);
-  if (!FOREIGN_ThreadHandle(tid).in_use &&
-      !FOREIGN_ThreadHandle(tid).zombie) {
-    UNLOCK(ThreadHandlesLock);
+  LOCK(GLOBAL_ThreadHandlesLock);
+  if (!Yap_local[tid] || 
+      (!REMOTE_ThreadHandle(tid).in_use && !REMOTE_ThreadHandle(tid).zombie)) {
+    UNLOCK(GLOBAL_ThreadHandlesLock);
     return FALSE;
   }
-  status &= Yap_unify(ARG2,MkIntegerTerm(FOREIGN_ThreadHandle(tid).ssize));
-  status &= Yap_unify(ARG3,MkIntegerTerm(FOREIGN_ThreadHandle(tid).tsize));
-  status &= Yap_unify(ARG4,MkIntegerTerm(FOREIGN_ThreadHandle(tid).sysize));
-  UNLOCK(ThreadHandlesLock);
+  status &= Yap_unify(ARG2,MkIntegerTerm(REMOTE_ThreadHandle(tid).ssize));
+  status &= Yap_unify(ARG3,MkIntegerTerm(REMOTE_ThreadHandle(tid).tsize));
+  status &= Yap_unify(ARG4,MkIntegerTerm(REMOTE_ThreadHandle(tid).sysize));
+  UNLOCK(GLOBAL_ThreadHandlesLock);
   return status;
 }
 
 static Int 
-p_thread_atexit(void)
+p_thread_atexit( USES_REGS1 )
 {				/* '$thread_signal'(+P)	 */
   Term t;
 
-  if (!MY_ThreadHandle.texit ||
-      MY_ThreadHandle.texit->Entry == MkAtomTerm(AtomTrue)) {
+  if (!LOCAL_ThreadHandle.texit ||
+      LOCAL_ThreadHandle.texit->Entry == MkAtomTerm(AtomTrue)) {
     return FALSE;
   }
   do {
-    t = Yap_PopTermFromDB(MY_ThreadHandle.texit);
-    MY_ThreadHandle.texit = NULL;
+    t = Yap_PopTermFromDB(LOCAL_ThreadHandle.texit);
+    LOCAL_ThreadHandle.texit = NULL;
     if (t == 0) {
-      if (Yap_Error_TYPE == OUT_OF_ATTVARS_ERROR) {
-	Yap_Error_TYPE = YAP_NO_ERROR;
+      if (LOCAL_Error_TYPE == OUT_OF_ATTVARS_ERROR) {
+	LOCAL_Error_TYPE = YAP_NO_ERROR;
 	if (!Yap_growglobal(NULL)) {
-	  Yap_Error(OUT_OF_ATTVARS_ERROR, TermNil, Yap_ErrorMessage);
+	  Yap_Error(OUT_OF_ATTVARS_ERROR, TermNil, LOCAL_ErrorMessage);
 	  thread_die(worker_id, FALSE);
 	  return FALSE;
 	}
       } else {
-	Yap_Error_TYPE = YAP_NO_ERROR;
-	if (!Yap_growstack(MY_ThreadHandle.tgoal->NOfCells*CellSize)) {
-	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, Yap_ErrorMessage);
+	LOCAL_Error_TYPE = YAP_NO_ERROR;
+	if (!Yap_growstack(LOCAL_ThreadHandle.tgoal->NOfCells*CellSize)) {
+	  Yap_Error(OUT_OF_STACK_ERROR, TermNil, LOCAL_ErrorMessage);
 	  thread_die(worker_id, FALSE);
 	  return FALSE;
 	}
       }
     }
   } while (t == 0);
-  return Yap_unify(ARG1, t) && Yap_unify(ARG2, MY_ThreadHandle.texit_mod);
+  return Yap_unify(ARG1, t) && Yap_unify(ARG2, LOCAL_ThreadHandle.texit_mod);
 }
 
 
 
 static Int 
-p_thread_signal(void)
+p_thread_signal( USES_REGS1 )
 {				/* '$thread_signal'(+P)	 */
   Int wid = IntegerOfTerm(Deref(ARG1));
   /* make sure the lock is available */
-  pthread_mutex_lock(&(FOREIGN_ThreadHandle(wid).tlock));
+  pthread_mutex_lock(&(REMOTE_ThreadHandle(wid).tlock));
   DEBUG_TLOCK_ACCESS(16, wid);
-  if (!FOREIGN_ThreadHandle(wid).in_use || 
-      !FOREIGN_ThreadHandle(wid).current_yaam_regs) {
+  if (!REMOTE_ThreadHandle(wid).in_use || 
+      !REMOTE_ThreadHandle(wid).current_yaam_regs) {
     DEBUG_TLOCK_ACCESS(17, wid);
-    pthread_mutex_unlock(&(FOREIGN_ThreadHandle(wid).tlock));
+    pthread_mutex_unlock(&(REMOTE_ThreadHandle(wid).tlock));
    return TRUE;
   }
-  LOCK(FOREIGN_WL(wid)->signal_lock);
-  FOREIGN_ThreadHandle(wid).current_yaam_regs->CreepFlag_ = 
-    Unsigned(FOREIGN_ThreadHandle(wid).current_yaam_regs->LCL0_);
-  FOREIGN_WL(wid)->active_signals |= YAP_ITI_SIGNAL;
-  UNLOCK(FOREIGN_WL(wid)->signal_lock);
+  LOCK(REMOTE_SignalLock(wid));
+  REMOTE_ThreadHandle(wid).current_yaam_regs->CreepFlag_ = 
+    Unsigned(REMOTE_ThreadHandle(wid).current_yaam_regs->LCL0_);
+  REMOTE_ActiveSignals(wid) |= YAP_ITI_SIGNAL;
+  UNLOCK(REMOTE_SignalLock(wid));
   DEBUG_TLOCK_ACCESS(18, wid);
-  pthread_mutex_unlock(&(FOREIGN_ThreadHandle(wid).tlock));
+  pthread_mutex_unlock(&(REMOTE_ThreadHandle(wid).tlock));
   return TRUE;
 }
 
 static Int 
-p_no_threads(void)
+p_no_threads( USES_REGS1 )
 {				/* '$thread_signal'(+P)	 */
   return FALSE;
 }
 
 static Int 
-p_nof_threads(void)
+p_nof_threads( USES_REGS1 )
 {				/* '$nof_threads'(+P)	 */
   int i = 0, wid;
-  LOCK(ThreadHandlesLock);
+  LOCK(GLOBAL_ThreadHandlesLock);
   for (wid = 0; wid < MAX_THREADS; wid++) {
-    if (FOREIGN_ThreadHandle(wid).in_use)
+    if (!Yap_local[wid]) break;
+    if (REMOTE_ThreadHandle(wid).in_use)
       i++;
   }
-  UNLOCK(ThreadHandlesLock);
+  UNLOCK(GLOBAL_ThreadHandlesLock);
   return Yap_unify(ARG1,MkIntegerTerm(i));
 }
 
 static Int 
-p_max_workers(void)
+p_max_workers( USES_REGS1 )
 {				/* '$max_workers'(+P)	 */
   return Yap_unify(ARG1,MkIntegerTerm(MAX_WORKERS));
 }
 
 static Int 
-p_max_threads(void)
+p_max_threads( USES_REGS1 )
 {				/* '$max_threads'(+P)	 */
   return Yap_unify(ARG1,MkIntegerTerm(MAX_THREADS));
 }
 
 static Int 
-p_nof_threads_created(void)
+p_nof_threads_created( USES_REGS1 )
 {				/* '$nof_threads'(+P)	 */
-  return Yap_unify(ARG1,MkIntTerm(NOfThreadsCreated));
+  return Yap_unify(ARG1,MkIntTerm(GLOBAL_NOfThreadsCreated));
 }
 
 static Int 
-p_thread_runtime(void)
+p_thread_runtime( USES_REGS1 )
 {				/* '$thread_runtime'(+P)	 */
-  return Yap_unify(ARG1,MkIntegerTerm(ThreadsTotalTime));
+  return Yap_unify(ARG1,MkIntegerTerm(GLOBAL_ThreadsTotalTime));
 }
 
 static Int 
-p_thread_self_lock(void)
+p_thread_self_lock( USES_REGS1 )
 {				/* '$thread_unlock'	 */
-  pthread_mutex_lock(&(MY_ThreadHandle.tlock));
+  pthread_mutex_lock(&(LOCAL_ThreadHandle.tlock));
   return Yap_unify(ARG1,MkIntegerTerm(worker_id));
 }
 
 static Int 
-p_thread_unlock(void)
+p_thread_unlock( USES_REGS1 )
 {				/* '$thread_unlock'	 */
   Int wid = IntegerOfTerm(Deref(ARG1));
   DEBUG_TLOCK_ACCESS(19, wid);
-  pthread_mutex_unlock(&(FOREIGN_ThreadHandle(wid).tlock));
+  pthread_mutex_unlock(&(REMOTE_ThreadHandle(wid).tlock));
   return TRUE;
 }
 
 void Yap_InitThreadPreds(void)
 {
-  Yap_InitCPred("$no_threads", 0, p_no_threads, HiddenPredFlag);
-  Yap_InitCPred("$max_workers", 1, p_max_workers, HiddenPredFlag);
-  Yap_InitCPred("$max_threads", 1, p_max_threads, HiddenPredFlag);
-  Yap_InitCPred("$thread_new_tid", 1, p_thread_new_tid, HiddenPredFlag);
-  Yap_InitCPred("$create_thread", 7, p_create_thread, HiddenPredFlag);
-  Yap_InitCPred("$thread_self", 1, p_thread_self, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$thread_status_lock", 1, p_thread_status_lock, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$thread_status_unlock", 1, p_thread_status_unlock, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$thread_zombie_self", 1, p_thread_zombie_self, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$thread_join", 1, p_thread_join, HiddenPredFlag);
-  Yap_InitCPred("$thread_destroy", 1, p_thread_destroy, HiddenPredFlag);
+  Yap_InitCPred("$no_threads", 0, p_no_threads, 0);
+  Yap_InitCPred("$max_workers", 1, p_max_workers, 0);
+  Yap_InitCPred("$max_threads", 1, p_max_threads, 0);
+  Yap_InitCPred("$thread_new_tid", 1, p_thread_new_tid, 0);
+  Yap_InitCPred("$create_thread", 7, p_create_thread, 0);
+  Yap_InitCPred("$thread_self", 1, p_thread_self, SafePredFlag);
+  Yap_InitCPred("$thread_status_lock", 1, p_thread_status_lock, SafePredFlag);
+  Yap_InitCPred("$thread_status_unlock", 1, p_thread_status_unlock, SafePredFlag);
+  Yap_InitCPred("$thread_zombie_self", 1, p_thread_zombie_self, SafePredFlag);
+  Yap_InitCPred("$thread_join", 1, p_thread_join, 0);
+  Yap_InitCPred("$thread_destroy", 1, p_thread_destroy, 0);
   Yap_InitCPred("thread_yield", 0, p_thread_yield, 0);
-  Yap_InitCPred("$detach_thread", 1, p_thread_detach, HiddenPredFlag);
-  Yap_InitCPred("$thread_detached", 1, p_thread_detached, HiddenPredFlag);
-  Yap_InitCPred("$thread_detached", 2, p_thread_detached2, HiddenPredFlag);
-  Yap_InitCPred("$thread_exit", 0, p_thread_exit, HiddenPredFlag);
+  Yap_InitCPred("$detach_thread", 1, p_thread_detach, 0);
+  Yap_InitCPred("$thread_detached", 1, p_thread_detached, 0);
+  Yap_InitCPred("$thread_detached", 2, p_thread_detached2, 0);
+  Yap_InitCPred("$thread_exit", 0, p_thread_exit, 0);
   Yap_InitCPred("thread_setconcurrency", 2, p_thread_set_concurrency, 0);
-  Yap_InitCPred("$valid_thread", 1, p_valid_thread, HiddenPredFlag);
-  Yap_InitCPred("$new_mutex", 1, p_new_mutex, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$destroy_mutex", 1, p_destroy_mutex, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$lock_mutex", 1, p_lock_mutex, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$trylock_mutex", 1, p_trylock_mutex, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$unlock_mutex", 1, p_unlock_mutex, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$mutex_info", 3, p_mutex_info, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$cond_create", 1, p_cond_create, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$cond_destroy", 1, p_cond_destroy, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$cond_signal", 1, p_cond_signal, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$cond_broadcast", 1, p_cond_broadcast, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$cond_wait", 2, p_cond_wait, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$thread_stacks", 4, p_thread_stacks, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$signal_thread", 1, p_thread_signal, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$nof_threads", 1, p_nof_threads, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$nof_threads_created", 1, p_nof_threads_created, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$thread_sleep", 4, p_thread_sleep, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$thread_runtime", 1, p_thread_runtime, SafePredFlag|HiddenPredFlag);
+  Yap_InitCPred("$valid_thread", 1, p_valid_thread, 0);
+  Yap_InitCPred("$new_mutex", 1, p_new_mutex, SafePredFlag);
+  Yap_InitCPred("$destroy_mutex", 1, p_destroy_mutex, SafePredFlag);
+  Yap_InitCPred("$lock_mutex", 1, p_lock_mutex, SafePredFlag);
+  Yap_InitCPred("$trylock_mutex", 1, p_trylock_mutex, SafePredFlag);
+  Yap_InitCPred("$unlock_mutex", 1, p_unlock_mutex, SafePredFlag);
+  Yap_InitCPred("$mutex_info", 3, p_mutex_info, SafePredFlag);
+  Yap_InitCPred("$cond_create", 1, p_cond_create, SafePredFlag);
+  Yap_InitCPred("$cond_destroy", 1, p_cond_destroy, SafePredFlag);
+  Yap_InitCPred("$cond_signal", 1, p_cond_signal, SafePredFlag);
+  Yap_InitCPred("$cond_broadcast", 1, p_cond_broadcast, SafePredFlag);
+  Yap_InitCPred("$cond_wait", 2, p_cond_wait, SafePredFlag);
+  Yap_InitCPred("$thread_stacks", 4, p_thread_stacks, SafePredFlag);
+  Yap_InitCPred("$signal_thread", 1, p_thread_signal, SafePredFlag);
+  Yap_InitCPred("$nof_threads", 1, p_nof_threads, SafePredFlag);
+  Yap_InitCPred("$nof_threads_created", 1, p_nof_threads_created, SafePredFlag);
+  Yap_InitCPred("$thread_sleep", 4, p_thread_sleep, SafePredFlag);
+  Yap_InitCPred("$thread_runtime", 1, p_thread_runtime, SafePredFlag);
   Yap_InitCPred("$thread_self_lock", 1, p_thread_self_lock, SafePredFlag);
   Yap_InitCPred("$thread_run_at_exit", 2, p_thread_atexit, SafePredFlag);
   Yap_InitCPred("$thread_unlock", 1, p_thread_unlock, SafePredFlag);
@@ -962,14 +1053,14 @@ p_max_workers(void)
 
 void Yap_InitThreadPreds(void)
 {
-  Yap_InitCPred("$max_workers", 1, p_max_workers, HiddenPredFlag);
-  Yap_InitCPred("$thread_self", 1, p_thread_self, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$no_threads", 0, p_no_threads, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$max_threads", 1, p_max_threads, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$nof_threads", 1, p_nof_threads, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$nof_threads_created", 1, p_nof_threads_created, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$thread_stacks", 4, p_thread_stacks, SafePredFlag|HiddenPredFlag);
-  Yap_InitCPred("$thread_runtime", 1, p_thread_runtime, SafePredFlag|HiddenPredFlag);
+  Yap_InitCPred("$max_workers", 1, p_max_workers, 0);
+  Yap_InitCPred("$thread_self", 1, p_thread_self, SafePredFlag);
+  Yap_InitCPred("$no_threads", 0, p_no_threads, SafePredFlag);
+  Yap_InitCPred("$max_threads", 1, p_max_threads, SafePredFlag);
+  Yap_InitCPred("$nof_threads", 1, p_nof_threads, SafePredFlag);
+  Yap_InitCPred("$nof_threads_created", 1, p_nof_threads_created, SafePredFlag);
+  Yap_InitCPred("$thread_stacks", 4, p_thread_stacks, SafePredFlag);
+  Yap_InitCPred("$thread_runtime", 1, p_thread_runtime, SafePredFlag);
   Yap_InitCPred("$thread_unlock", 1, p_thread_unlock, SafePredFlag);
 }
 
