@@ -16,13 +16,15 @@
 *************************************************************************/
 #define IN_UNIFY_C 1
 
+#define HAS_CACHE_REGS 1
+
 #include "absmi.h"
 
 STD_PROTO(int   Yap_rational_tree_loop, (CELL *, CELL *, CELL **, CELL **));
 
 STATIC_PROTO(int    OCUnify_complex, (CELL *, CELL *, CELL *));
 STATIC_PROTO(int    OCUnify, (register CELL, register CELL));
-STATIC_PROTO(Int   p_ocunify, (void));
+STATIC_PROTO(Int   p_ocunify, ( USES_REGS1 ));
 #ifdef THREADED_CODE
 STATIC_PROTO(int    rtable_hash_op, (OPCODE));
 STATIC_PROTO(void   InitReverseLookupOpcode, (void));
@@ -35,7 +37,6 @@ STATIC_PROTO(void   InitReverseLookupOpcode, (void));
 int
 Yap_rational_tree_loop(CELL *pt0, CELL *pt0_end, CELL **to_visit, CELL **to_visit_max)
 {
-
   CELL ** base = to_visit;
 rtree_loop:
   while (pt0 < pt0_end) {
@@ -53,7 +54,7 @@ rtree_loop:
       if (IsPairTerm(d0)) {
 	to_visit -= 3;
 	if (to_visit < to_visit_max) {
-	  to_visit = Yap_shift_visit(to_visit, &to_visit_max);
+	  to_visit = Yap_shift_visit(to_visit, &to_visit_max, &base);
 	}
 	to_visit[0] = pt0;
 	to_visit[1] = pt0_end;
@@ -75,7 +76,7 @@ rtree_loop:
 	}
 	to_visit -= 3;
 	if (to_visit < to_visit_max) {
-	  to_visit = Yap_shift_visit(to_visit, &to_visit_max);
+	  to_visit = Yap_shift_visit(to_visit, &to_visit_max, &base);
 	}
 	to_visit[0] = pt0;
 	to_visit[1] = pt0_end;
@@ -114,6 +115,7 @@ cufail:
 
 static inline int
 rational_tree(Term d0) {
+CACHE_REGS
   CELL  **to_visit_max = (CELL **)AuxBase, **to_visit  = (CELL **)AuxSp;
 
   if (IsPairTerm(d0)) {
@@ -134,6 +136,7 @@ rational_tree(Term d0) {
 static int 
 OCUnify_complex(CELL *pt0, CELL *pt0_end, CELL *pt1)
 {
+CACHE_REGS
 #ifdef THREADS
 #undef Yap_REGS
   register REGSTORE *regp = Yap_regp;
@@ -189,7 +192,7 @@ loop:
 #endif
 	  if ((void *)to_visit < (void *)unif) {
 	    CELL **urec = (CELL **)unif;
-	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec);
+	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec, NULL);
 	    unif = (struct unif_record *)urec;
 	  }
 	  to_visit->start0 = pt0;
@@ -234,7 +237,7 @@ loop:
 #endif
 	  if ((void *)to_visit < (void *)unif) {
 	    CELL **urec = (CELL **)unif;
-	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec);
+	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec, NULL);
 	    unif = (struct unif_record *)urec;
 	  }
 	  to_visit->start0 = pt0;
@@ -256,12 +259,7 @@ loop:
 
       derefa_body(d1, ptd1, unify_comp_nvar_unk, unify_comp_nvar_nvar);
       /* d1 and pt2 have the unbound value, whereas d0 is bound */
-      BIND_GLOBAL(ptd1, d0, bind_ocunify1);
-#ifdef COROUTINING
-      DO_TRAIL(ptd1, d0);
-      if (IsAttVar(ptd1)) Yap_WakeUp(ptd1);
-    bind_ocunify1:
-#endif
+      Bind_Global(ptd1, d0);
       if (Yap_rational_tree_loop(ptd1-1, ptd1, (CELL **)to_visit, (CELL **)unif))
 	goto cufail;
       continue;
@@ -279,12 +277,7 @@ loop:
       deref_head(d1, unify_comp_var_unk);
     unify_comp_var_nvar:
       /* pt2 is unbound and d1 is bound */
-      BIND_GLOBAL(ptd0, d1, bind_ocunify2);
-#ifdef COROUTINING
-      DO_TRAIL(ptd0, d1);
-      if (IsAttVar(ptd0)) Yap_WakeUp(ptd0);
-    bind_ocunify2:
-#endif
+      Bind_Global(ptd0, d1);
       if (Yap_rational_tree_loop(ptd0-1, ptd0, (CELL **)to_visit, (CELL **)unif))
 	goto cufail;
       continue;
@@ -339,7 +332,7 @@ cufail:
 static int 
 OCUnify(register CELL d0, register CELL d1)
 {
-
+CACHE_REGS
   register CELL *pt0, *pt1;
 
 #if SHADOW_HB
@@ -400,12 +393,7 @@ oc_unify_nvar_nvar:
 
   deref_body(d1, pt1, oc_unify_nvar_unk, oc_unify_nvar_nvar);
   /* d0 is bound and d1 is unbound */
-  BIND(pt1, d0, bind_ocunify4);
-#ifdef COROUTINING
-  DO_TRAIL(pt1, d0);
-  if (IsAttVar(pt1)) Yap_WakeUp(pt1);
- bind_ocunify4:
-#endif
+  Bind(pt1, d0);
   /* local variables cannot be in a term */
   if (pt1 > H && pt1 < LCL0)
     return TRUE;
@@ -418,12 +406,7 @@ oc_unify_nvar_nvar:
   deref_head(d1, oc_unify_var_unk);
 oc_unify_var_nvar:
   /* pt0 is unbound and d1 is bound */
-  BIND(pt0, d1, bind_ocunify5);
-#ifdef COROUTINING
-  DO_TRAIL(pt0, d1);
-  if (IsAttVar(pt0)) Yap_WakeUp(pt0);
- bind_ocunify5:
-#endif
+  Bind(pt0, d1);
   /* local variables cannot be in a term */
   if (pt0 > H && pt0 < LCL0)
     return TRUE;
@@ -433,31 +416,19 @@ oc_unify_var_nvar:
 
   deref_body(d1, pt1, oc_unify_var_unk, oc_unify_var_nvar);
   /* d0 and pt1 are unbound */
-  UnifyCells(pt0, pt1, uc1, uc2);
-#ifdef COROUTINING
-  DO_TRAIL(pt0, (CELL)pt1);
-  if (IsAttVar(pt0)) Yap_WakeUp(pt0);
- uc1:
-#endif
+  UnifyCells(pt0, pt1);
   return (TRUE);
-#ifdef COROUTINING
- uc2:
-  DO_TRAIL(pt1, (CELL)pt0);
-  if (IsAttVar(pt1)) {
-    Yap_WakeUp(pt1);
-  }
-#endif
   return (TRUE);
 }
 
 static Int
-p_ocunify(void)
+p_ocunify( USES_REGS1 )
 {
   return(OCUnify(ARG1,ARG2));
 }
 
 static Int
-p_cyclic(void)
+p_cyclic( USES_REGS1 )
 {
   Term t = Deref(ARG1);
   if (IsVarTerm(t))
@@ -465,8 +436,13 @@ p_cyclic(void)
   return rational_tree(t);
 }
 
+int Yap_IsAcyclicTerm(Term t)
+{
+  return !rational_tree(t);
+}
+
 static Int
-p_acyclic(void)
+p_acyclic( USES_REGS1 )
 {
   Term t = Deref(ARG1);
   if (IsVarTerm(t))
@@ -477,6 +453,7 @@ p_acyclic(void)
 int 
 Yap_IUnify(register CELL d0, register CELL d1)
 {
+CACHE_REGS
 #if THREADS
 #undef Yap_REGS
   register REGSTORE *regp = Yap_regp;
@@ -547,12 +524,7 @@ unify_nvar_nvar:
 
   deref_body(d1, pt1, unify_nvar_unk, unify_nvar_nvar);
   /* d0 is bound and d1 is unbound */
-  BIND(pt1, d0, bind_unify3);
-#ifdef COROUTINING
-  DO_TRAIL(pt1, d0);
-  if (IsAttVar(pt1)) Yap_WakeUp(pt1);
- bind_unify3:
-#endif
+  Bind(pt1, d0);
   return (TRUE);
 
   deref_body(d0, pt0, unify_unk, unify_nvar);
@@ -560,12 +532,7 @@ unify_nvar_nvar:
   deref_head(d1, unify_var_unk);
 unify_var_nvar:
   /* pt0 is unbound and d1 is bound */
-  BIND(pt0, d1, bind_unify4);
-#ifdef COROUTINING
-  DO_TRAIL(pt0, d1);
-  if (IsAttVar(pt0)) Yap_WakeUp(pt0);
- bind_unify4:
-#endif
+  Bind(pt0, d1);
   return TRUE;
 
 #if TRAILING_REQUIRES_BRANCH
@@ -576,21 +543,9 @@ unify_var_nvar_trail:
 
   deref_body(d1, pt1, unify_var_unk, unify_var_nvar);
   /* d0 and pt1 are unbound */
-  UnifyCells(pt0, pt1, uc1, uc2);
-#ifdef COROUTINING
-  DO_TRAIL(pt0, (CELL)pt1);
-  if (IsAttVar(pt0)) Yap_WakeUp(pt0);
- uc1:
-#endif
+  UnifyCells(pt0, pt1);
   return (TRUE);
-#ifdef COROUTINING
- uc2:
-  DO_TRAIL(pt1, (CELL)pt0);
-  if (IsAttVar(pt1)) {
-    Yap_WakeUp(pt1);
-  }
-  return (TRUE);
-#endif
+
 #if THREADS
 #undef Yap_REGS
 #define Yap_REGS (*Yap_regp)  
@@ -628,6 +583,7 @@ InitReverseLookupOpcode(void)
       }
     }
   }
+  bzero(OP_RTABLE, sz);
   opeptr = OP_RTABLE;
   /* clear up table */
   {
@@ -655,16 +611,19 @@ InitReverseLookupOpcode(void)
 }
 #endif
 
-#define UnifiableGlobalCells(a, b)                                    \
-     if((a) > (b)) {                                              \
-	BIND_GLOBALCELL_NONATT((a),(CELL)(b));                           \
-     } else if((a) < (b)){                                        \
-	BIND_GLOBALCELL_NONATT((b),(CELL) (a));                          \
+#define UnifyAndTrailGlobalCells(a, b)                                \
+     if((a) > (b)) {                                                  \
+       *(a) = (CELL)(b);                                              \
+        DO_TRAIL((a), (CELL)(b));                                     \
+     } else if((a) < (b)){                                            \
+       *(b) = (CELL)(a);                                              \
+        DO_TRAIL((b), (CELL)(a));                                     \
      }
 
 static int 
 unifiable_complex(CELL *pt0, CELL *pt0_end, CELL *pt1)
 {
+CACHE_REGS
 #ifdef THREADS
 #undef Yap_REGS
   register REGSTORE *regp = Yap_regp;
@@ -718,7 +677,7 @@ loop:
 #endif
 	  if ((void *)to_visit < (void *)unif) {
 	    CELL **urec = (CELL **)unif;
-	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec);
+	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec, NULL);
 	    unif = (struct unif_record *)urec;
 	  }
 	  to_visit->start0 = pt0;
@@ -763,7 +722,7 @@ loop:
 #endif
 	  if ((void *)to_visit < (void *)unif) {
 	    CELL **urec = (CELL **)unif;
-	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec);
+	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec, NULL);
 	    unif = (struct unif_record *)urec;
 	  }
 	  to_visit->start0 = pt0;
@@ -785,11 +744,8 @@ loop:
 
       derefa_body(d1, ptd1, unifiable_comp_nvar_unk, unifiable_comp_nvar_nvar);
 	/* d1 and pt2 have the unbound value, whereas d0 is bound */
-      BIND(ptd1, d0, bind_unifiable3);
-#ifdef COROUTINING
+      *(ptd1) = d0;
       DO_TRAIL(ptd1, d0);
-    bind_unifiable3:
-#endif
       continue;
     }
 
@@ -805,16 +761,13 @@ loop:
       deref_head(d1, unifiable_comp_var_unk);
     unifiable_comp_var_nvar:
       /* pt2 is unbound and d1 is bound */
-      BIND(ptd0, d1, bind_unifiable4);
-#ifdef COROUTINING
-      DO_TRAIL(ptd0, d1);
-    bind_unifiable4:
-#endif
+      *ptd0 = d1;
+       DO_TRAIL(ptd0, d1);
       continue;
 
       derefa_body(d1, ptd1, unifiable_comp_var_unk, unifiable_comp_var_nvar);
       /* ptd0 and ptd1 are unbound */
-      UnifiableGlobalCells(ptd0, ptd1);
+      UnifyAndTrailGlobalCells(ptd0, ptd1);
     }
   }
   /* Do we still have compound terms to visit */
@@ -865,6 +818,7 @@ cufail:
 static int 
 unifiable(CELL d0, CELL d1)
 {
+CACHE_REGS
 #if THREADS
 #undef Yap_REGS
   register REGSTORE *regp = Yap_regp;
@@ -935,11 +889,8 @@ unifiable_nvar_nvar:
 
   deref_body(d1, pt1, unifiable_nvar_unk, unifiable_nvar_nvar);
   /* d0 is bound and d1 is unbound */
-  BIND(pt1, d0, bind_unifiable3);
-#ifdef COROUTINING
+  *(pt1) = d0;
   DO_TRAIL(pt1, d0);
- bind_unifiable3:
-#endif
   return (TRUE);
 
   deref_body(d0, pt0, unifiable_unk, unifiable_nvar);
@@ -947,32 +898,14 @@ unifiable_nvar_nvar:
   deref_head(d1, unifiable_var_unk);
 unifiable_var_nvar:
   /* pt0 is unbound and d1 is bound */
-  BIND(pt0, d1, bind_unifiable4);
-#ifdef COROUTINING
-  DO_TRAIL(pt0, d1);
- bind_unifiable4:
-#endif
+  *pt0 = d1;
+   DO_TRAIL(pt0, d1);
   return TRUE;
 
-#if TRAILING_REQUIRES_BRANCH
-unifiable_var_nvar_trail:
-  DO_TRAIL(pt0);
-  return TRUE;
-#endif
-
   deref_body(d1, pt1, unifiable_var_unk, unifiable_var_nvar);
   /* d0 and pt1 are unbound */
-  UnifyCells(pt0, pt1, uc1, uc2);
-#ifdef COROUTINING
-  DO_TRAIL(pt0, (CELL)pt1);
- uc1:
-#endif
+  UnifyAndTrailCells(pt0, pt1);
   return (TRUE);
-#ifdef COROUTINING
- uc2:
-  DO_TRAIL(pt1, (CELL)pt0);
-  return (TRUE);
-#endif
 #if THREADS
 #undef Yap_REGS
 #define Yap_REGS (*Yap_regp)  
@@ -985,15 +918,15 @@ unifiable_var_nvar_trail:
 
 
 static Int
-p_unifiable(void)
+p_unifiable( USES_REGS1 )
 {
-  tr_fr_ptr trp;
+  tr_fr_ptr trp, trp0 = TR;
   Term tf = TermNil;
   if (!unifiable(ARG1,ARG2)) {
     return FALSE;
   }
   trp = TR;
-  while (trp != B->cp_tr) {
+  while (trp != trp0) {
     Term t[2];
     --trp;
     t[0] = TrailTerm(trp);
@@ -1004,15 +937,36 @@ p_unifiable(void)
   return Yap_unify(ARG3, tf);
 }
 
+int
+Yap_Unifiable( Term d0, Term d1 )
+{
+  CACHE_REGS
+  tr_fr_ptr trp, trp0 = TR;
+
+  if (!unifiable(d0,d1)) {
+    return FALSE;
+  }
+  trp = TR;
+  while (trp != trp0) {
+    Term t;
+
+    --trp;
+    t = TrailTerm(trp);
+    RESET_VARIABLE(t);
+  }
+  return TRUE;
+}
+
 void 
 Yap_InitUnify(void)
 {
+  CACHE_REGS
   Term cm = CurrentModule;
   Yap_InitCPred("unify_with_occurs_check", 2, p_ocunify, SafePredFlag);
   Yap_InitCPred("acyclic_term", 1, p_acyclic, SafePredFlag|TestPredFlag);
   CurrentModule = TERMS_MODULE;
   Yap_InitCPred("cyclic_term", 1, p_cyclic, SafePredFlag|TestPredFlag);
-  Yap_InitCPred("protected_unifiable", 3, p_unifiable, 0);
+  Yap_InitCPred("unifiable", 3, p_unifiable, 0);
   CurrentModule = cm;
 }
 
@@ -1030,6 +984,7 @@ Yap_InitAbsmi(void)
 void
 Yap_TrimTrail(void)
 {
+  CACHE_REGS
 #ifdef saveregs
 #undef saveregs
 #define saveregs()
