@@ -325,6 +325,7 @@ p_socket(void)
   Term t1 = Deref(ARG1);
   Term t2 = Deref(ARG2);
   Term t3 = Deref(ARG3);
+  Int t;
   char *sdomain, *stype;
   Int domain = AF_UNSPEC, type, protocol;
   int fd;
@@ -460,10 +461,10 @@ p_socket(void)
 #endif
     return(FALSE);
   }
-  if (domain == AF_UNIX || domain == AF_LOCAL )
-    out = Yap_InitSocketStream(fd, new_socket, af_unix);
-  else if (domain == AF_INET )
-    out = Yap_InitSocketStream(fd, new_socket, af_inet);
+  if (domain == AF_UNIX || domain == AF_LOCAL ) {
+    sout = Sfdopen(fd, "r+");
+  } else if (domain == AF_INET )
+    sout = Sfdopen(fd, "r+");
   else {
     /* ok, we currently don't support these sockets */
 #if _MSC_VER || defined(__MINGW32__)
@@ -473,81 +474,18 @@ p_socket(void)
 #endif
     return(FALSE);
   }
-  if (out == TermNil) return(FALSE);
-  return(Yap_unify(out,ARG4));
-}
-
-Int
-Yap_CloseSocket(int fd, socket_info status, socket_domain domain)
-{
-#if _MSC_VER || defined(__MINGW32__)
-  /* prevent further writing
-     to the socket */
-  if (status == server_session_socket ||
-      status == client_socket) {
-    char bfr;
-
-    if (shutdown(fd, 1) != 0) {
-      Yap_Error(SYSTEM_ERROR, TermNil,
-	    "socket_close/1 (close)");
-      return(FALSE);
-    }
-    /* read all pending characters
-       from the socket */
-    while( recv( fd, &bfr, 1, 0 ) > 0 );
-    /* prevent further reading
-       from the socket */
-    if (shutdown(fd, 0) < 0)  {
-      Yap_Error(SYSTEM_ERROR, TermNil,
-	    "socket_close/1 (close)");
-      return(FALSE);
-    }
-
-    /* close the socket */
-    if (closesocket(fd) != 0) {
-#if HAVE_STRERROR
-      Yap_Error(SYSTEM_ERROR, TermNil, 
-	    "socket_close/1 (close: %s)", strerror(socket_errno));
-#else
-      Yap_Error(SYSTEM_ERROR, TermNil,
-	    "socket_close/1 (close)");
-#endif
-    }
-#else
-    if (status == server_session_socket ||
-      status == client_socket) {
-    if (shutdown(fd,2) < 0) {
-#if HAVE_STRERROR
-      Yap_Error(SYSTEM_ERROR, TermNil, 
-	    "socket_close/1 (shutdown: %s)", strerror(socket_errno));
-#else
-      Yap_Error(SYSTEM_ERROR, TermNil,
-	    "socket_close/1 (shutdown)");
-#endif
-      return(FALSE);
-    }
-  }
-  if (close(fd) != 0) {
-#if HAVE_STRERROR
-    Yap_Error(SYSTEM_ERROR, TermNil, 
-	    "socket_close/1 (close: %s)", strerror(socket_errno));
-#else
-    Yap_Error(SYSTEM_ERROR, TermNil,
-	    "socket_close/1 (close)");
-#endif
-#endif
-    return(FALSE);
-  }
-  return(TRUE);
+  if (!out) return FALSE;
+  t = Yap_PutInSlot(ARG4);
+  return PL_unify_stream_or_alias(t, sout);
 }
 
 static Int
 p_socket_close(void)
 {
-  Term t1 = Deref(ARG1);
-  int sno;
+  Int t1 = Yap_PutInSlot(ARG1);
+  IOSTREAM *s;
 
-  if ((sno = Yap_CheckSocketStream(t1, "socket_close/1")) < 0) {
+  if (!PL_get_stream_handle(t1, &s)) {
     return (FALSE);
   }
   Yap_CloseStream(sno);
