@@ -96,13 +96,12 @@ register struct yami* P1REG asm ("bp"); /* can't use yamop before Yap.h */
 #ifdef BP_FREE
 #undef BP_FREE
 #endif
-#define SHADOW_REGS    1
 #define SHADOW_S       1
 //#define SHADOW_Y       1
 #define S_IN_MEM       1
 #define      Y_IN_MEM  1
 #define     TR_IN_MEM  1
-#define USE_PREFETCH   1
+#define LIMITED_PREFETCH   1
 #endif /* __x86_64__ */
 
 #else /* other compilers */
@@ -178,19 +177,27 @@ register struct yami* P1REG asm ("bp"); /* can't use yamop before Yap.h */
 
 /* regp is a global variable */
 
-inline EXTERN void
+INLINE_ONLY inline EXTERN void
+init_absmi_regs(REGSTORE * absmi_regs);
+
+INLINE_ONLY inline EXTERN void
 init_absmi_regs(REGSTORE * absmi_regs)
 {
+  CACHE_REGS
   memcpy(absmi_regs, Yap_regp, sizeof(REGSTORE));
 }
 
-inline EXTERN void
+INLINE_ONLY inline EXTERN void
+restore_absmi_regs(REGSTORE * old_regs);
+
+INLINE_ONLY inline EXTERN void
 restore_absmi_regs(REGSTORE * old_regs)
 {
+  CACHE_REGS
   memcpy(old_regs, Yap_regp, sizeof(REGSTORE));
 #ifdef THREADS
   pthread_setspecific(Yap_yaamregs_key, (void *)old_regs);
-  MY_ThreadHandle.current_yaam_regs = old_regs;
+  LOCAL_ThreadHandle.current_yaam_regs = old_regs;
 #else
   Yap_regp = old_regs;
 #endif
@@ -258,6 +265,8 @@ restore_absmi_regs(REGSTORE * old_regs)
 
 #define CACHE_Y_AS_ENV(A) { register CELL *ENV_YREG = (A)
 
+#define FETCH_Y_FROM_ENV(A) ENV_YREG = (A)
+
 #define WRITEBACK_Y_AS_ENV()   YREG = ENV_YREG
 
 #define ENDCACHE_Y_AS_ENV() }
@@ -274,6 +283,8 @@ restore_absmi_regs(REGSTORE * old_regs)
 
 #define CACHE_Y_AS_ENV(A) { YREG = (A)
 
+#define FETCH_Y_FROM_ENV(A) (A)
+
 #define ENDCACHE_Y_AS_ENV() }
 
 #define saveregs_and_ycache() saveregs()
@@ -359,7 +370,7 @@ restore_absmi_regs(REGSTORE * old_regs)
 
 #define DO_PREFETCH_W(TYPE) to_go = (void *)(NEXTOP(PREG,TYPE)->u.o.opcw)
 
-#if LIMITED_PREFETCH
+#if LIMITED_PREFETCH||USE_PREFETCH
 
 #define ALWAYS_START_PREFETCH(TYPE) \
  { register void *to_go; DO_PREFETCH(TYPE)
@@ -446,7 +457,7 @@ restore_absmi_regs(REGSTORE * old_regs)
 #define JMPNextW()						\
 	JMP((void *)(PREG->u.o.opcw))
 
-#if USE_THREADED_CODE && LIMITED_PREFETCH
+#if USE_THREADED_CODE && (LIMITED_PREFETCH || USE_PREFETCH)
 
 #define ALWAYS_GONext() JMP(to_go)
 
@@ -680,21 +691,21 @@ Macros to check the limits of stacks
 
 #else
 
-#define check_trail(x) if (Unsigned(Yap_TrailTop) - Unsigned(x) < MinTrailGap) \
+#define check_trail(x) if (__builtin_expect((Unsigned(CurrentTrailTop) < Unsigned(x)),0)) \
 			goto notrailleft
 
-#define check_trail_in_indexing(x) if (Unsigned(Yap_TrailTop) - Unsigned(x) < MinTrailGap) \
+#define check_trail_in_indexing(x) if (__builtin_expect((Unsigned(CurrentTrailTop) < Unsigned(x)),0)) \
 			goto notrailleft_from_index
 
 #endif
 
-#if (defined(SBA) && defined(YAPOR)) || defined(TABLING)
+#if (defined(YAPOR_SBA) && defined(YAPOR)) || defined(TABLING)
 #define check_stack(Label, GLOB)                             \
- if ( (Int)(Unsigned(YOUNGEST_CP((choiceptr)ENV_YREG,B_FZ)) - Unsigned(YOUNGEST_H(H_FZ,GLOB))) < CreepFlag  ) goto Label
+  if (__builtin_expect( ((Int)(Unsigned(YOUNGEST_CP((choiceptr)ENV_YREG,B_FZ)) - Unsigned(YOUNGEST_H(H_FZ,GLOB))) < CreepFlag), 0)  ) goto Label
 #else
 #define check_stack(Label, GLOB)                             \
- if ( (Int)(Unsigned(ENV_YREG) - Unsigned(GLOB)) < CreepFlag ) goto Label
-#endif /* SBA && YAPOR */
+  if  (__builtin_expect(((Int)(Unsigned(ENV_YREG) - Unsigned(GLOB)) < CreepFlag ), 0) ) goto Label
+#endif /* YAPOR_SBA && YAPOR */
 
 /***************************************************************
 * Macros for choice point manipulation                         *
@@ -723,7 +734,7 @@ Macros to check the limits of stacks
 #define store_at_least_one_arg(arity)                             \
                  BEGP(pt0);                                       \
 		 pt0 = XREGS+(arity);                             \
-                 do { register CELL x = pt0[0];                   \
+                 do { CELL x = pt0[0];                   \
                      S_YREG = (S_YREG)-1;			          \
                      --pt0;                                       \
                      (S_YREG)[0] = x;	                          \
@@ -731,8 +742,8 @@ Macros to check the limits of stacks
 		 while ( pt0 > XREGS );                           \
                  ENDP(pt0)
 
-#if LOW_LEVEL_TRACER
-#define COUNT_CPS() Yap_total_choicepoints++
+#if LOW_LEVEL_TRACER && 0
+#define COUNT_CPS() LOCAL_total_choicepoints++
 #else
 #define COUNT_CPS()
 #endif
@@ -811,7 +822,7 @@ Macros to check the limits of stacks
 
 
 #ifdef FROZEN_STACKS
-#ifdef SBA
+#ifdef YAPOR_SBA
 #define PROTECT_FROZEN_H(CPTR)                                  \
        ((Unsigned((Int)((CPTR)->cp_h)-(Int)(H_FZ)) <            \
 	 Unsigned((Int)(B_FZ)-(Int)(H_FZ))) ?                   \
@@ -828,7 +839,7 @@ Macros to check the limits of stacks
 #else /* TABLING */
 #define PROTECT_FROZEN_B(CPTR)  (YOUNGER_CP(CPTR, B_FZ) ? CPTR        : B_FZ)
 #define PROTECT_FROZEN_H(CPTR)  (((CPTR)->cp_h > H_FZ) ? (CPTR)->cp_h : H_FZ)
-#endif /* SBA */
+#endif /* YAPOR_SBA */
 #else
 #define PROTECT_FROZEN_B(CPTR)  (CPTR)
 #define PROTECT_FROZEN_H(CPTR)  (CPTR)->cp_h
@@ -970,57 +981,39 @@ Macros to check the limits of stacks
  *                                                                    *
  **********************************************************************/
 
-#ifdef COROUTINING
-#define UnifyCells(a, b, l1, l2)                                  \
-     if((a) > (b)) {                                              \
-	if ((a)<=H) { BIND_GLOBAL((a),(CELL)(b),l1); }            \
-	else if ((b)<= H) { Bind_Local((a),(CELL)(b)); goto l1;}  \
-	else { Bind_Local((b),(CELL) (a));  goto l1;}             \
-     } else if((a) < (b)){                                        \
-	if((b) <= H) { BIND_GLOBAL2((b),(CELL) (a),l2,l1); }      \
-	else if ((a) <= H) { Bind_Local((b),(CELL) (a));  goto l1;} \
-	else { Bind_Local((a),(CELL) (b));  goto l1;}             \
-     } else goto l1;
-
-/* I know (a) <= H */
-#define UnifyGlobalRegCells(a, b, l1, l2)                         \
-     if((a) > (b)) {                                              \
-	BIND_GLOBAL((a),(CELL)(b),l1);                            \
-     } else if((a) < (b)){                                        \
-	if((b) <= H) { BIND_GLOBAL2((b),(CELL) (a),l2,l1); }      \
-	Bind_Local((b),(CELL) (a));                               \
-	goto l1;				                  \
-     } else goto l1;
-
-#else
-#define UnifyCells(a, b, l1, l2)                                  \
-     if((a) > (b)) {                                              \
-	if ((a)<=H) { BIND_GLOBAL((a),(CELL)(b),l1); }            \
-	else if ((b)<= H) { Bind_Local((a),(CELL)(b)); }          \
-	else { Bind_Local((b),(CELL) (a)); }                      \
-     } else if((a) < (b)){                                        \
-	if((b) <= H) { BIND_GLOBAL2((b),(CELL) (a),l2,l1); }      \
-	else if ((a) <= H) { Bind_Local((b),(CELL) (a)); }        \
-	else { Bind_Local((a),(CELL) (b)); }                      \
-     }
-
-/* I know (a) <= H */
-#define UnifyGlobalRegCells(a, b, l1, l2)                         \
-     if((a) > (b)) {                                              \
-	BIND_GLOBAL((a),(CELL)(b),l1);                            \
-     } else if((a) < (b)){                                        \
-	if((b) <= H) { BIND_GLOBAL2((b),(CELL) (a),l2,l1); }      \
-	Bind_Local((b),(CELL) (a));                               \
-     }
-
-#endif
-
 #define UnifyGlobalCells(a, b)                                    \
-     if((a) > (b)) {                                              \
-	BIND_GLOBALCELL((a),(CELL)(b));                           \
-     } else if((a) < (b)){                                        \
-	BIND_GLOBALCELL((b),(CELL) (a));                          \
-     }
+  if ((b) > (a)) {                                                \
+    if (GlobalIsAttVar(b) && !GlobalIsAttVar(a)) {                    \
+      Bind_Global((a),(CELL)(b));                                 \
+    } else {                                                      \
+      Bind_Global((b),(CELL)(a));                                 \
+    }                                                             \
+  } else if ((b) < (a)) {                                         \
+    if (GlobalIsAttVar(a) && !GlobalIsAttVar(b)) {                    \
+      Bind_Global((b),(CELL)(a));                                 \
+    } else {                                                      \
+      Bind_Global((a),(CELL)(b));                                 \
+    }                                                             \
+  }                                                               
+
+#define UnifyGlobalCellToCell(b, a)	                          \
+if ((a) < H) { /* two globals */				  \
+  UnifyGlobalCells(a,b);					  \
+} else {							  \
+      Bind_Local((a),(CELL)(b));				  \
+}
+
+#define UnifyCells(a, b)		                          \
+if ((a) < H) { /* at least one global */			  \
+  if ((b) > H) { Bind_Local((b),(CELL)(a)); }			  \
+  else { UnifyGlobalCells(a,b); }				  \
+} else {                                                          \
+  if ((b) > (a)) { Bind_Local((a),(CELL)(b)); }			  \
+  else if ((a) > (b)) {						  \
+    if ((b) < H) { Bind_Local((a),(CELL)(b)); }                   \
+    else { Bind_Local((b),(CELL)(a)); }			          \
+  }								  \
+}
 
 /* unify two complex terms.
  * 
@@ -1113,6 +1106,7 @@ typedef struct v_record {
 static int 
 IUnify_complex(CELL *pt0, CELL *pt0_end, CELL *pt1)
 {
+  CACHE_REGS
 #ifdef THREADS
 #undef Yap_REGS
   register REGSTORE *regp = Yap_regp;
@@ -1166,7 +1160,7 @@ loop:
 #endif
 	  if ((void *)to_visit < (void *)unif) {
 	    CELL **urec = (CELL **)unif;
-	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec);
+	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec, NULL);
 	    unif = (struct unif_record *)urec;
 	  }
 	  to_visit->start0 = pt0;
@@ -1211,7 +1205,7 @@ loop:
 #endif
 	  if ((void *)to_visit < (void *)unif) {
 	    CELL **urec = (CELL **)unif;
-	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec);
+	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec, NULL);
 	    unif = (struct unif_record *)urec;
 	  }
 	  to_visit->start0 = pt0;
@@ -1233,7 +1227,8 @@ loop:
 
       derefa_body(d1, ptd1, unify_comp_nvar_unk, unify_comp_nvar_nvar);
 	/* d1 and pt2 have the unbound value, whereas d0 is bound */
-      BIND_GLOBALCELL(ptd1, d0);
+      Bind_Global(ptd1, d0);
+      continue;
     }
 
     derefa_body(d0, ptd0, unify_comp_unk, unify_comp_nvar);
@@ -1248,7 +1243,8 @@ loop:
       deref_head(d1, unify_comp_var_unk);
     unify_comp_var_nvar:
       /* pt2 is unbound and d1 is bound */
-      BIND_GLOBALCELL(ptd0, d1);
+      Bind_Global(ptd0, d1);
+      continue;
 
       derefa_body(d1, ptd1, unify_comp_var_unk, unify_comp_var_nvar);
       /* ptd0 and ptd1 are unbound */
@@ -1310,6 +1306,7 @@ iequ_complex(register CELL *pt0, register CELL *pt0_end,
 	       register CELL *pt1
 )
 {
+CACHE_REGS
 #ifdef THREADS
 #undef Yap_REGS
   register REGSTORE *regp = Yap_regp;
@@ -1363,7 +1360,7 @@ loop:
 #endif
 	  if ((void *)to_visit < (void *)unif) {
 	    CELL **urec = (CELL **)unif;
-	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec);
+	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec, NULL);
 	    unif = (struct unif_record *)urec;
 	  }
 	  to_visit->start0 = pt0;
@@ -1408,7 +1405,7 @@ loop:
 #endif
 	  if ((void *)to_visit < (void *)unif) {
 	    CELL **urec = (CELL **)unif;
-	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec);
+	    to_visit = (struct v_record *)Yap_shift_visit((CELL **)to_visit, &urec, NULL);
 	    unif = (struct unif_record *)urec;
 	  }
 	  to_visit->start0 = pt0;
@@ -1548,3 +1545,64 @@ Yap_regtoregno(wamreg reg)
 #define copy_jmp_addressa(X) 
 #endif
 
+static inline void
+prune(choiceptr cp USES_REGS)
+{
+#ifdef YAPOR
+  CUT_prune_to(cp);
+#endif /* YAPOR */
+  if (SHOULD_CUT_UP_TO(B,cp))
+    {
+      if (ASP > (CELL *)PROTECT_FROZEN_B(B))
+	ASP = (CELL *)PROTECT_FROZEN_B(B);
+      while (B->cp_b < cp) {
+	if (POP_CHOICE_POINT(B->cp_b))
+	  {
+	    POP_EXECUTE();
+	  }
+	B = B->cp_b;
+      }
+      if (POP_CHOICE_POINT(B->cp_b))
+	{
+	  POP_EXECUTE();
+	}
+      /* cut ! */
+#ifdef TABLING
+      abolish_incomplete_subgoals(B);
+#endif /* TABLING */
+      HB = PROTECT_FROZEN_H(B->cp_b);
+#include "trim_trail.h"
+      B = B->cp_b;
+      SET_BB(PROTECT_FROZEN_B(B));
+    }
+}
+
+#define SET_ASP(Y,S) SET_ASP__(Y,S PASS_REGS)
+
+static inline
+void SET_ASP__(CELL *yreg, Int sz USES_REGS) {
+  ASP = (CELL *) (((char *) yreg) + sz);
+  if (ASP > (CELL *)PROTECT_FROZEN_B(B))
+    ASP = (CELL *)PROTECT_FROZEN_B(B);
+}
+
+#if YAPOR
+#define INITIALIZE_PERMVAR(PTR, V) Bind_Local((PTR), (V))
+#else
+#define INITIALIZE_PERMVAR(PTR, V) *(PTR) = (V)
+#endif
+
+/* l1: bind a, l2 bind b, l3 no binding */
+#define UnifyAndTrailCells(a, b)                                            \
+     if((a) > (b)) {                                                        \
+       if ((a) < H) { *(a) = (CELL)(b); DO_TRAIL((a),(CELL)(b)); }	    \
+       else if ((b) <= H) { *(a) =(CELL)(b); DO_TRAIL((a),(CELL)(b));}	   \
+       else { *(b) = (CELL)(a);  DO_TRAIL((b),(CELL)(a)); }		    \
+     } else if((a) < (b)){                                                  \
+       if ((b) <= H) { *(b) = (CELL)(a); DO_TRAIL((b),(CELL)(a)); }         \
+       else if ((a) <= H) { *(b) = (CELL) (a); DO_TRAIL((b),(CELL)(a));}    \
+       else { *(a) = (CELL) (b);  DO_TRAIL((a),(CELL)(b));}                 \
+     }
+
+
+#define CHECK_ALARM(CONT)
