@@ -29,15 +29,15 @@
 
 static Int null_id = 0;
 
-STATIC_PROTO(Int c_db_odbc_connect,(void));
-STATIC_PROTO(Int c_db_odbc_disconnect,(void));
-STATIC_PROTO(Int c_db_odbc_number_of_fields,(void));
-STATIC_PROTO(Int c_db_odbc_get_attributes_types,(void));
-STATIC_PROTO(Int c_db_odbc_query,(void));
-STATIC_PROTO(Int c_db_odbc_row,(void));
-STATIC_PROTO(Int c_db_odbc_row_cut,(void));
-STATIC_PROTO(Int c_db_odbc_get_fields_properties,(void));
-STATIC_PROTO(Int c_db_odbc_number_of_fields_in_query,(void));
+STATIC_PROTO(Int c_db_odbc_connect,( USES_REGS1 ));
+STATIC_PROTO(Int c_db_odbc_disconnect,( USES_REGS1 ));
+STATIC_PROTO(Int c_db_odbc_number_of_fields,( USES_REGS1 ));
+STATIC_PROTO(Int c_db_odbc_get_attributes_types,( USES_REGS1 ));
+STATIC_PROTO(Int c_db_odbc_query,( USES_REGS1 ));
+STATIC_PROTO(Int c_db_odbc_row,( USES_REGS1 ));
+STATIC_PROTO(Int c_db_odbc_row_cut,( USES_REGS1 ));
+STATIC_PROTO(Int c_db_odbc_get_fields_properties,( USES_REGS1 ));
+STATIC_PROTO(Int c_db_odbc_number_of_fields_in_query,( USES_REGS1 ));
 
 static int
 odbc_error(SQLSMALLINT type, SQLHANDLE hdbc, char *msg, char *print)
@@ -154,18 +154,6 @@ static int SQLBINDCOL(SQLHSTMT       sthandle,
   return TRUE;
 }
 
-static int SQLFREESTMT(SQLHSTMT       sthandle,
-		       SQLUSMALLINT   opt,
-		       char *         print)
-{                                                                 
-  SQLRETURN retcode;                                              
-  retcode = SQLFreeStmt(sthandle,opt);
-
-  if (retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO) 
-    return odbc_error(SQL_HANDLE_STMT, sthandle, "SQLFreeStmt", print);
-  return TRUE;
-}
-
 static int SQLNUMRESULTCOLS(SQLHSTMT        sthandle,
 			    SQLSMALLINT *   ncols,
 			    char *          print)
@@ -321,7 +309,7 @@ static int SQLCOLATTRIBUTE( SQLHSTMT        StatementHandle,
 
 
 static Int
-c_db_odbc_connect(void) {
+c_db_odbc_connect( USES_REGS1 ) {
   Term arg_driver = Deref(ARG1); 
   Term arg_user = Deref(ARG2);
   Term arg_passwd = Deref(ARG3);
@@ -371,7 +359,7 @@ c_db_odbc_connect(void) {
 
 /* db_query: SQLQuery x ResultSet x Arity x BindList x Connection */
 static Int
-c_db_odbc_query(void) {
+c_db_odbc_query( USES_REGS1 ) {
   Term arg_sql_query = Deref(ARG1);
   Term arg_result_set = Deref(ARG2);
   Term arg_arity = Deref(ARG3);
@@ -421,8 +409,9 @@ c_db_odbc_query(void) {
 	/* +1 because of '\0' */
 	bind_space = malloc(sizeof(char)*(ColumnSizePtr+1));
 	data_info = malloc(sizeof(SQLINTEGER));
-	if (!SQLBINDCOL(hstmt,i,SQL_C_CHAR,bind_space,(ColumnSizePtr+1),data_info,"db_query"))
+	if (!SQLBINDCOL(hstmt,i,SQL_C_CHAR,bind_space,(ColumnSizePtr+1),data_info,"db_query")) {
 	  return FALSE;
+	}
 	
 	properties[0] = MkIntegerTerm((Int)bind_space);
 	properties[2] = MkIntegerTerm((Int)data_info);
@@ -444,7 +433,7 @@ c_db_odbc_query(void) {
     {
       if (!SQLCLOSECURSOR(hstmt,"db_query"))
 	return FALSE;
-      if (!SQLFREESTMT(hstmt,SQL_CLOSE,"db_query"))
+      if (!SQLFREEHANDLE(SQL_HANDLE_STMT, hstmt, "db_query"))
 	return FALSE;
       return FALSE;
     }
@@ -452,7 +441,7 @@ c_db_odbc_query(void) {
 }
 
 static Int
-c_db_odbc_number_of_fields(void) {
+c_db_odbc_number_of_fields( USES_REGS1 ) {
   Term arg_relation = Deref(ARG1);
   Term arg_conn = Deref(ARG2);
   Term arg_fields = Deref(ARG3);
@@ -466,7 +455,7 @@ c_db_odbc_number_of_fields(void) {
   char sql[256];
   SQLSMALLINT number_fields;
 
-  sprintf(sql,"DESCRIBE %s",relation);
+  sprintf(sql,"SELECT column_name from INFORMATION_SCHEMA.COLUMNS where table_name = \'%s\'",relation);
   
   if (!SQLALLOCHANDLE(SQL_HANDLE_STMT, hdbc, &hstmt, "db_number_of_fields"))
     return FALSE;
@@ -482,7 +471,7 @@ c_db_odbc_number_of_fields(void) {
   
   if (!SQLCLOSECURSOR(hstmt,"db_number_of_fields"))
     return FALSE;
-  if (!SQLFREESTMT(hstmt,SQL_CLOSE,"db_number_of_fields"))
+  if (!SQLFREEHANDLE(SQL_HANDLE_STMT, hstmt, "db_number_of_fields"))
     return FALSE;
   
   if (!Yap_unify(arg_fields, MkIntegerTerm(number_fields)))
@@ -493,7 +482,7 @@ c_db_odbc_number_of_fields(void) {
 
 /* db_get_attributes_types: RelName x Connection -> TypesList */
 static Int
-c_db_odbc_get_attributes_types(void) {
+c_db_odbc_get_attributes_types( USES_REGS1 ) {
   Term arg_relation = Deref(ARG1);
   Term arg_conn = Deref(ARG2);
   Term arg_types_list = Deref(ARG3);
@@ -506,7 +495,7 @@ c_db_odbc_get_attributes_types(void) {
   Term head, list;
   list = arg_types_list;
 
-  sprintf(sql,"DESCRIBE %s",relation);
+  sprintf(sql,"SELECT column_name,data_type FROM INFORMATION_SCHEMA.COLUMNS where table_name = \'%s\'",relation);
 
   if (!SQLALLOCHANDLE(SQL_HANDLE_STMT, hdbc, &hstmt, "db_get_attributes_types"))
     return FALSE;
@@ -547,14 +536,14 @@ c_db_odbc_get_attributes_types(void) {
   
   if (!SQLCLOSECURSOR(hstmt,"db_get_attributes_types"))
     return FALSE;
-  if (!SQLFREESTMT(hstmt,SQL_CLOSE, "db_get_attributes_types"))
+  if (!SQLFREEHANDLE(SQL_HANDLE_STMT, hstmt, "db_get_attributes_types"))
     return FALSE;
   return TRUE;
 }
 
 /* db_disconnect */
 static Int
-c_db_odbc_disconnect(void) {
+c_db_odbc_disconnect( USES_REGS1 ) {
   Term arg_conn = Deref(ARG1);
 
   SQLHDBC conn = (SQLHDBC) (IntegerOfTerm(arg_conn));
@@ -579,21 +568,40 @@ c_db_odbc_disconnect(void) {
 }
 
 static Int
-c_db_odbc_row_cut(void) {
+c_db_odbc_row_cut( USES_REGS1 ) {
     
   SQLHSTMT hstmt = (SQLHSTMT) IntegerOfTerm(EXTRA_CBACK_CUT_ARG(Term,1)); 
   
   if (!SQLCLOSECURSOR(hstmt,"db_row_cut"))
     return FALSE; 
-  if (!SQLFREESTMT(hstmt,SQL_CLOSE,"db_row_cut"))
+  if (!SQLFREEHANDLE(SQL_HANDLE_STMT, hstmt, "db_row_cut"))
     return FALSE; 
    
   return TRUE;
 }
 
+static int
+release_list_args(Term arg_list_args, Term arg_bind_list, const char *error_msg)
+{
+  Term list = arg_list_args;
+  Term list_bind = arg_bind_list;
+  
+  while (IsPairTerm(list_bind))
+    {
+      Term head_bind = HeadOfTerm(list_bind);
+
+      list = TailOfTerm(list);
+      list_bind = TailOfTerm(list_bind);
+      
+      free((char *)IntegerOfTerm(ArgOfTerm(1,head_bind)));
+      free((SQLINTEGER *)IntegerOfTerm(ArgOfTerm(3,head_bind)));
+    }
+  return TRUE;
+}
+
 /* db_row: ResultSet x BindList x ListOfArgs -> */
 static Int
-c_db_odbc_row(void) {
+c_db_odbc_row( USES_REGS1 ) {
   Term arg_result_set = Deref(ARG1);
   Term arg_bind_list = Deref(ARG2);
   Term arg_list_args = Deref(ARG3);
@@ -611,9 +619,12 @@ c_db_odbc_row(void) {
     {
       if (!SQLCLOSECURSOR(hstmt,"db_row"))
 	return FALSE; 
-      if (!SQLFREESTMT(hstmt,SQL_CLOSE,"db_row"))
+      if (!SQLFREEHANDLE(SQL_HANDLE_STMT, hstmt, "db_row"))
 	return FALSE; 
-  
+      if (!release_list_args(arg_list_args, arg_bind_list, "db_row")) {
+	return FALSE;
+      }
+ 
       cut_fail(); 
       return FALSE;
     }
@@ -674,7 +685,7 @@ c_db_odbc_row(void) {
 /* Mudar esta funcao de forma a nao fazer a consulta, pois 
  no predicate db_sql_selet vai fazer duas vezes a mesma consutla*/ 
 static Int
-c_db_odbc_number_of_fields_in_query(void) {
+c_db_odbc_number_of_fields_in_query( USES_REGS1 ) {
   Term arg_query = Deref(ARG1);
   Term arg_conn = Deref(ARG2);
   Term arg_fields = Deref(ARG3);
@@ -699,7 +710,7 @@ c_db_odbc_number_of_fields_in_query(void) {
   if (!Yap_unify(arg_fields, MkIntegerTerm(number_cols))){
     if (!SQLCLOSECURSOR(hstmt,"db_number_of_fields_in_query"))
       return FALSE;
-    if (!SQLFREESTMT(hstmt,SQL_CLOSE, "db_number_of_fields_in_query"))
+    if (!SQLFREEHANDLE(SQL_HANDLE_STMT, hstmt, "db_number_of_fields_in_query"))
       return FALSE;
  
     return FALSE;
@@ -707,14 +718,14 @@ c_db_odbc_number_of_fields_in_query(void) {
   
   if (!SQLCLOSECURSOR(hstmt,"db_number_of_fields_in_query"))
     return FALSE;
-  if (!SQLFREESTMT(hstmt,SQL_CLOSE, "db_number_of_fields_in_query"))
+  if (!SQLFREEHANDLE(SQL_HANDLE_STMT, hstmt, "db_number_of_fields_in_query"))
     return FALSE;
  
   return TRUE;
 }
 
 static Int
-c_db_odbc_get_fields_properties(void) {
+c_db_odbc_get_fields_properties( USES_REGS1 ) {
   Term nome_relacao = Deref(ARG1);
   Term arg_conn = Deref(ARG2);
   Term fields_properties_list = Deref(ARG3);
@@ -775,7 +786,7 @@ c_db_odbc_get_fields_properties(void) {
 
   if (!SQLCLOSECURSOR(hstmt2,"db_get_fields_properties"))
     return FALSE;
-  if (!SQLFREESTMT(hstmt2,SQL_CLOSE,"db_get_fields_properties"))
+  if (!SQLFREEHANDLE(SQL_HANDLE_STMT, hstmt2, "db_get_fields_properties"))
     return FALSE;
   
   for (i=1;i<=num_fields;i++)
@@ -816,9 +827,8 @@ c_db_odbc_get_fields_properties(void) {
   
   if (!SQLCLOSECURSOR(hstmt,"db_get_fields_properties"))
     return FALSE;
-  if (!SQLFREESTMT(hstmt,SQL_CLOSE,"db_get_fields_properties"))
+  if (!SQLFREEHANDLE(SQL_HANDLE_STMT, hstmt2, "db_get_fields_properties"))
     return FALSE;
-    
   return TRUE;
 }
 
