@@ -52,6 +52,7 @@ STD_PROTO(static inline void move_up_to_prune_request, (void));
 
 static inline
 void PUT_NO_WORK_IN_UPPER_NODES(void) {
+  CACHE_REGS
   or_fr_ptr current_node, nearest_livenode;
   current_node = LOCAL_top_or_fr;
   while ((nearest_livenode = OrFr_nearest_livenode(current_node))) {
@@ -64,42 +65,43 @@ void PUT_NO_WORK_IN_UPPER_NODES(void) {
 
 static inline
 void PUT_IDLE(int worker_num) {
-  LOCK(GLOBAL_LOCKS_bm_idle_workers);
+  LOCK(GLOBAL_locks_bm_idle_workers);
   BITMAP_insert(GLOBAL_bm_idle_workers, worker_num);
-  UNLOCK(GLOBAL_LOCKS_bm_idle_workers);
+  UNLOCK(GLOBAL_locks_bm_idle_workers);
   return;
 }
 
 
 static inline
 void PUT_BUSY(int worker_num) {
-  LOCK(GLOBAL_LOCKS_bm_idle_workers);
+  LOCK(GLOBAL_locks_bm_idle_workers);
   BITMAP_delete(GLOBAL_bm_idle_workers, worker_num);
-  UNLOCK(GLOBAL_LOCKS_bm_idle_workers);
+  UNLOCK(GLOBAL_locks_bm_idle_workers);
   return;
 }
 
 
 static inline
 void PUT_IN_ROOT_NODE(int worker_num) {
-  LOCK(GLOBAL_LOCKS_bm_root_cp_workers);
+  LOCK(GLOBAL_locks_bm_root_cp_workers);
   BITMAP_insert(GLOBAL_bm_root_cp_workers, worker_num);
-  UNLOCK(GLOBAL_LOCKS_bm_root_cp_workers);
+  UNLOCK(GLOBAL_locks_bm_root_cp_workers);
   return;
 }
 
 
 static inline
 void PUT_OUT_ROOT_NODE(int worker_num) {
-  LOCK(GLOBAL_LOCKS_bm_root_cp_workers);
+  LOCK(GLOBAL_locks_bm_root_cp_workers);
   BITMAP_delete(GLOBAL_bm_root_cp_workers, worker_num);
-  UNLOCK(GLOBAL_LOCKS_bm_root_cp_workers);
+  UNLOCK(GLOBAL_locks_bm_root_cp_workers);
   return;
 }
 
 
 static inline
 void move_up_to_prune_request(void) {
+  CACHE_REGS
   YAPOR_ERROR_CHECKING(move_up_to_prune_request, EQUAL_OR_YOUNGER_CP(Get_LOCAL_prune_request(), Get_LOCAL_top_cp()));
 
   do {
@@ -140,6 +142,7 @@ void move_up_to_prune_request(void) {
 ** -------------------------- */
 
 int get_work(void) {
+  CACHE_REGS
   int counter;
   bitmap stable_busy;
   yamop *alt_with_work;
@@ -244,7 +247,7 @@ int get_work(void) {
       PUT_BUSY(worker_id);
       return TRUE;
     }
-    if (++counter == SCHEDULER_LOOP) {
+    if (++counter == GLOBAL_scheduler_loop) {
       if (search_for_hidden_shared_work(stable_busy)) {
         PUT_BUSY(worker_id);
         return TRUE;
@@ -265,6 +268,7 @@ int get_work(void) {
 
 static
 int move_up_one_node(or_fr_ptr nearest_livenode) {
+  CACHE_REGS
   YAPOR_ERROR_CHECKING(move_up_one_node, Get_LOCAL_prune_request() && EQUAL_OR_YOUNGER_CP(Get_LOCAL_prune_request(), Get_LOCAL_top_cp()));
 
   LOCK_OR_FRAME(LOCAL_top_or_fr);
@@ -287,7 +291,7 @@ int move_up_one_node(or_fr_ptr nearest_livenode) {
     Set_OrFr_pend_prune_cp(LOCAL_top_or_fr, NULL);
     BRANCH(worker_id, OrFr_depth(LOCAL_top_or_fr)) = OrFr_pend_prune_ltt(LOCAL_top_or_fr);
     UNLOCK_OR_FRAME(LOCAL_top_or_fr);
-    prune_shared_branch(prune_cp);
+    prune_shared_branch(prune_cp, &OrFr_pend_prune_ltt(LOCAL_top_or_fr));
 #ifdef TABLING
     while (YOUNGER_CP(aux_cp->cp_b, Get_LOCAL_top_cp()))
       aux_cp = aux_cp->cp_b;
@@ -519,22 +523,23 @@ int move_up_one_node(or_fr_ptr nearest_livenode) {
 
 static
 int get_work_below(void){
+  CACHE_REGS
   int i, worker_p, big_load;
   bitmap busy_below, idle_below;
 
   worker_p = -1;
-  big_load = DELAYED_RELEASE_LOAD;
+  big_load = GLOBAL_delayed_release_load ;
   BITMAP_difference(busy_below, OrFr_members(LOCAL_top_or_fr), GLOBAL_bm_idle_workers);
   BITMAP_difference(idle_below, OrFr_members(LOCAL_top_or_fr), busy_below);
   BITMAP_delete(idle_below, worker_id);
-  for (i = 0; i < number_workers; i++) {
+  for (i = 0; i < GLOBAL_number_workers; i++) {
     if (BITMAP_member(idle_below ,i) && YOUNGER_CP(REMOTE_top_cp(i), Get_LOCAL_top_cp()))
       BITMAP_minus(busy_below, OrFr_members(REMOTE_top_or_fr(i)));
   }
   if (BITMAP_empty(busy_below))
     return FALSE;
   /* choose the worker with highest load */
-  for (i = 0 ; i < number_workers; i++) {
+  for (i = 0 ; i < GLOBAL_number_workers; i++) {
     if (BITMAP_member(busy_below ,i) && REMOTE_load(i) > big_load) {
       worker_p = i;
       big_load = REMOTE_load(i);
@@ -548,18 +553,19 @@ int get_work_below(void){
 
 static
 int get_work_above(void){
+  CACHE_REGS
   int i, worker_p, big_load;
   bitmap visible_busy_above, visible_idle_above;
 
   worker_p = -1; 
-  big_load = DELAYED_RELEASE_LOAD;
+  big_load = GLOBAL_delayed_release_load ;
   BITMAP_difference(visible_busy_above, GLOBAL_bm_present_workers, OrFr_members(LOCAL_top_or_fr));
   BITMAP_minus(visible_busy_above, GLOBAL_bm_invisible_workers);
   BITMAP_copy(visible_idle_above, visible_busy_above); 
   BITMAP_minus(visible_busy_above, GLOBAL_bm_idle_workers);
   BITMAP_and(visible_idle_above, GLOBAL_bm_idle_workers);
   BITMAP_insert(visible_busy_above, worker_id);
-  for (i = 0 ; i < number_workers; i++) {
+  for (i = 0 ; i < GLOBAL_number_workers; i++) {
     if (BITMAP_member(visible_idle_above, i))
       BITMAP_minus(visible_busy_above, OrFr_members(REMOTE_top_or_fr(i)));
   }
@@ -567,7 +573,7 @@ int get_work_above(void){
     return FALSE;
   BITMAP_delete(visible_busy_above, worker_id);
   /* choose the worker with higher load */
-  for (i = 0; i < number_workers; i++) {
+  for (i = 0; i < GLOBAL_number_workers; i++) {
     if (BITMAP_member(visible_busy_above ,i) && REMOTE_load(i) > big_load) {
       worker_p = i;
       big_load = REMOTE_load(i);
@@ -576,14 +582,14 @@ int get_work_above(void){
   if (worker_p == -1)
     return FALSE;
   /* put workers invisibles */
-  LOCK(GLOBAL_LOCKS_bm_invisible_workers);
+  LOCK(GLOBAL_locks_bm_invisible_workers);
   if (BITMAP_member(GLOBAL_bm_invisible_workers, worker_p)) {
-    UNLOCK(GLOBAL_LOCKS_bm_invisible_workers);
+    UNLOCK(GLOBAL_locks_bm_invisible_workers);
     return FALSE;
   }
   BITMAP_insert(GLOBAL_bm_invisible_workers, worker_id);
   BITMAP_insert(GLOBAL_bm_invisible_workers, worker_p);
-  UNLOCK(GLOBAL_LOCKS_bm_invisible_workers);
+  UNLOCK(GLOBAL_locks_bm_invisible_workers);
   /* move up to cp with worker_p */
   do {
     if (! move_up_one_node(NULL)) {
@@ -591,23 +597,24 @@ int get_work_above(void){
     }
   } while (! BITMAP_member(OrFr_members(LOCAL_top_or_fr), worker_p));
   /* put workers visibles */
-  LOCK(GLOBAL_LOCKS_bm_invisible_workers);
+  LOCK(GLOBAL_locks_bm_invisible_workers);
   BITMAP_delete(GLOBAL_bm_invisible_workers, worker_id);
   BITMAP_delete(GLOBAL_bm_invisible_workers, worker_p);
-  UNLOCK(GLOBAL_LOCKS_bm_invisible_workers);
+  UNLOCK(GLOBAL_locks_bm_invisible_workers);
   return (q_share_work(worker_p));
 }
 
 
 static
 int find_a_better_position(void){
+  CACHE_REGS
   int i;
   bitmap busy_above, idle_above;
   BITMAP_difference(busy_above, GLOBAL_bm_present_workers, OrFr_members(LOCAL_top_or_fr));
   BITMAP_copy(idle_above, busy_above); 
   BITMAP_minus(busy_above, GLOBAL_bm_idle_workers);
   BITMAP_and(idle_above, GLOBAL_bm_idle_workers);
-  for (i = 0; i < number_workers; i++) {
+  for (i = 0; i < GLOBAL_number_workers; i++) {
     if (BITMAP_member(idle_above, i)) {
       if (BITMAP_empty(busy_above)) 
         break;
@@ -630,19 +637,20 @@ int find_a_better_position(void){
 
 static
 int search_for_hidden_shared_work(bitmap stable_busy){
+  CACHE_REGS
   int i;
   bitmap invisible_work, idle_below;
   BITMAP_intersection(invisible_work, stable_busy, GLOBAL_bm_requestable_workers);
   BITMAP_intersection(idle_below, OrFr_members(LOCAL_top_or_fr), GLOBAL_bm_idle_workers);
   BITMAP_delete(idle_below, worker_id);
-  for (i = 0; i < number_workers; i++) {
+  for (i = 0; i < GLOBAL_number_workers; i++) {
     if (BITMAP_member(idle_below ,i) && YOUNGER_CP(REMOTE_top_cp(i), Get_LOCAL_top_cp()))
       BITMAP_minus(invisible_work, OrFr_members(REMOTE_top_or_fr(i)));
   }
   if (BITMAP_empty(invisible_work))
     return FALSE;
   /* choose the first available worker */
-  for (i = 0; i < number_workers; i++ ) {
+  for (i = 0; i < GLOBAL_number_workers; i++ ) {
     if (BITMAP_member(invisible_work ,i))
       break;
   }
