@@ -386,17 +386,13 @@ new_floats_matrix_set(void)
 static YAP_Term
 float_matrix_to_list(int *mat) {
   double *data = matrix_double_data(mat, mat[MAT_NDIMS]);
-  int i = 0;
-  YAP_Term tf = YAP_TermNil(), tnil = tf;
 
-  for (i = mat[MAT_SIZE]-1; i>= 0; i--) {
-    tf = YAP_MkPairTerm(YAP_MkFloatTerm(data[i]),tf);
-    if (tf == tnil) {
-      /* error */
-      return tnil;
-    }
+  /* prepare for worst case with double taking two cells */
+  if (YAP_RequiresExtraStack(6*mat[MAT_SIZE])) {
+    mat = (int *)YAP_BlobOfTerm(YAP_ARG1);
+    data = matrix_double_data(mat, mat[MAT_NDIMS]);
   }
-  return tf;
+  return YAP_FloatsToList(data, mat[MAT_SIZE]);
 }
 
 static YAP_Term
@@ -438,6 +434,11 @@ static YAP_Term
 long_matrix_to_list(int *mat) {
   long int *data = matrix_long_data(mat, mat[MAT_NDIMS]);
 
+  /* prepare for worst case with longs evrywhere (3cells + 1) */
+  if (YAP_RequiresExtraStack(5*mat[MAT_SIZE])) {
+    mat = (int *)YAP_BlobOfTerm(YAP_ARG1);
+    data = matrix_long_data(mat, mat[MAT_NDIMS]);
+  }
   return mk_long_list(mat[MAT_SIZE], data);
 }
 
@@ -1278,6 +1279,7 @@ matrix_agg_lines(void)
     tf = new_int_matrix(dims-1,mat+(MAT_DIMS+1),NULL);
     if (tf == YAP_TermNil())
       return FALSE;
+    mat = (int *)YAP_BlobOfTerm(YAP_ARG1);
     nmat = (int *)YAP_BlobOfTerm(tf);
     data = matrix_long_data(mat, dims);
     ndata = matrix_long_data(nmat, dims-1);
@@ -1360,6 +1362,7 @@ matrix_agg_cols(void)
     tf = new_int_matrix(1,mat+MAT_DIMS,NULL);
     if (tf == YAP_TermNil())
       return FALSE;
+    mat = (int *)YAP_BlobOfTerm(YAP_ARG1);
     nmat = (int *)YAP_BlobOfTerm(tf);
     data = matrix_long_data(mat, dims);
     ndata = matrix_long_data(nmat, 1);
@@ -2259,8 +2262,8 @@ matrix_transpose(void)
 static int
 matrix_select(void)
 {
-  int ndims, i, j, *dims, newdims, prdim, leftarg;
-  int indx[MAX_DIMS], nindx[MAX_DIMS];
+  int ndims, i, j, newdims, prdim, leftarg, *dims, indx[MAX_DIMS];
+  int nindx[MAX_DIMS];
   YAP_Term tpdim, tdimarg, tf;
   int *mat = (int *)YAP_BlobOfTerm(YAP_ARG1), *nmat;
   if (!mat) {
@@ -2282,7 +2285,7 @@ matrix_select(void)
   leftarg = YAP_IntOfTerm(tdimarg);
   for (i=0, j=0; i< ndims; i++) {
     if (i != prdim) {
-      nindx[j]= (mat+MAT_DIMS)[i];
+      nindx[j]= dims[i];
       j++;
     }
   }
@@ -2429,7 +2432,7 @@ matrix_column(void)
 static int
 matrix_sum_out(void)
 {
-  int ndims, i, j, *dims, newdims, prdim;
+  int ndims, i, j, newdims, prdim;
   int indx[MAX_DIMS], nindx[MAX_DIMS];
   YAP_Term tpdim, tf;
   int *mat = (int *)YAP_BlobOfTerm(YAP_ARG1), *nmat;
@@ -2440,7 +2443,6 @@ matrix_sum_out(void)
   /* we now have our target matrix, let us grab our conversion arguments */
   tpdim = YAP_ARG2;
   ndims = mat[MAT_NDIMS];
-  dims = mat+MAT_DIMS;
   if (!YAP_IsIntTerm(tpdim)) {
     return FALSE;
   }
@@ -2613,7 +2615,7 @@ static int
 matrix_sum_out_logs(void)
 {
   int ndims, i, j, *dims, newdims, prdim;
-  int indx[MAX_DIMS], nindx[MAX_DIMS];
+  int nindx[MAX_DIMS];
   YAP_Term tpdim, tf;
   int *mat = (int *)YAP_BlobOfTerm(YAP_ARG1), *nmat;
   if (!mat) {
@@ -2631,12 +2633,13 @@ matrix_sum_out_logs(void)
   newdims = ndims-1;
   for (i=0, j=0; i< ndims; i++) {
     if (i != prdim) {
-      nindx[j]= (mat+MAT_DIMS)[i];
+      nindx[j]= dims[i];
       j++;
     }
   }
   if (mat[MAT_TYPE] == INT_MATRIX) {
     long int *data, *ndata;
+    int d = 1, j = 0, dd = 1;
 
     /* create a new matrix with the same size */
     tf = new_int_matrix(newdims,nindx,NULL);
@@ -2647,28 +2650,25 @@ matrix_sum_out_logs(void)
     nmat = (int *)YAP_BlobOfTerm(tf);  
     data = matrix_long_data(mat,ndims);
     ndata = matrix_long_data(nmat,newdims);
-    /* create a new matrix with smaller size */
-    for (i=0;i<nmat[MAT_SIZE];i++) 
-      ndata[i] = 0;
-    for (i=0; i< mat[MAT_SIZE]; i++) {
-      int j, k;
-      /*
-	not very efficient, we could try to take advantage of the fact
-	that we usually only change an index at a time
-      */
-      matrix_get_index(mat, i, indx);
-      for (j = 0, k=0; j < ndims; j++)  {
-	if (j != prdim) {
-	  nindx[k++]= indx[j];
-	}
-      }
-      ndata[matrix_get_offset(nmat, nindx)] += exp(data[i]);
+    while (j < prdim) {
+      d = d*dims[j];
+      j++;
+    }
+    dd = d*dims[prdim];
+    for (i=0;i<nmat[MAT_SIZE];i++) {
+      int j = i % d + (i/dd)*d;
+      ndata[j] = exp(data[i]);
+    }
+    for (; i< mat[MAT_SIZE]; i++) {
+      int j = i % d + (i/dd)*d;
+      ndata[j] += exp(data[i]);
     }
     for (i=0; i< nmat[MAT_SIZE]; i++) {
       ndata[i] = log(ndata[i]);
     }
   } else {
     double *data, *ndata;
+    int d = 1, j = 0, dd = 1;
 
     /* create a new matrix with the same size */
     tf = new_float_matrix(newdims,nindx,NULL);
@@ -2679,22 +2679,17 @@ matrix_sum_out_logs(void)
     nmat = (int *)YAP_BlobOfTerm(tf);  
     data = matrix_double_data(mat,ndims);
     ndata = matrix_double_data(nmat,newdims);
-    /* create a new matrix with smaller size */
-    for (i=0;i<nmat[MAT_SIZE];i++) 
-      ndata[i] = 0.0;
+    
+    j = ndims-1;
+    while (j > prdim) {
+      d = d*dims[j];
+      j--;
+    }
+    dd = d*dims[prdim];
+    bzero(ndata, sizeof(double)*nmat[MAT_SIZE]);
     for (i=0; i< mat[MAT_SIZE]; i++) {
-      int j, k;
-      /*
-	not very efficient, we could try to take advantage of the fact
-	that we usually only change an index at a time
-      */
-      matrix_get_index(mat, i, indx);
-      for (j = 0, k=0; j < ndims; j++)  {
-	if (j != prdim) {
-	  nindx[k++]= indx[j];
-	}
-      }
-      ndata[matrix_get_offset(nmat, nindx)] += exp(data[i]);
+      YAP_Int k = i % d + (i/dd)*d;
+      ndata[k] += exp(data[i]);
     }
     for (i=0; i< nmat[MAT_SIZE]; i++) {
       ndata[i] = log(ndata[i]);
@@ -2712,6 +2707,7 @@ matrix_sum_out_logs_several(void)
   int indx[MAX_DIMS], nindx[MAX_DIMS], conv[MAX_DIMS];
   YAP_Term tf, tconv;
   int *mat = (int *)YAP_BlobOfTerm(YAP_ARG1), *nmat;
+
   if (!mat) {
     /* Error */
     return FALSE;
@@ -2799,7 +2795,7 @@ matrix_sum_out_logs_several(void)
   return YAP_Unify(YAP_ARG3, tf);
 }
 
-/* given a matrix M and a set of dims, build contract a matrix to follow
+/* given a matrix M and a set of dims, build a matrix to follow
    the new order
 */ 
 static int
