@@ -608,6 +608,8 @@ execute_command(void)
 {
   YAP_Term ti = YAP_ARG2, to = YAP_ARG3, te = YAP_ARG4;
   int res;
+  YAP_Term AtomNull = YAP_MkAtomTerm(YAP_LookupAtom("null"));
+
 #if defined(__MINGW32__) || _MSC_VER
   HANDLE inpf, outf, errf;
   DWORD CreationFlags = 0;
@@ -666,7 +668,7 @@ execute_command(void)
 #else /* UNIX CODE */
   int inpf, outf, errf;
   /* process input first */
-  if (YAP_IsAtomTerm(ti)) {
+  if (ti == AtomNull) {
     inpf = open("/dev/null", O_RDONLY);
   } else {
     int sd;
@@ -681,7 +683,7 @@ execute_command(void)
     return(YAP_Unify(YAP_ARG6, YAP_MkIntTerm(errno)));
   }
   /* then output stream */
-  if (YAP_IsAtomTerm(to)) {
+  if (to == AtomNull) {
     outf = open("/dev/zero", O_WRONLY);
   } else {
     int sd;
@@ -697,7 +699,7 @@ execute_command(void)
     return(YAP_Unify(YAP_ARG6, YAP_MkIntTerm(errno)));
   }
   /* then error stream */
-  if (YAP_IsAtomTerm(te)) {
+  if (te == AtomNull) {
     errf = open("/dev/zero", O_WRONLY);
   } else {
     int sd;
@@ -729,13 +731,16 @@ execute_command(void)
     /* close current streams, but not std streams */
     YAP_CloseAllOpenStreams();
     close(0);
-    dup(inpf);
+    if (dup(inpf) != 0)
+      exit(1);
     close(inpf);
     close(1);
-    dup(outf);
+    if (dup(outf) != 1)
+      exit(1);
     close(outf);
     close(2);
-    dup(errf);
+    if (dup(errf) != 2)
+      exit(2);
     close(errf);
     argv[0] = "sh";
     argv[1] = "-c";
@@ -867,44 +872,6 @@ p_wait(void)
 #endif
 }
 
-/* execute a command as a detached process */
-static int
-p_popen(void)
-{
-  char *command = (char *)YAP_AtomName(YAP_AtomOfTerm(YAP_ARG1));
-  long int mode = YAP_IntOfTerm(YAP_ARG2);
-  FILE *pfd;
-  YAP_Term tsno;
-  int flags;
-  
-#if HAVE_POPEN
-#if defined(__MINGW32__) || _MSC_VER
-  /* This will only work for console applications. FIX */
-  if (mode == 0)
-    pfd = _popen(command, "r");
-  else
-    pfd = _popen(command, "w");
-#else
-  if (mode == 0)
-    pfd = popen(command, "r");
-  else
-    pfd = popen(command, "w");
-#endif
-  if (pfd == NULL) {
-    return(YAP_Unify(YAP_ARG4, YAP_MkIntTerm(errno)));    
-  }
-  if (mode == 0)
-    flags = YAP_INPUT_STREAM | YAP_POPEN_STREAM;
-  else
-    flags = YAP_OUTPUT_STREAM | YAP_POPEN_STREAM;
-  tsno = YAP_OpenStream((void *)pfd,
-		       "pipe",
-		       YAP_MkAtomTerm(YAP_LookupAtom("pipe")),
-		       flags);
-#endif
-  return(YAP_Unify(YAP_ARG3, tsno));
-}
-
 static int
 p_sleep(void)
 {
@@ -1058,7 +1025,6 @@ init_sys(void)
   YAP_UserCPredicate("exec_command", execute_command, 6);
   YAP_UserCPredicate("do_shell", do_shell, 5);
   YAP_UserCPredicate("do_system", do_system, 3);
-  YAP_UserCPredicate("popen", p_popen, 4);
   YAP_UserCPredicate("wait", p_wait, 3);
   YAP_UserCPredicate("host_name", host_name, 2);
   YAP_UserCPredicate("host_id", host_id, 2);
