@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #define O_DEBUG 1
@@ -111,6 +111,23 @@ leave the details to this function.
 #ifdef __CYGWIN__
 #undef HAVE_H_ERRNO
 #endif
+#ifdef __MINGW32__
+
+//disgusting stuff :(
+#define __try if (1)
+#define __except(A) else
+#endif
+
+#if defined(__MINGW32__)
+#define __try
+#define __except(_) if (0)
+#define __finally
+#endif
+
+#if defined(__MINGW32__)
+#define WINVER 0x0501
+#include <ws2tcpip.h>
+#endif
 
 #include "nonblockio.h"
 
@@ -128,6 +145,15 @@ leave the details to this function.
 #endif
 
 
+
+#ifdef __WINDOWS__
+#define GET_ERRNO WSAGetLastError()
+#define GET_H_ERRNO WSAGetLastError()
+#else
+#define GET_ERRNO errno
+#define GET_H_ERRNO h_errno
+#endif
+
 #ifndef __WINDOWS__
 #define closesocket(n) close((n))	/* same on Unix */
 #endif
@@ -147,30 +173,41 @@ leave the details to this function.
 static CRITICAL_SECTION mutex;
 static CRITICAL_SECTION mutex_free;
 
-#define LOCK() EnterCriticalSection(&mutex)
-#define UNLOCK() LeaveCriticalSection(&mutex)
-#define LOCK_FREE() EnterCriticalSection(&mutex_free)
-#define UNLOCK_FREE() LeaveCriticalSection(&mutex_free)
-#define INITLOCK() (InitializeCriticalSection(&mutex), \
-		    InitializeCriticalSection(&mutex_free))
-#else
+#define LOCK()			EnterCriticalSection(&mutex)
+#define UNLOCK()		LeaveCriticalSection(&mutex)
+#define LOCK_FREE()		EnterCriticalSection(&mutex_free)
+#define UNLOCK_FREE()		LeaveCriticalSection(&mutex_free)
+#define INITLOCK()		( InitializeCriticalSection(&mutex), \
+				  InitializeCriticalSection(&mutex_free))
+#define LOCK_SOCKET(s)		EnterCriticalSection(&s->socket_mutex)
+#define UNLOCK_SOCKET(s)	LeaveCriticalSection(&s->socket_mutex)
+#define INIT_SOCKET_LOCK(s)	InitializeCriticalSection(&s->socket_mutex)
+#define FREE_SOCKET_LOCK(s)	DeleteCriticalSection(&s->socket_mutex)
+#else /*__WINDOWS__*/
 #include <pthread.h>
 
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 
-#define LOCK() pthread_mutex_lock(&mutex)
-#define UNLOCK() pthread_mutex_unlock(&mutex)
-#define LOCK_FREE()
-#define UNLOCK_FREE()
-#define INITLOCK()
-#endif
-#else
-#define LOCK()
-#define UNLOCK()
-#define LOCK_FREE()
-#define UNLOCK_FREE()
-#define INITLOCK()
+#define LOCK()			pthread_mutex_lock(&mutex)
+#define UNLOCK()		pthread_mutex_unlock(&mutex)
+#define LOCK_FREE()		(void)0
+#define UNLOCK_FREE()		(void)0
+#define INITLOCK()		(void)0
+#define FREE_SOCKET_LOCK(s)	(void)0
+#endif /*__WINDOWS__*/
+#else /* _REENTRANT */
+#define LOCK()			(void)0
+#define UNLOCK()		(void)0
+#define LOCK_FREE()		(void)0
+#define UNLOCK_FREE()		(void)0
+#define INITLOCK()		(void)0
+#define FREE_SOCKET_LOCK(s)	(void)0
+#if __WINDOWS__
+#define LOCK_SOCKET(s)		(void)0
+#define UNLOCK_SOCKET(s)	(void)0
+#define INIT_SOCKET_LOCK(s)	(void)0
 #endif
+#endif /* _REENTRANT */
 
 #define set(s, f)   ((s)->flags |= (f))
 #define clear(s, f) ((s)->flags &= ~(f))
@@ -196,8 +233,10 @@ typedef struct _plsocket
   nbio_request	    request;		/* our request */
   DWORD		    thread;		/* waiting thread */
   DWORD		    error;		/* error while executing request */
+  DWORD             close_timeout;      /* Time to reap socket if FD_CLOSE is not received */
   int		    done;		/* request completed */
   int		    w32_flags;		/* or of received FD_* */
+  CRITICAL_SECTION  socket_mutex;       /* synchronization mutex */
   union
   { struct
     { struct sockaddr_in addr;		/* accepted address */
@@ -242,9 +281,11 @@ typedef struct _plsocket
 static plsocket *allocSocket(SOCKET socket);
 #ifdef __WINDOWS__
 static plsocket *lookupOSSocket(SOCKET socket);
+static int socketIsPendingClose(plsocket *s);
 static const char *WinSockError(unsigned long eno);
 #endif
 
+#ifndef __WINDOWS__
 static int
 need_retry(int error)
 { if ( error == EINTR || error == EAGAIN || error == EWOULDBLOCK )
@@ -252,6 +293,7 @@ need_retry(int error)
 
   return FALSE;
 }
+#endif
 
 #ifdef O_DEBUG
 static int debugging;
@@ -343,8 +385,12 @@ nbio_fcntl(nbio_sock_t socket, int op, int arg)
       switch(arg)
       { case O_NONBLOCK:
 	{ int rval;
+#if defined(__MINGW32__)
+	  u_long non_block;
+#else
+          /* FIXME: is this really `int' for MSC? */
 	  int non_block;
-
+#endif
 	  non_block = 1;
 	  rval = ioctlsocket(s->socket, FIONBIO, &non_block);
 	  if ( rval )
@@ -377,9 +423,12 @@ static int
 doneRequest(plsocket *s)
 { SOCKET sock;
 
+  LOCK_SOCKET(s);
   s->done = TRUE;
   s->request = REQ_NONE;
+  UNLOCK_SOCKET(s);
 
+  /* If we have FD_CLOSE, then we know we can release the socket to the OS */
   if ( (s->w32_flags & FD_CLOSE) && (sock=s->socket) >= 0 )
   { s->socket = -1;
     closesocket(sock);
@@ -420,6 +469,8 @@ waitRequest(plsocket *s)
     } else if ( GetMessage(&msg, NULL, 0, 0) )
     { TranslateMessage(&msg);
       DispatchMessage(&msg);
+      if ( PL_exception(0) )
+	return FALSE;			/* DispatchMessage handled a signal */
     } else
     { ExitThread(0);			/* WM_QUIT received */
       return FALSE;			/* NOTREACHED */
@@ -435,11 +486,13 @@ nbio_wait(nbio_sock_t socket, nbio_request request)
   if ( !(s=nbio_to_plsocket(socket)) )
     return -1;
 
+  LOCK_SOCKET(s);
   s->flags  |= PLSOCK_WAITING;
   s->done    = FALSE;
   s->error   = 0;
   s->thread  = GetCurrentThreadId();
   s->request = request;
+  UNLOCK_SOCKET(s);
 
   SendMessage(State()->hwnd, WM_REQUEST, 1, (LPARAM)&s);
 
@@ -501,11 +554,13 @@ nbio_select(int n,
       { plsocket *s = nbio_to_plsocket(i);
 
 	if ( s )
-	{ s->flags  |= PLSOCK_WAITING;
+	{ LOCK_SOCKET(s);
+	  s->flags  |= PLSOCK_WAITING;
 	  s->done    = FALSE;
 	  s->error   = 0;
 	  s->thread  = GetCurrentThreadId();
 	  s->request = (s->flags & PLSOCK_LISTEN) ? REQ_ACCEPT : REQ_READ;
+	  UNLOCK_SOCKET(s);
 	  sockets[i] = s;
 	} else
 	{ DEBUG(2, Sdprintf("nbio_select(): no socket for %d\n", i));
@@ -586,11 +641,13 @@ placeRequest(plsocket *s, nbio_request request)
 { if ( s->magic != PLSOCK_MAGIC )
     Sdprintf("placeRequest: %p has bad magic\n", s);
 
+  LOCK_SOCKET(s);
   s->error   = 0;
   s->done    = FALSE;
   s->thread  = GetCurrentThreadId();
   s->request = request;
   clear(s, PLSOCK_WAITING);
+  UNLOCK_SOCKET(s);
 
   SendMessage(State()->hwnd, WM_REQUEST, 1, (LPARAM)&s);
   DEBUG(2, Sdprintf("%d (%ld): Placed %s request for %d\n",
@@ -661,7 +718,7 @@ doRequest(plsocket *s)
 			    (int)s->socket, WinSockError(s->error)));
 
 	  if ( s->error != WSAEWOULDBLOCK )
-	  { s->rdata.accept.slave = (SOCKET)-1;
+	  { s->rdata.accept.slave = (nbio_sock_t)-1;
 	    doneRequest(s);
 	  }
 	} else
@@ -775,7 +832,14 @@ doRequest(plsocket *s)
 	{ s->rdata.write.bytes = (int)s->rdata.write.written;
 	  doneRequest(s);
 	}
+      } else if ( s->w32_flags & FD_CLOSE )
+      { if ( s->rdata.write.written > 0 )
+	  s->rdata.write.bytes = (int)s->rdata.write.written;
+	else
+	  s->rdata.write.bytes = -1;
+	doneRequest(s);
       }
+      break;
     case REQ_SENDTO:
       if ( s->w32_flags & FD_WRITE )
       { int n;
@@ -809,6 +873,7 @@ doRequest(plsocket *s)
 	s->rdata.sendto.bytes = n;
 	doneRequest(s);
       }
+      break;
   }
 
   return TRUE;
@@ -823,21 +888,13 @@ socket_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 
     for(i=0; i<n; i++)
     { if ( s[i] )
-      {
-#if __MINGW32__
-	{ if ( s[i]->magic != PLSOCK_MAGIC )
-	  { goto nosocket;
-	  }
-	} 
-#else
-	__try
+      { __try
 	{ if ( s[i]->magic != PLSOCK_MAGIC )
 	  { goto nosocket;
 	  }
 	} __except(EXCEPTION_EXECUTE_HANDLER)
 	{ goto nosocket;
 	}
-#endif
 
 	doRequest(s[i]);
       }
@@ -892,13 +949,21 @@ socket_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 
 	s->error = err;
 	if ( sock )
-	{ s->socket = -1;
-	  closesocket(sock);
+	{ /* We cannot close the socket yet, since the late arrival
+             of FD_CLOSE might be delivered to this socket even after
+	     it has been reallocated. Instead, calculate a timeout to
+             allow for the case when FD_CLOSE never comes, and then
+	     continue as normal
+          */
+          socketIsPendingClose(s);
 	}
 
 	switch(s->request)
 	{ case REQ_CONNECT:
 	    break;
+          case REQ_NONE:
+            /* FIXME: is this OK? */
+            break;
 	  case REQ_ACCEPT:
 	    s->rdata.accept.slave = SOCKET_ERROR;
 	    break;
@@ -961,17 +1026,6 @@ HiddenFrameClass()
 }
 
 
-static void
-destroyHiddenWindow(int rval, void *closure)
-{ local_state *s = State();
-
-  if ( s->hwnd )
-  { DestroyWindow(s->hwnd);
-    s->hwnd = 0;
-  }
-}
-
-
 static HWND
 SocketHiddenWindow()
 { local_state *s = State();
@@ -1088,7 +1142,7 @@ nbio_fcntl(nbio_sock_t socket, int op, int arg)
   { if ( op == F_SETFL && arg == O_NONBLOCK )
       s->flags |= PLSOCK_NONBLOCK;
   } else
-    nbio_error(errno, TCP_ERRNO);
+    nbio_error(GET_ERRNO, TCP_ERRNO);
 
   return rc;
 }
@@ -1236,6 +1290,8 @@ allocSocket(SOCKET socket)
   size_t i;
 
 #ifdef __WINDOWS__
+  DWORD now = GetTickCount();
+
   if ( (p=lookupOSSocket(socket)) )
   { DEBUG(1, Sdprintf("WinSock %d already registered on %d\n",
 		      (int)socket, p->id));
@@ -1260,10 +1316,26 @@ allocSocket(SOCKET socket)
   }
 
   for(i=0; i<socks_allocated; i++)
-  { if ( sockets[i] == NULL )
+  { if ( (p=sockets[i]) == NULL )
     { sockets[i] = p = PL_malloc(sizeof(*p));
       socks_count++;
       break;
+#ifdef __WINDOWS__
+    } else
+    { if ( p->close_timeout && p->close_timeout < now )
+      { SOCKET sock;
+
+	p->close_timeout = 0;
+	if ( (sock=p->socket) )		/* is this ever the case? */
+	{ int rval;
+
+	  do
+	  { rval = closesocket(sock);
+	  } while(rval == SOCKET_ERROR && errno == EINTR);
+	}
+	break;
+      }
+#endif
     }
   }
   UNLOCK();
@@ -1278,6 +1350,7 @@ allocSocket(SOCKET socket)
 #ifdef __WINDOWS__
   p->w32_flags = 0;
   p->request   = REQ_NONE;
+  INIT_SOCKET_LOCK(p);
 #endif
   p->input = p->output = (IOSTREAM*)NULL;
 
@@ -1288,6 +1361,7 @@ allocSocket(SOCKET socket)
 }
 
 
+#ifndef __WINDOWS__
 static int
 freeSocket(plsocket *s)
 { int rval;
@@ -1309,6 +1383,7 @@ freeSocket(plsocket *s)
   sock = s->socket;
   socket = s->id;
   s->magic = 0;
+  FREE_SOCKET_LOCK(s);
   PL_free(s);
   UNLOCK_FREE();
 
@@ -1326,6 +1401,7 @@ freeSocket(plsocket *s)
 
   return rval;
 }
+#endif
 
 
 		 /*******************************
@@ -1333,13 +1409,13 @@ freeSocket(plsocket *s)
 		 *******************************/
 
 #ifdef __WINDOWS__
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 The code in BILLY_GETS_BETTER is, according to various documents the
 right code, but it doesn't work, so we do it by hand.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #ifdef BILLY_GETS_BETTER
-
 static const char *
 WinSockError(unsigned long eno)
 { char buf[1024];
@@ -1421,10 +1497,10 @@ WinSockError(unsigned long error)
     { WSANOTINITIALISED, "Socket layer not initialised" },
 					/* WinSock 2 errors */
     { WSAHOST_NOT_FOUND, "Host not found" },
+    { WSANO_DATA, "Valid name, no data record of requested type" },
     { 0, NULL }
   };
   char tmp[100];
-
   for(ep=edefs; ep->string; ep++)
   { if ( ep->index == (int)error )
       return ep->string;
@@ -1475,7 +1551,6 @@ nbio_error(int code, nbio_error_map mapid)
 
   if ( code == EPLEXCEPTION )
     return FALSE;
-
   switch( mapid )
   { case TCP_HERRNO:
       map = h_errno_codes;
@@ -1483,7 +1558,6 @@ nbio_error(int code, nbio_error_map mapid)
     default:
       map = NULL;
   }
-
   {
 #ifdef __WINDOWS__
   msg = WinSockError(code);
@@ -1541,15 +1615,12 @@ nbio_last_error(nbio_sock_t socket)
 
 NBIO_EXPORT(int)
 nbio_init(const char *module)
-{ INITLOCK();				/* is this ok? */
-
-  LOCK();
-  if ( initialised )
-  { UNLOCK();
-    return TRUE;
-  }
+{ if ( initialised )			/* called from install handlers, which */
+    return TRUE;			/* are serialized by the compiler mutex */
   initialised = TRUE;
 
+  INITLOCK();
+
   FUNCTOR_module2  = PL_new_functor(PL_new_atom(":"), 2);
   FUNCTOR_ip4	   = PL_new_functor(PL_new_atom("ip"), 4);
   FUNCTOR_ip1	   = PL_new_functor(PL_new_atom("ip"), 1);
@@ -1570,14 +1641,11 @@ nbio_init(const char *module)
 #endif
 
   if ( WSAStartup(MAKEWORD(2,0), &WSAData) )
-  { UNLOCK();
     return PL_warning("nbio_init() - WSAStartup failed.");
-  }
   startSocketThread();
 }
 #endif /*__WINDOWS__*/
 
-  UNLOCK();
   return TRUE;
 }
 
@@ -1594,6 +1662,25 @@ nbio_cleanup(void)
   return 0;
 }
 
+#ifdef __WINDOWS__
+
+/* socketIsPendingClose() is called if a Windows socket is closed, but
+   we did not yet see an FD_CLOSE message for it.  We will reuse this
+   socket using lookupTimedOutSocket() if we have not seen the FD_CLOSE
+   after 30 seconds.  This is a random time.  Are we guaranteed to get
+   an FD_CLOSE?
+*/
+
+static int
+socketIsPendingClose(plsocket *s)
+{ s->close_timeout = GetTickCount() + 30 * 1000;
+  shutdown(s->socket, SD_BOTH);
+  DEBUG(3, Sdprintf("Setting timeout for FD_CLOSE on socket %d...\n", s->id));
+
+  return 0;
+}
+
+#endif /*__WINDOWS__*/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 socket(-Socket)
@@ -1609,7 +1696,7 @@ nbio_socket(int domain, int type, int protocol)
   assert(initialised);
 
   if ( (sock = socket(domain, type , protocol)) < 0)
-  { nbio_error(errno, TCP_ERRNO);
+  { nbio_error(GET_ERRNO, TCP_ERRNO);
     return -1;
   }
   if ( !(s=allocSocket(sock)) )		/* register it */
@@ -1648,14 +1735,12 @@ nbio_closesocket(nbio_sock_t socket)
     }
   } else
   {
+  // We cannot free the socket in Windows since we might subsequently get an FD_CLOSE. Instead set the timeout
 #ifdef __WINDOWS__
-    if ( true(s, PLSOCK_CONNECT) )
-    { if ( s->socket >= 0 )
-	shutdown(s->socket, SD_SEND);
-    }
+  socketIsPendingClose(s);
+#else
+  freeSocket(s);
 #endif
-
-    freeSocket(s);
   }
 
   return 0;
@@ -1681,7 +1766,7 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
 
       if( setsockopt(s->socket, SOL_SOCKET, SO_REUSEADDR,
 		     (const char *)&val, sizeof(val)) == -1 )
-      { nbio_error(h_errno, TCP_HERRNO);
+      { nbio_error(GET_H_ERRNO, TCP_HERRNO);
 	rc = -1;
       } else
 	rc = 0;
@@ -1697,7 +1782,7 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
 #endif
       if ( setsockopt(s->socket, IPPROTO_TCP, TCP_NODELAY,
 		      (const char *)&val, sizeof(val)) == -1 )
-      { nbio_error(h_errno, TCP_HERRNO);
+      { nbio_error(GET_H_ERRNO, TCP_HERRNO);
 	rc = -1;
       } else
       { rc = 0;
@@ -1713,7 +1798,7 @@ nbio_setopt(nbio_sock_t socket, nbio_option opt, ...)
 
       if ( setsockopt(s->socket, SOL_SOCKET, SO_BROADCAST,
 		     (const char *)&val, sizeof(val)) == -1 )
-      { nbio_error(h_errno, TCP_HERRNO);
+      { nbio_error(GET_H_ERRNO, TCP_HERRNO);
 	rc = -1;
       } else
 	rc = 0;
@@ -1803,11 +1888,13 @@ nbio_get_port(term_t Port, int *port)
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Convert a term Host:Port to a socket address.  Port is either an integer
 or the name of a registered port (e.g. 'smtp').
+
+(*) TBD: Supply the port/service here too, simplifying the rest
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 int
 nbio_get_sockaddr(term_t Address, struct sockaddr_in *addr)
-{ int		  port;
+{ int port;
 
   addr->sin_family = AF_INET;
   addr->sin_addr.s_addr = INADDR_ANY;
@@ -1818,13 +1905,18 @@ nbio_get_sockaddr(term_t Address, struct sockaddr_in *addr)
 
     _PL_get_arg(1, Address, arg);
     if ( PL_get_atom_chars(arg, &hostName) )
-    { struct hostent *host;
+    { struct addrinfo hints;
+      struct addrinfo *res;
 
-      if( !(host = gethostbyname(hostName)) )
-	return nbio_error(h_errno, TCP_HERRNO);
-      if ( (int)sizeof(addr->sin_addr) < host->h_length )
-	return PL_warning("Oops, host address too long!");
-      memcpy(&addr->sin_addr, host->h_addr, host->h_length);
+      memset(&hints, 0, sizeof(hints));
+      hints.ai_family = AF_INET;
+      if ( getaddrinfo(hostName, NULL, &hints, &res) !=  0) /* see (*) */
+	return nbio_error(GET_H_ERRNO, TCP_HERRNO);
+      assert(res->ai_family == AF_INET);
+      memcpy(&addr->sin_addr,
+	     &((struct sockaddr_in*)res->ai_addr)->sin_addr,
+	     sizeof(addr->sin_addr));
+      freeaddrinfo(res);
     } else if ( !nbio_get_ip(arg, &addr->sin_addr) )
     { return pl_error(NULL, 0, NULL, ERR_ARGTYPE, 1, arg, "atom|ip/4");
     }
@@ -1908,7 +2000,8 @@ nbio_bind(nbio_sock_t socket, struct sockaddr *my_addr, size_t addrlen)
 #else
   if ( bind(s->socket, my_addr, addrlen) )
 #endif
-  { nbio_error(errno, TCP_ERRNO);
+  {
+    nbio_error(GET_ERRNO, TCP_ERRNO);
     return -1;
   }
 
@@ -1935,7 +2028,8 @@ nbio_connect(nbio_sock_t socket,
     { s->rdata.connect.addrlen = addrlen;
       memcpy(&s->rdata.connect.addr, serv_addr, addrlen);
       placeRequest(s, REQ_CONNECT);
-      waitRequest(s);
+      if ( !waitRequest(s) )
+	return -1;
     }
 
     if ( s->error )
@@ -1946,12 +2040,12 @@ nbio_connect(nbio_sock_t socket,
 #else /*!__WINDOWS__*/
   for(;;)
   { if ( connect(s->socket, serv_addr, addrlen) )
-    { if ( need_retry(errno) )
+    { if ( need_retry(GET_ERRNO) )
       { if ( PL_handle_signals() < 0 )
 	  return -1;
 	continue;
       }
-      nbio_error(errno, TCP_ERRNO);
+      nbio_error(GET_ERRNO, TCP_ERRNO);
       return -1;
     } else
       break;
@@ -2007,13 +2101,13 @@ nbio_accept(nbio_sock_t master, struct sockaddr *addr, socklen_t *addrlen)
     slave = accept(m->socket, addr, addrlen);
 
     if ( slave == SOCKET_ERROR )
-    { if ( need_retry(errno) )
+    { if ( need_retry(GET_ERRNO) )
       { if ( PL_handle_signals() < 0 )
 	  return -1;
 
 	continue;
       } else
-      { nbio_error(errno, TCP_ERRNO);
+      { nbio_error(GET_ERRNO, TCP_ERRNO);
 	return -1;
       }
     } else
@@ -2041,7 +2135,7 @@ nbio_listen(nbio_sock_t socket, int backlog)
     return -1;
 
   if( listen(s->socket, backlog) == -1 )
-  { nbio_error(errno, TCP_ERRNO);
+  { nbio_error(GET_ERRNO, TCP_ERRNO);
     return -1;
   }
 
@@ -2103,7 +2197,7 @@ nbio_read(int socket, char *buf, size_t bufSize)
 
     n = recv(s->socket, buf, bufSize, 0);
 
-    if ( n == -1 && need_retry(errno) )
+    if ( n == -1 && need_retry(GET_ERRNO) )
     { if ( PL_handle_signals() < 0 )
       { errno = EPLEXCEPTION;
 	return -1;
@@ -2175,7 +2269,7 @@ nbio_write(nbio_sock_t socket, char *buf, size_t bufSize)
 
     n = send(s->socket, str, len, 0);
     if ( n < 0 )
-    { if ( need_retry(errno) )
+    { if ( need_retry(GET_ERRNO) )
       { if ( PL_handle_signals() < 0 )
 	{ errno = EPLEXCEPTION;
 	  return -1;
@@ -2213,19 +2307,17 @@ nbio_close_input(nbio_sock_t socket)
     if ( (sock=s->socket) < 0 )
     { s->error = WSAECONNRESET;
       rc = -1;
-    } else if ( shutdown(sock, SD_RECEIVE) == SOCKET_ERROR )
-    { DEBUG(1, Sdprintf("shutdown(%d=%d, SD_RECEIVE) failed: %s\n",
-			socket, s->socket,
-			WinSockError(WSAGetLastError())));
-      Sseterr(s->input, SIO_FERR, WinSockError(WSAGetLastError()));
-      rc = -1;
     }
   }
 #endif
 
   s->input = NULL;
   if ( !(s->flags & (PLSOCK_INSTREAM|PLSOCK_OUTSTREAM)) )
+#ifdef __WINDOWS__
+    return socketIsPendingClose(s);
+#else
     return freeSocket(s);
+#endif
 
   return rc;
 }
@@ -2252,14 +2344,6 @@ nbio_close_output(nbio_sock_t socket)
     if ( (sock=s->socket) < 0 )
     { s->error = WSAECONNRESET;
       rc = -1;
-    } else if ( shutdown(sock, SD_SEND) == SOCKET_ERROR )
-    { const char *msg;
-
-      msg = WinSockError(WSAGetLastError());
-      Sseterr(s->output, SIO_FERR, msg);
-      DEBUG(1, Sdprintf("shutdown(%d=%d, SD_SEND) failed: %s\n",
-			socket, s->socket, msg));
-      rc = -1;
     }
 #endif
   }
@@ -2267,7 +2351,11 @@ nbio_close_output(nbio_sock_t socket)
   DEBUG(3, Sdprintf("%d->flags = 0x%x\n", socket, s->flags));
   s->output = NULL;
   if ( !(s->flags & (PLSOCK_INSTREAM|PLSOCK_OUTSTREAM)) )
+#ifdef __WINDOWS__
+    return socketIsPendingClose(s);
+#else
     return freeSocket(s);
+#endif
 
   return rc;
 }
@@ -2330,7 +2418,7 @@ nbio_recvfrom(int socket, void *buf, size_t bufSize, int flags,
 
     n = recvfrom(s->socket, buf, bufSize, flags, from, fromlen);
 
-    if ( n == -1 && need_retry(errno) )
+    if ( n == -1 && need_retry(GET_ERRNO) )
     { if ( PL_handle_signals() < 0 )
       { errno = EPLEXCEPTION;
 	return -1;
