@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifdef HAVE_CONFIG_H
@@ -63,7 +63,6 @@ static atom_t ATOM_infinite;
 static functor_t FUNCTOR_error2;
 static functor_t FUNCTOR_type_error2;
 static functor_t FUNCTOR_domain_error2;
-static functor_t FUNCTOR_resource_error1;
 static functor_t FUNCTOR_process_error2;
 static functor_t FUNCTOR_system_error2;
 static functor_t FUNCTOR_pipe1;
@@ -79,6 +78,24 @@ static functor_t FUNCTOR_eq2;		/* =/2 */
 #define DEBUG(g) (void)0
 #endif
 
+#ifdef __WINDOWS__
+#include <windows.h>
+#include <stdio.h>
+#include <fcntl.h>
+#include <io.h>
+#if !defined(__MINGW32__)
+typedef DWORD  pid_t;
+#endif
+typedef wchar_t echar;			/* environment character */
+
+#ifndef CREATE_BREAKAWAY_FROM_JOB
+#define CREATE_BREAKAWAY_FROM_JOB 0x1000000
+#endif
+
+#else
+typedef char echar;
+#endif
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ISSUES:
 	- Deal with child errors (no cwd, cannot execute, etc.)
@@ -125,39 +142,10 @@ domain_error(term_t actual, const char *expected)
 }
 
 
-static int
-resource_error(const char *resource)
-{ term_t ex;
-
-  if ( (ex=PL_new_term_ref()) &&
-       PL_unify_term(ex,
-		     PL_FUNCTOR, FUNCTOR_error2,
-		       PL_FUNCTOR, FUNCTOR_resource_error1,
-		         PL_CHARS, resource,
-		       PL_VARIABLE) )
-    return PL_raise_exception(ex);
-
-  return FALSE;
-}
-
-
 		 /*******************************
 		 *	       ADMIN		*
 		 *******************************/
 
-#ifdef __WINDOWS__
-#include <windows.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <io.h>
-#ifndef __MINGW32__
-typedef DWORD  pid_t;
-#endif
-typedef wchar_t echar;			/* environment character */
-#else
-typedef char echar;
-#endif
-
 typedef enum std_type
 { std_std,
   std_null,
@@ -292,10 +280,11 @@ parse_environment(term_t t, p_options *info)
   term_t head = PL_new_term_ref();
   term_t tmp  = PL_new_term_ref();
   ecbuf *eb   = &info->envbuf;
-  int count = 0, c = 0;
+  int count = 0;
 #ifndef __WINDOWS__
   echar *q;
   char **ep;
+  int c = 0;
 #endif
 
   assert(eb->size == 0);
@@ -443,9 +432,13 @@ get_exe(term_t exe, p_options *info)
   if ( !PL_get_chars(arg, &info->exe, CVT_ATOM|CVT_EXCEPTION|BUF_MALLOC|REP_FN) )
     return FALSE;
 
-  info->argv = PL_malloc((arity+2)*sizeof(char*));
+  if ( !(info->argv = PL_malloc((arity+2)*sizeof(char*))) )
+    return PL_resource_error("memory");
   memset(info->argv, 0, (arity+2)*sizeof(char*));
-  info->argv[0] = strdup(info->exe);
+  if ( !(info->argv[0] = PL_malloc(strlen(info->exe)+1)) )
+    return PL_resource_error("memory");
+  strcpy(info->argv[0], info->exe);
+
   { int i;
 
     for(i=1; i<=arity; i++)
@@ -618,7 +611,7 @@ open_process_pipe(process_context *pc, int which, int fd)
 
   pc->open_mask |= (1<<which);
 #ifdef __WINDOWS__
-  pc->pipes[which] = _open_osfhandle((long)fd, _O_BINARY);
+  pc->pipes[which] = _open_osfhandle((intptr_t)fd, _O_BINARY);
 #else
   pc->pipes[which] = fd;
 #endif
@@ -653,57 +646,7 @@ win_init()
 }
 
 
-static atom_t
-WinError()
-{ int id = GetLastError();
-  char *msg;
-  static WORD lang;
-  static lang_initialised = 0;
-
-  if ( !lang_initialised )
-    lang = MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_UK);
-
-again:
-  if ( FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
-		     FORMAT_MESSAGE_IGNORE_INSERTS|
-		     FORMAT_MESSAGE_FROM_SYSTEM,
-		     NULL,			/* source */
-		     id,			/* identifier */
-		     lang,
-		     (LPTSTR) &msg,
-		     0,				/* size */
-		     NULL) )			/* arguments */
-  { atom_t a = PL_new_atom(msg);
-
-    LocalFree(msg);
-    lang_initialised = 1;
-
-    return a;
-  } else
-  { if ( lang_initialised == 0 )
-    { lang = MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT);
-      lang_initialised = 1;
-      goto again;
-    }
-
-    return PL_new_atom("Unknown Windows error");
-  }
-}
-
-
-static int
-win_error(const char *op)
-{ atom_t msg = WinError();
-  term_t ex = PL_new_term_ref();
-
-  PL_unify_term(ex, PL_FUNCTOR, FUNCTOR_error2,
-		      PL_FUNCTOR, FUNCTOR_system_error2,
-		        PL_CHARS, op,
-		        PL_ATOM, msg,
-		      PL_VARIABLE);
-
-  return PL_raise_exception(ex);
-}
+#include "win_error.c"
 
 
 typedef struct arg_string
@@ -771,7 +714,7 @@ win_command_line(term_t t, int arity, const wchar_t *exe, wchar_t **cline)
     cmdlen = av[0].len+(av[0].quote?2:0)+1;
 
     for( i=1; i<=arity; i++)
-    { PL_get_arg(i, t, arg);
+    { _PL_get_arg(i, t, arg);
 
       if ( !PL_get_wchars(arg, &av[i].len, &av[i].text,
 			  CVT_ATOMIC|CVT_EXCEPTION|BUF_MALLOC) )
@@ -873,9 +816,8 @@ find_process_from_pid(DWORD pid, const char *pred)
   if ( pred )
   { term_t ex = PL_new_term_ref();
 
-    PL_put_integer(ex, pid);
-    pl_error(NULL, 2, NULL, ERR_EXISTENCE,
-	     "process", ex);
+    if ( PL_put_integer(ex, pid) )
+      PL_existence_error("process", ex);
   }
 
   return (HANDLE)0;
@@ -976,17 +918,17 @@ win_wait_success(atom_t exe, HANDLE process)
     return FALSE;
 
   if ( rc != 0 )
-  { term_t code = PL_new_term_ref();
-    term_t ex = PL_new_term_ref();
+  { term_t ex = PL_new_term_ref();
 
-    PL_unify_term(ex,
-		  PL_FUNCTOR, FUNCTOR_error2,
-		    PL_FUNCTOR, FUNCTOR_process_error2,
-		      PL_ATOM, exe,
-		      PL_FUNCTOR, FUNCTOR_exit1,
-		        PL_LONG, rc,
-		    PL_VARIABLE);
-    return PL_raise_exception(ex);
+    if ( PL_unify_term(ex,
+		       PL_FUNCTOR, FUNCTOR_error2,
+		         PL_FUNCTOR, FUNCTOR_process_error2,
+		           PL_ATOM, exe,
+		           PL_FUNCTOR, FUNCTOR_exit1,
+		             PL_LONG, rc,
+		         PL_VARIABLE) )
+      return PL_raise_exception(ex);
+    return FALSE;
   }
 
   return TRUE;
@@ -1006,8 +948,14 @@ create_pipes(p_options *info)
   { p_stream *s = &info->streams[i];
 
     if ( s->term )
-    { if ( !CreatePipe(&s->fd[0], &s->fd[1], &sa, 1<<13) )
-      { return win_error("CreatePipe");
+    { if ( i == 2 && info->streams[1].term &&
+	   PL_compare(info->streams[1].term, info->streams[2].term) == 0 )
+      { s->fd[0] = info->streams[1].fd[0];
+	s->fd[1] = info->streams[1].fd[1];
+      } else
+      { if ( !CreatePipe(&s->fd[0], &s->fd[1], &sa, 1<<13) )
+	{ return win_error("CreatePipe");
+	}
       }
     }
   }
@@ -1018,7 +966,7 @@ create_pipes(p_options *info)
 
 static IOSTREAM *
 Sopen_handle(HANDLE h, const char *mode)
-{ return Sfdopen(_open_osfhandle((long)h, _O_BINARY), mode);
+{ return Sfdopen(_open_osfhandle((intptr_t)h, _O_BINARY), mode);
 }
 
 
@@ -1073,6 +1021,11 @@ do_create_process(p_options *info)
       break;
   }
 
+  if ( info->detached )
+    flags |= CREATE_BREAKAWAY_FROM_JOB;
+  if ( info->envbuf.buffer )
+    flags |= CREATE_UNICODE_ENVIRONMENT;
+
   memset(&si, 0, sizeof(si));
   si.cb = sizeof(si);
   si.dwFlags = STARTF_USESTDHANDLES;
@@ -1200,9 +1153,15 @@ create_pipes(p_options *info)
   { p_stream *s = &info->streams[i];
 
     if ( s->term )
-    { if ( pipe(s->fd) )
-      { assert(errno = EMFILE);
-	return resource_error("open_files");
+    { if ( i == 2 && info->streams[1].term &&
+	   PL_compare(info->streams[1].term, info->streams[2].term) == 0 )
+      { s->fd[0] = info->streams[1].fd[0];
+	s->fd[1] = info->streams[1].fd[1];
+      } else
+      { if ( pipe(s->fd) )
+	{ assert(errno = EMFILE);
+	  return PL_resource_error("open_files");
+	}
       }
     }
   }
@@ -1318,13 +1277,16 @@ wait_success(atom_t name, pid_t pid)
 }
 
 
+#ifndef HAVE_VFORK
+#define vfork fork
+#endif
+
 static int
 do_create_process(p_options *info)
 { int pid;
 
-  if ( !(pid=fork()) )			/* child */
+  if ( !(pid=vfork()) )			/* child */
   { int fd;
-    int rc;
 
     PL_cleanup_fork();
 
@@ -1379,21 +1341,18 @@ do_create_process(p_options *info)
     }
 
     if ( info->envp )
-      rc = execve(info->exe, info->argv, info->envp);
+      execve(info->exe, info->argv, info->envp);
     else
-      rc = execv(info->exe, info->argv);
+      execv(info->exe, info->argv);
 
-    if ( rc )
-    { perror(info->exe);
-      exit(1);
-    }
+    perror(info->exe);
+    exit(1);
+  } else if ( pid < 0 )			/* parent */
+  { term_t exe = PL_new_term_ref();
+    PL_put_atom_chars(exe, info->exe);
 
-    { term_t exe = PL_new_term_ref();
-      PL_put_atom_chars(exe, info->exe);
-
-      return pl_error(NULL, 0, "execv", ERR_ERRNO, errno, "exec", "process", exe);
-    }
-  } else				/* parent */
+    return pl_error(NULL, 0, "fork", ERR_ERRNO, errno, "fork", "process", exe);
+  } else
   { if ( info->pipes > 0 && info->pid == 0 )
     { IOSTREAM *s;
       process_context *pc = PL_malloc(sizeof(*pc));
@@ -1462,7 +1421,7 @@ Basic process creation interface takes
 	* List of arguments
 	* standard streams		% std, null, pipe(S)
 	* Working directory
-	* detached			% Unix
+	* detached
 	* window			% Windows
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
@@ -1547,7 +1506,7 @@ process_wait(term_t pid, term_t code, term_t options)
 
 static foreign_t
 process_kill(term_t pid, term_t signal)
-{ int p;
+{ pid_t p;
 
   if ( !get_pid(pid, &p) )
     return FALSE;
@@ -1617,7 +1576,6 @@ install_process()
   MKFUNCTOR(domain_error, 2);
   MKFUNCTOR(process_error, 2);
   MKFUNCTOR(system_error, 2);
-  MKFUNCTOR(resource_error, 1);
   MKFUNCTOR(exit, 1);
   MKFUNCTOR(killed, 1);
 
