@@ -1,11 +1,9 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2009, VU University Amsterdam
+    Copyright (C): 2009-2012, VU University Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -19,7 +17,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifdef HAVE_CONFIG_H
@@ -121,7 +119,7 @@ domain_error(const char *expected, term_t found)
 		 *******************************/
 
 #define	ESC_PATH       (CH_PCHAR|CH_EX_PATH)
-#define	ESC_QUERY      (CH_PCHAR|CH_EX_QF)
+#define	ESC_QUERY      (CH_UNRESERVED|CH_PSUBDELIM|CH_EX_PCHAR|CH_EX_QF)
 #define	ESC_QVALUE     (CH_UNRESERVED|CH_QSUBDELIM|CH_EX_PCHAR|CH_EX_QF)
 #define	ESC_QNAME      (CH_PCHAR)
 #define	ESC_FRAGMENT   (CH_PCHAR|CH_EX_QF)
@@ -142,7 +140,8 @@ domain_error(const char *expected, term_t found)
 #define CH_EX_QF      0x0080		/* Extra query and fragment chars */
 #define CH_EX_SCHEME  0x0100
 #define CH_QSUBDELIM  0x0200
-#define CH_EX_PATH    0x0400
+#define CH_PSUBDELIM  0x0400
+#define CH_EX_PATH    0x0800
 
 #define CH_SCHEME	(CH_ALPHA|CH_DIGIT|CH_EX_SCHEME)
 #define CH_UNRESERVED	(CH_ALPHA|CH_DIGIT|CH_EX_UNRES)
@@ -172,7 +171,8 @@ fill_flags()
     set_flags("-._~",        CH_EX_UNRES);
     set_flags(":/?#[]@",     CH_GENDELIM);
     set_flags("!$&'()+*,;=", CH_SUBDELIM);
-    set_flags("!$'()*,;",    CH_QSUBDELIM); /* = CH_SUBDELIM - "&=+" */
+    set_flags("!$&'()*,;=",  CH_PSUBDELIM); /* = CH_SUBDELIM - "+"  */
+    set_flags("!$'()*,",     CH_QSUBDELIM); /* = CH_SUBDELIM - "&=+" */
     set_flags(":@",          CH_EX_PCHAR);
     set_flags("/",           CH_EX_PATH);
     set_flags("/?",          CH_EX_QF);
@@ -487,6 +487,7 @@ add_decoded_range_charbuf(charbuf *cb, const range *r, int flags)
 static int
 add_normalized_range_charbuf(charbuf *cb, const range *r, int iri, int flags)
 { const pl_wchar_t *s = r->start;
+
   while(s<r->end)
   { int c;
 
@@ -797,11 +798,13 @@ uri_is_global(term_t URI)
     const pl_wchar_t *end = &s[len];
     range r;
 
+    fill_flags();
+
     e = skip_not(s, end, L":/?#");
     if ( e > s && e[0] == ':' )
     { r.start = s;
       r.end = e;
-      if ( range_is_unreserved(&r, TRUE, CH_SCHEME) )
+      if ( range_is_unreserved(&r, FALSE, CH_SCHEME) )
 	return TRUE;
     }
   }
@@ -849,7 +852,7 @@ unify_query_string_components(term_t list, size_t len, const pl_wchar_t *qs)
       name.end   = skip_not(qs, end, L"=");
       if ( name.end < end )
       { value.start = name.end+1;
-	value.end   = skip_not(value.start, end, L"&");
+	value.end   = skip_not(value.start, end, L"&;");
 
 	qs = value.end+1;
       } else
@@ -1152,7 +1155,7 @@ normalize_in_charbuf(charbuf *cb, uri_component_ranges *ranges, int iri)
   }
   if ( ranges->fragment.start )
   { add_charbuf(cb, '#');
-    add_range_charbuf(cb, &ranges->fragment, iri, ESC_FRAGMENT);
+    add_range_charbuf(cb, &ranges->fragment, iri, ESC_QVALUE);
   }
 
   return TRUE;
@@ -1228,8 +1231,8 @@ ranges_in_charbuf(charbuf *cb, uri_component_ranges *ranges)
 
 
 typedef struct
-{ atom_t     	       atom;
-  pl_wchar_t 	      *text;
+{ atom_t	       atom;
+  pl_wchar_t	      *text;
   uri_component_ranges ranges;
 } base_cache;
 
@@ -1258,7 +1261,7 @@ myBase()
 
   if ( (base=pthread_getspecific(base_key)) )
     return base;
-  base = PL_malloc(sizeof(*base));
+  base = PL_malloc_uncollectable(sizeof(*base)); /* Due to bug in Boehm-GC */
   memset(base, 0, sizeof(*base));
 
   pthread_setspecific(base_key, base);
@@ -1596,3 +1599,12 @@ install_uri()
   PL_register_foreign("uri_encoded",	      3, uri_encoded,	       0);
   PL_register_foreign("uri_iri",	      2, uri_iri,	       0);
 }
+
+
+install_t
+uninstall_uri()
+{
+#ifdef _REENTRANT
+ pthread_key_delete(base_key);
+#endif
+}
