@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -61,7 +61,8 @@ available test sets. The public goals are:
 
 test_rdf_db :-
 	test,
-	run_tests([ lang_matches
+	run_tests([ lang_matches,
+		    lit_ranges
 		  ]).
 
 
@@ -727,24 +728,35 @@ graph(a, p, b).
 graph(b, p, c).
 graph(c, p, d).
 graph(b, p, d).
+graph(e, p, d).
 
-graph :-
+graph(Symmetric) :-
+	rdf_set_predicate(p, symmetric(Symmetric)),
 	forall(graph(S,P,O),
 	       rdf_assert(S,P,O)).
 
 reachable(1) :-
 	rdf_reachable(a, x, a).
 reachable(2) :-
-	graph,
+	graph(false),
 	rdf_reachable(a, p, d).
 reachable(3) :-
-	graph,
+	graph(false),
 	rdf_reachable(a, p, X),
 	X == c, !.
 reachable(4) :-
-	graph,
+	graph(false),
 	findall(O, rdf_reachable(a, p, O), Os),
 	Os = [a,b,c,d].
+reachable(5) :-
+	graph(false),
+	\+ rdf_reachable(d, p, a).
+reachable(6) :-
+	graph(true),
+	rdf_reachable(d, p, a).
+reachable(6) :-
+	graph(true),
+	rdf_reachable(e, p, a).
 
 
 		 /*******************************
@@ -812,7 +824,7 @@ run_test_script(Script) :-
 	file_base_name(Script, Base),
 	file_name_extension(Pred, _, Base),
 	load_files(Script, [silent(true)]),
-	Pred.
+	Pred:Pred.
 
 run_test_scripts(Directory) :-
 	(   script_dir(ScriptDir),
@@ -824,7 +836,7 @@ run_test_scripts(Directory) :-
 	atom_concat(Dir, '/*.pl', Pattern),
 	expand_file_name(Pattern, Files),
 	file_base_name(Dir, BaseDir),
-	format('Running scripts from ~w ', [BaseDir]), flush,
+	format('Running scripts from ~w ', [BaseDir]), flush_output,
 	run_scripts(Files),
 	format(' done~n').
 
@@ -832,10 +844,10 @@ run_scripts([]).
 run_scripts([H|T]) :-
 	(   catch(run_test_script(H), Except, true)
 	->  (   var(Except)
-	    ->  put(.), flush
+	    ->  put(.), flush_output
 	    ;   Except = blocked(Reason)
 	    ->  assert(blocked(H, Reason)),
-		put(!), flush
+		put(!), flush_output
 	    ;   script_failed(H, Except)
 	    )
 	;   script_failed(H, fail)
@@ -925,17 +937,17 @@ report_failed :-
 
 runtest(Name) :-
 	format('Running test set "~w" ', [Name]),
-	flush,
+	flush_output,
 	functor(Head, Name, 1),
 	nth_clause(Head, _N, R),
 	clause(Head, _, R),
 	rdf_reset_db,			% reset before each script
 	(   catch(Head, Except, true)
 	->  (   var(Except)
-	    ->  put(.), flush
+	    ->  put(.), flush_output
 	    ;   Except = blocked(Reason)
 	    ->  assert(blocked(Head, Reason)),
-		put(!), flush
+		put(!), flush_output
 	    ;   test_failed(R, Except)
 	    )
 	;   test_failed(R, fail)
@@ -987,3 +999,39 @@ test(lang_matches, true) :-
 	lang_matches('en-GB-x-y', 'en-*-y').
 
 :- end_tests(lang_matches).
+
+:- begin_tests(lit_ranges, [cleanup(rdf_reset_db)]).
+
+letters :-
+	rdf_reset_db,
+	forall(between(0'a, 0'z, X),
+	       (   char_code(C, X),
+		   rdf_assert(a,b,literal(C))
+	       )).
+
+integers :-
+	rdf_reset_db,
+	forall(between(0, 9, X),
+	       rdf_assert(a,b,literal(X))).
+
+ge(S, X) :-
+	rdf(_,b,literal(ge(S),X)).
+le(S, X) :-
+	rdf(_,b,literal(le(S),X)).
+bt(L,H,X) :-
+	rdf(_,b,literal(between(L,H),X)).
+
+test(ge, [setup(letters), cleanup(rdf_reset_db), all(X==[x,y,z])]) :-
+	ge(x, X).
+test(le, [setup(letters), cleanup(rdf_reset_db), all(X==[a,b,c,d,e])]) :-
+	le(e, X).
+test(bt, [setup(letters), cleanup(rdf_reset_db), all(X==[m,n,o,p])]) :-
+	bt(m, p, X).
+test(ge, [setup(integers), cleanup(rdf_reset_db), all(X==[4,5,6,7,8,9])]) :-
+	ge(4, X).
+test(le, [setup(integers), cleanup(rdf_reset_db), all(X==[0,1,2,3])]) :-
+	le(3, X).
+test(bt, [setup(integers), cleanup(rdf_reset_db), all(X==[6,7,8])]) :-
+	bt(6,8, X).
+
+:- end_tests(lit_ranges).
