@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #define _ISOC99_SOURCE 1		/* fwprintf(), etc prototypes */
@@ -732,13 +732,15 @@ call_prolog(parser_data *pd, predicate_t pred, term_t av)
 { qid_t qid = PL_open_query(NULL, PL_Q_PASS_EXCEPTION, pred, av);
   int rc = PL_next_solution(qid);
 
-  if ( !rc && PL_exception(qid) )
-    pd->exception = TRUE;
-  else
-    pd->exception = FALSE;
-
   PL_close_query(qid);
 
+  if ( rc )
+  { pd->exception = FALSE;
+  } else
+  { if ( (pd->exception = PL_exception(0)) )
+      pd->stopped = TRUE;
+  }
+
   return rc;
 }
 
@@ -1886,6 +1888,8 @@ pl_sgml_parse(term_t parser, term_t options)
     else
       p->encoded = FALSE;		/* already decoded */
 
+    pd->stopped = FALSE;
+
     if ( !recursive )
     { pd->source = in;
       begin_document_dtd_parser(p);
@@ -2604,3 +2608,11 @@ install()
 #endif
 }
 
+
+install_t
+uninstall()
+{ 
+#ifdef _REENTRANT
+  stop_ring();
+#endif
+}
